@@ -14,7 +14,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: install.py 439 2005-03-02 14:19:56Z rspivak $
+# $Id: install.py 33409 2006-02-23 20:56:03Z atchertchian $
 
 """
 CPSBlog Installer
@@ -126,91 +126,45 @@ class ClientInstaller(CPSInstaller):
         self.allowContentTypes(('Blog', 'BlogAggregator'),
                                ('Workspace', 'Section'))
 
-        self.setupBoxes()
+        # portlet cache parameters
+        ptltool = self.getTool('portal_cpsportlets')
+        ptltool.updateCacheParameters({
+            'BlogAggregator Portlet': ['no-cache'],
+            })
+
 
         ########################################
         # WORKFLOW DEFINITION
         ########################################
 
         wfscripts = {
-            'add_blog_boxes': {
+                'add_blog_portlets': {
             '_owner': None,
             'script': """
 ##parameters=state_change
 blog_proxy = state_change.object
 blog_rel_url = context.portal_url.getRelativeContentURL(blog_proxy)
-kw = {'type_name' : 'Blog Calendar Box',
-      'slot_name' : 'right',
-      'title' : blog_proxy.Title(),
-      'events_in' : blog_rel_url,
-      'event_types' : ('BlogEntry', ),
-      'box_skin': 'here/box_lib/macros/wbox2'
-      }
-blog_proxy.box_create(**kw)
 
-# Search box
-kw = {'type_name' : 'Base Box',
-      'slot_name' : 'right',
-      'title' : 'Search',
-      'provider' : 'cpsblog',
-      'btype' : 'blogsearch',
-      'box_skin': 'here/box_lib/macros/sbox'
-      }
-blog_proxy.box_create(**kw)
+ptool = context.portal_cpsportlets
 
-# Archives box
-kw = {'type_name' : 'Base Box',
-      'slot_name' : 'right',
-      'title' : 'Archives',
-      'provider' : 'cpsblog',
-      'btype' : 'blogarchive',
-      'box_skin': 'here/box_lib/macros/sbox'
-      }
-blog_proxy.box_create(**kw)
+# Calendar portlet
 
-# Categories box
-kw = {'type_name' : 'Base Box',
-      'slot_name' : 'right',
-      'title' : 'Categories',
-      'provider' : 'cpsblog',
-      'btype' : 'blogcategories',
-      'box_skin': 'here/box_lib/macros/sbox'
+kw = {'slot': 'right',
+      'order': 0,
+      'Title': blog_proxy.Title(),
+      'render_method': 'blogcalendar_portlet',
       }
-blog_proxy.box_create(**kw)
-            """
-            },
-            }
+ptool.createPortlet(ptype_id='Custom Portlet', context=blog_proxy, **kw)
 
-        # If True, redefine workflow script
-        if CPSPORTLETS:
-            wfscripts = {
-                'add_blog_boxes': {
-            '_owner': None,
-            'script': """
-##parameters=state_change
-blog_proxy = state_change.object
-blog_rel_url = context.portal_url.getRelativeContentURL(blog_proxy)
-kw = {'type_name' : 'Blog Calendar Box',
-      'slot_name' : 'right',
-      'title' : blog_proxy.Title(),
-      'events_in' : blog_rel_url,
-      'event_types' : ('BlogEntry', ),
-      'box_skin': 'here/box_lib/macros/wbox2'
-      }
-blog_proxy.box_create(**kw)
+# Search portlet
 
-# Search box
-kw = {'type_name' : 'Base Box',
-      'slot_name' : 'right',
-      'title' : 'Search',
-      'provider' : 'cpsblog',
-      'btype' : 'blogsearch',
-      'box_skin': 'here/box_lib/macros/sbox'
+kw = {'slot': 'right',
+      'order': 0,
+      'Title': 'Search',
       }
-blog_proxy.box_create(**kw)
+ptool.createPortlet(ptype_id='Search Portlet', context=blog_proxy, **kw)
 
 # Archives portlet
-ptool = context.portal_cpsportlets
 
 kw = {'slot' : 'right',
       'order': 0,
@@ -220,7 +174,6 @@ kw = {'slot' : 'right',
 ptool.createPortlet(ptype_id='Custom Portlet', context=blog_proxy, **kw)
 
 # Categories portlet
-ptool = context.portal_cpsportlets
 
 kw = {'slot' : 'right',
       'order': 0,
@@ -274,7 +227,7 @@ ptool.createPortlet(ptype_id='Custom Portlet', context=blog_proxy, **kw)
                 'transition_behavior': (TRANSITION_INITIAL_CREATE,),
                 'clone_allowed_transitions': None,
                 'actbox_category': 'workflow',
-                'after_script_name' : 'add_blog_boxes',
+                'after_script_name' : 'add_blog_portlets',
                 'props': {'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; '
                                         'WorkspaceMember',
@@ -336,7 +289,7 @@ ptool.createPortlet(ptype_id='Custom Portlet', context=blog_proxy, **kw)
                 'transition_behavior': (TRANSITION_INITIAL_CREATE,),
                 'clone_allowed_transitions': None,
                 'actbox_category': 'workflow',
-                'after_script_name' : 'add_blog_boxes',
+                'after_script_name' : 'add_blog_portlets',
                 'props': {'guard_permissions': '',
                           'guard_roles': 'Manager; SectionManager',
                           'guard_expr': ''},
@@ -382,8 +335,7 @@ blog_entry_proxy.reindexObject(idxs=['effective', 'start', 'end'])
                                             'WorkspaceManager',
                                             'SectionManager',
                                             'SectionReviewer',
-                                            'BlogManager',
-                                            'BlogPoster'),
+                                            'BlogManager'),
                                      ModifyPortalContent: ('Manager',
                                                            'Owner',
                                                            'BlogManager',
@@ -398,6 +350,7 @@ blog_entry_proxy.reindexObject(idxs=['effective', 'start', 'end'])
                                                  'SectionReader',
                                                  'WorkspaceManager',
                                                  'WorkspaceMember',
+                                                 'WorkspaceReader',
                                                  'BlogManager',
                                                  'BlogPoster')}
                           },
@@ -427,8 +380,8 @@ blog_entry_proxy.reindexObject(idxs=['effective', 'start', 'end'])
                 'props': {'guard_permissions':'',
                           'guard_roles':'Manager; SectionManager; '
                                         'SectionReviewer; '
-                                        'BlogManager; BlogPoster; '
-                                        'WorkspaceManager; WorkspaceMember; ',
+                                        'BlogManager; '
+                                        'WorkspaceManager; Owner; ',
                           'guard_expr':''},
                 },
             'unpublish': {
@@ -443,8 +396,8 @@ blog_entry_proxy.reindexObject(idxs=['effective', 'start', 'end'])
                 'props': {'guard_permissions':'',
                           'guard_roles':'Manager; SectionManager; '
                                         'SectionReviewer; '
-                                        'BlogManager; BlogPoster; '
-                                        'WorkspaceManager; WorkspaceMember',
+                                        'BlogManager; '
+                                        'WorkspaceManager; Owner',
                           'guard_expr':''},
                 },
             }
@@ -500,37 +453,6 @@ blog_entry_proxy.reindexObject(idxs=['effective', 'start', 'end'])
             pass
         self.log("### End of CPSSubscriptions setup ###")
 
-    def setupBoxes(self):
-        from Products.CPSBlog.BlogCalendarBox import factory_type_information
-        blogcalendarbox_fti = factory_type_information[0]
-
-        types = (
-            {'id' : blogcalendarbox_fti['id'],
-             'meta_type' : blogcalendarbox_fti['meta_type'],
-             'title' : blogcalendarbox_fti['title'],
-             'description' : blogcalendarbox_fti['description']
-             },
-            )
-
-        ttool = self.portal.portal_types
-        ptypes = ttool.objectIds()
-
-        for boxtype in types:
-            boxid = boxtype['id']
-            if boxid in ptypes:
-                self.log('Deleted Box ' + boxid)
-                ttool.manage_delObjects(boxid)
-            self.log('Adding Box ' + boxid)
-            ttool.manage_addTypeInformation(
-                id=boxid,
-                add_meta_type='Factory-based Type Information',
-                typeinfo_name='CPSBlog: ' + boxid,
-                )
-            ttool.manage_changeProperties(
-                title=boxtype['title'],
-                description=boxtype['description'],
-                content_meta_type=boxtype['meta_type'],
-                )
 
     def setupActions(self):
         from Products.CPSSubscriptions.CPSSubscriptionsPermissions import \
