@@ -1,9 +1,12 @@
 ##parameters=trackbacks_ids,REQUEST=None
-# $Id: blog_entry_remove_trackbacks.py 372 2005-02-04 15:52:01Z rspivak $
+# $Id: blog_entry_remove_trackbacks.py 25801 2005-08-13 21:39:17Z rspivak $
 """Removes trackbacks from blog entry. This script is called from edit
 layout for blog entry, so 'context' is repository blog entry object,
 not proxy."""
 
+if getattr(context, 'removeTrackbacks', None) is None:
+    context = context.getContent()
+
 context.removeTrackbacks(trackbacks_ids)
 
 REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERER'])
