@@ -1,5 +1,5 @@
 ##parameters=
-# $Id: getBlogArchiveDates.py 438 2005-03-02 14:02:32Z rspivak $
+# $Id: getBlogArchiveDates.py 25837 2005-08-15 10:56:19Z rspivak $
 """Returns list of dictionaries containing start and end date strings
 in form of '%Y/%m/%d' for constructing blog archive search urls."""
 
@@ -10,11 +10,14 @@ brains = catalog.searchResults(meta_type='BlogEntry',
                                sort_order='reverse',
                                path='/'.join(context.getPhysicalPath()))
 
+proxies = [brain.getObject()
+           for brain in brains if brain.getObject() is not None]
+
 # { year : [list of months integers] }
 dyear = {}
 
-for brain in brains:
-    date = brain.getObject().effective()
+for proxy in proxies:
+    date = proxy.effective()
     year = date.year()
     month = date.month()
     if year in dyear:
