@@ -1,12 +1,15 @@
 ##parameters=proxy=None
-# $Id: getBlogProxy.py 216 2005-01-12 00:55:05Z rspivak $
+# $Id: getBlogProxy.py 25813 2005-08-14 13:07:23Z ebarroca $
 """Iterates over tree and returns first blog proxy it encounters."""
 
 if proxy is None:
     proxy = context
 
 def isBlogType(proxy):
-    return proxy.portal_type == 'Blog'
+    try:
+        return proxy.portal_type == 'Blog'
+    except AttributeError:
+        return False
 
 # check if current context is of Blog type
 if isBlogType(proxy):
