@@ -1,10 +1,10 @@
 ##parameters=key=None, is_i18n=None
-# $Id: getVocBlogLanguages.py 301 2005-01-25 17:29:40Z rspivak $
+# $Id: getVocBlogLanguages.py 33411 2006-02-23 21:02:51Z atchertchian $
 """Returns blog languages for method vocabulary."""
 
 blog = context.getBlogContent()
 lang_voc = context.portal_vocabularies.language_voc
-mcat = context.Localizer.default
+mcat = context.translation_service
 
 t = [(mcat(lang_voc.getMsgid(lang)).lower(), lang) for lang in blog.langs]
 t.sort()
