@@ -1,4 +1,4 @@
-# $Id: testGlobalInstall.py 147 2004-12-26 14:33:40Z sfermigier $
+# $Id: testGlobalInstall.py 33409 2006-02-23 20:56:03Z atchertchian $
 
 import os, sys
 if __name__ == '__main__':
@@ -20,16 +20,13 @@ class TestGlobalInstall(CPSBlogTestCase.CPSBlogTestCase):
     def beforeTearDown(self):
         self.logout()
 
-    def testInstallerScript(self):
-        self.assert_('cpsblog_installer' in self.portal.objectIds(),
-                     'Installer script was not correctly added to portal')
-
-    def testCPSBlogPortalTypes(self):
+    def testPortalTypes(self):
         ttool = self.portal.portal_types
-        self.assert_('Blog' in ttool.objectIds(),
-                     'Blog type is not in portal_types')
-        self.assert_('BlogEntry' in ttool.objectIds(),
-                     'BlogEntry type is not in portal_types')
+        portal_types = ['Blog', 'BlogEntry', 'BlogAggregator',
+                        'BlogAggregator Portlet']
+        for portal_type in portal_types:
+            self.assert_(portal_type in ttool.objectIds(),
+                         '%s is not in portal_types' % portal_type)
 
 
 def test_suite():
