@@ -1,4 +1,5 @@
-# (C) Copyright 2004 Nuxeo SARL <http://nuxeo.com>
+# (C) Copyright 2004-2005 Nuxeo SARL <http://nuxeo.com>
+# Author: Ruslan Spivak <rspivak@nuxeo.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as published
@@ -14,10 +15,10 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: test_batch.py 134 2004-12-25 22:29:28Z rspivak $
+# $Id: test_batch.py 25014 2005-07-19 10:56:39Z rspivak $
 
 import unittest
-from Testing.ZopeTestCase.doctest import DocTestSuite, DocFileSuite
+from Testing.ZopeTestCase.zopedoctest import DocTestSuite, DocFileSuite
 from Products.CPSBlog.CPSBatch import Batch
 # Freebie Batch tests from ZTUtils
 from ZTUtils.tests.testBatch import BatchTests
