@@ -22,9 +22,14 @@
 #
 # Replace MonkeyPatch of Membershiptool by real object use
 #
-# $Id: CPSMembershipTool.py 21827 2005-05-25 16:20:01Z atchertchian $
+# $Id: CPSMembershipTool.py 33523 2006-02-27 16:17:45Z fguillaume $
 
 import sys
+
+# python sets from 2.3 onward
+try: set
+except NameError: from sets import Set as set
+
 from types import StringType
 from Globals import InitializeClass, DTMLFile
 from AccessControl import ClassSecurityInfo
@@ -37,6 +42,9 @@ from AccessControl.User import UnrestrictedUser
 from Acquisition import aq_base, aq_parent, aq_inner
 from ZODB.POSException import ConflictError
 
+from zope.interface import implements
+from Products.CMFCore.interfaces import IMembershipTool
+
 from Products.CMFCore.permissions import View, ManagePortal
 from Products.CMFCore.permissions import ListPortalMembers
 from AccessControl.Permissions import manage_users as ManageUsers
@@ -64,46 +72,30 @@ class CPSUnrestrictedUser(UnrestrictedUser):
 class CPSMembershipTool(MembershipTool):
     """ Replace MonkeyPatch of Membershiptool by real object use."""
 
-    #
-    # Actions for CPS
-    #
-    _actions = [
-      AI(id='login',
-         title='Login',
-         description='Click here to Login',
-         action=Expression(text='string:${portal_url}/login_form'),
-         permissions=(View,),
-         category='user',
-         condition=Expression(text='not: member'),
-         visible=1),
-      AI(id='logout',
-         title='Log out',
-         description='Click here to logout',
-         action=Expression(text='string:${portal_url}/logout'),
-         permissions=(View,),
-         category='user',
-         condition=Expression(text='member'),
-         visible=1),
-      AI(id='mystuff',
-         title='My stuff',
-         description='Goto your home folder',
-         action=Expression(text='string:${portal/portal_membership'
-                           + '/getHomeUrl}/folder_contents'),
-         permissions=(View,),
-         category='user',
-         condition=Expression(text='python: member and '
-                              + 'portal.portal_membership.getHomeFolder()'),
-         visible=1)
-    ]
+    implements(IMembershipTool)
+
+    _actions = ()
 
     meta_type = 'CPS Membership Tool'
 
-    # XXX AT: Since there is now a membership tool in CPSDefault, these
-    # variables could be initialized with more 'standard' values here, and
-    # overloaded with CPS specific values in CPSDefault/MembershipTool.py
-    membersfolder_id = 'workspaces/members' # Members
-    memberfolder_portal_type = 'Workspace' # Folder
-    memberfolder_roles = ('Owner', 'WorkspaceManager') # Owner
+    _properties = (
+        {'id': 'membersfolder_id', 'type': 'string', 'mode': 'w',
+         'label': 'Members folder relative path'},
+        {'id': 'memberareaCreationFlag', 'type': 'boolean', 'mode': 'w',
+         'label': 'Member folder created at login'},
+        {'id': 'memberfolder_portal_type', 'type': 'string', 'mode': 'w',
+         'label': 'Member folder portal type'},
+        {'id': 'memberfolder_roles', 'type': 'tokens', 'mode': 'w',
+         'label': 'Member folder owner local roles'},
+        {'id': 'pending_members', 'type': 'lines', 'mode': 'w',
+         'label': 'Recently deleted members'},
+        )
+    memberareaCreationFlag = True
+    membersfolder_id = 'Members'
+    memberfolder_portal_type = 'Folder'
+    memberfolder_roles = ('Owner',)
+    pending_members = ()
+
 
     # XXX this is slightly better than what was done before (hardcoded roles
     # in each method) as this one centralises the allowed roles declaration
@@ -114,6 +106,21 @@ class CPSMembershipTool(MembershipTool):
 
     security = ClassSecurityInfo()
 
+    security.declarePublic('assertViewable')
+    def assertViewable(self, context):
+        """Raise Unauthorized if member doesn't have the View permission."""
+
+        if not _checkPermission(View, context):
+            raise Unauthorized
+
+    security.declarePublic('canMemberChangeLocalRoles')
+    def canMemberChangeLocalRoles(self, context):
+        """Can the authenticated member change the local roles
+        """
+        user = getSecurityManager().getUser()
+        return (user.has_role('Manager') or
+                user.has_role(self.roles_managing_local_roles, context))
+
     security.declareProtected(View, 'getMergedLocalRoles')
     def getMergedLocalRoles(self, object, withgroups=1):
         """Return aquisition roles"""
@@ -124,7 +131,7 @@ class CPSMembershipTool(MembershipTool):
         """Return aquisition roles with path"""
         return mergedLocalRolesWithPath(object, withgroups)
 
-    security.declareProtected(View, 'getCandidateLocalRoles')
+    security.declareProtected(View, 'getCPSCandidateLocalRoles')
     def getCPSCandidateLocalRoles(self, obj):
         """What local roles according to the context ?"""
 
@@ -371,11 +378,6 @@ class CPSMembershipTool(MembershipTool):
                                              list(self.memberfolder_roles))
             member_area.reindexObjectSecurity()
 
-            # Rebuild the tree with corrected local roles.
-            # This needs a user that can View the object.
-            portal_eventservice = getToolByName(self, 'portal_eventservice')
-            portal_eventservice.notify('sys_modify_security', member_area, {})
-
             self._createMemberContentAsManager(member, member_id, member_area)
 
         finally:
@@ -419,10 +421,19 @@ class CPSMembershipTool(MembershipTool):
 
     security.declarePublic('isHomeless')
     def isHomeless(self, member=None):
-        """Return 1 if member have no home using homeless attribute."""
+        """Return 1 if member have no home using homeless attribute.
+
+        Anonymous users and users registered above CPS level are homeless.
+        """
         ret = 0
         if member is None:
             member = self.getAuthenticatedMember()
+        if member.getUserName() == "Anonymous User":
+            return 1
+        # users registered above the portal level are homeless
+        member_acl_users = aq_parent(aq_inner(member.getUser()))
+        if aq_base(member_acl_users) is not aq_base(self.acl_users):
+            return 1
 
         if hasattr(member, 'getProperty'):
             ret = member.getProperty('homeless', 0)
@@ -454,18 +465,18 @@ class CPSMembershipTool(MembershipTool):
             try:
                 member_area_id = self.getHomeFolderId(id)
                 folder = members._getOb(member_area_id)
-                if verifyPermission and not _checkPermission('View', folder):
+                if verifyPermission and not _checkPermission(View, folder):
                     # Don't return the folder if the user can't get to it.
                     return None
                 return folder
-            except AttributeError:
+            except (AttributeError, TypeError, KeyError):
                 pass
         return None
 
     security.declarePublic('getHomeFolderId')
     def getHomeFolderId(self, id, max_chars_for_id=128):
         """Compute an home folder id for the given member id."""
-        id = generateId(id, max_chars_for_id)
+        id = generateId(id, max_chars_for_id, False)
         return id
 
     security.declarePublic('homeFolderExists')
@@ -477,7 +488,7 @@ class CPSMembershipTool(MembershipTool):
                 member_area_id = self.getHomeFolderId(id)
                 folder = members._getOb(member_area_id)
                 return 1
-            except AttributeError:
+            except (AttributeError, KeyError):
                 return 0
 
     security.declarePrivate('listActions')
@@ -534,7 +545,9 @@ class CPSMembershipTool(MembershipTool):
                       delete_localroles=0, check_permission=1):
         """Delete members specified by member_ids.
 
-        XXX does not call local roles deletion by default.
+        XXX does not call local roles deletion by default but stores their
+        ids in the pending_members property. Use purgeDeletedMembersLocalRoles()
+        to delete them
         """
 
         # Delete members in acl_users.
@@ -576,9 +589,40 @@ class CPSMembershipTool(MembershipTool):
             utool = getToolByName(self, 'portal_url')
             self.deleteLocalRoles(utool.getPortalObject(), member_ids,
                                   reindex=1, recursive=1)
+        # Or stores their ids in the pending_members property for a
+        # delayed purge
+        else:
+            already_pending =  set(self.getProperty('pending_members'))
+            self.pending_members = tuple(already_pending.union(member_ids))
 
         return tuple(member_ids)
 
+    security.declareProtected(ManageUsers, 'purgeDeletedMembersLocalRoles')
+    def purgeDeletedMembersLocalRoles(self):
+        """Trigger the purge of localroles of member_ids stored in
+        pending_members
+
+        return the list of ids that where cleaned
+
+        WARNING: This can take a while (security reindexing)
+        """
+        # Remove duplicates
+        pending_members = set(self.getProperty('pending_members'))
+
+        # Filter out the member_ids that correspond to actual members
+        member_ids = self.listMemberIds()
+        pending_members = [id for id in pending_members if id not in member_ids]
+
+        # Purge
+        utool = getToolByName(self, 'portal_url')
+        self.deleteLocalRoles(utool.getPortalObject(), pending_members,
+                reindex=1, recursive=1)
+
+        # Updating the pending_members property
+        self.pending_members = ()
+
+        return pending_members
+
     #
     #   ZMI interface methods
     #
@@ -605,6 +649,22 @@ class CPSMembershipTool(MembershipTool):
                                       '/manage_mapRoles' +
                                       '?manage_tabs_message=%s' % message)
 
+    security.declareProtected(ManagePortal, 'manage_purgeLocalRoles')
+    def manage_purgeLocalRoles(self, REQUEST=None):
+        """Call 'purgeDeletedMembersLocalRoles' to clean localroles for ids
+        stored in the pending_members property
+        """
+        ids = self.purgeDeletedMembersLocalRoles()
+        if ids:
+            message = "Ids %s cleaned." % ids
+        else:
+            message = "No member id to clean"
+
+        if REQUEST is not None:
+            REQUEST.RESPONSE.redirect(self.absolute_url() +
+                                      '/manage_mapRoles' +
+                                      '?manage_tabs_message=%s' % message)
+
 
 InitializeClass(CPSMembershipTool)
 
