@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: ObjectRepositoryTool.py 3295 2005-04-11 13:07:45Z janguenot $
+# $Id: ObjectRepositoryTool.py 32312 2006-01-27 10:59:08Z atchertchian $
 """Object Repository Tool.
 
 The object repository tool stores versions of documents.
@@ -26,7 +26,7 @@ from zLOG import LOG, ERROR, DEBUG, TRACE
 import random
 from Globals import InitializeClass, DTMLFile
 from cStringIO import StringIO
-from Acquisition import aq_base, aq_parent, aq_inner
+from Acquisition import aq_base, aq_parent, aq_inner, aq_get
 from AccessControl import ClassSecurityInfo
 from AccessControl.Permission import Permission, name_trans
 
@@ -58,38 +58,6 @@ class NoWorkflowConfiguration:
 InitializeClass(NoWorkflowConfiguration)
 
 
-def setLocalRolesWithGroups(ob, lroles):
-    """Set all the local roles and group local roles.
-
-    Returns True if something was changed.
-    """
-    # XXX move this to NuxUserGroups
-    #LOG('obrep', DEBUG, 'setlocal: ob=%s lroles=%s'
-    #    % ('/'.join(ob.getPhysicalPath()), lroles,))
-    udict = {}
-    gdict = {}
-    for k, roles in lroles.items():
-        if k.startswith('user:'):
-            uid = k[5:]
-            udict[uid] = list(roles)
-        elif k.startswith('group:'):
-            gid = k[6:]
-            gdict[gid] = list(roles)
-    changed = 0
-    uroles = getattr(ob, '__ac_local_roles__', None) or {}
-    groles = getattr(ob, '__ac_local_group_roles__', None) or {}
-    if uroles != udict:
-        #LOG('obrep', DEBUG, ' set udict=%s' % (udict,))
-        ob.__ac_local_roles__ = udict
-        changed = 1
-    if groles != gdict:
-        #LOG('obrep', DEBUG, ' set gdict=%s' % (gdict,))
-        ob.__ac_local_group_roles__ = gdict
-        changed = 1
-    #LOG('obrep', DEBUG, ' changed=%s' % changed)
-    return changed
-
-
 class ObjectRepositoryTool(UniqueObject,
                            BTreeFolder2, PortalFolder,
                            TypeConstructor, TypeContainer):
@@ -113,6 +81,7 @@ class ObjectRepositoryTool(UniqueObject,
     portal_type = meta_type
 
     security = ClassSecurityInfo()
+    security.declareObjectProtected(ManagePortal)
 
     def __init__(self):
         BTreeFolder2.__init__(self, self.id)
@@ -163,6 +132,7 @@ class ObjectRepositoryTool(UniqueObject,
         id = self._getId(docid_, rev)
         self.constructContent(type_name_, id, *args, **kw)
         ob = self.get(id)
+        self._markObjectInRepository(ob)
         evtool = getEventService(self)
         evtool.notify('sys_add_cmf_object', ob, {})
         # XXX or call ob.manage_afterCMFAdd(ob, self) ? recurse ?
@@ -255,11 +225,37 @@ class ObjectRepositoryTool(UniqueObject,
         else:
             return (None, None)
 
-    security.declarePublic('isObjectInRepository')
+    security.declarePrivate('_markObjectInRepository')
+    def _markObjectInRepository(self, ob):
+        """Mark an object as being in the repository."""
+        ob._isInCPSRepository = True
+
+    security.declarePrivate('isObjectInRepository')
     def isObjectInRepository(self, ob):
         """Test if an object is in the repository."""
+        # We have to use an attribute on the object, as the repository
+        # objects are always rewrapped under the acquisition context of
+        # the proxy.
+        if getattr(ob, '_isInCPSRepository', False):
+            return True
+        # During creation, in manage_afterAdd, the attribute hasn't been
+        # set yet, which is why we have to do the explicit path check.
         return ob.getPhysicalPath()[:-1] == self.getPhysicalPath()
 
+
+    security.declarePrivate('isObjectUnderRepository')
+    def isObjectUnderRepository(self, ob):
+        """Test if an object is under the repository.
+
+        Returns true for all objects contained somewhere under the
+        repository.
+        """
+        if aq_get(ob, '_isInCPSRepository', False, 1):
+            return True
+        # Do the path check as above.
+        repo_path = self.getPhysicalPath()
+        return ob.getPhysicalPath()[:len(repo_path)] == repo_path
+
     security.declarePrivate('freezeRevision')
     def freezeRevision(self, docid, rev):
         """Freeze a version of a document.
@@ -322,76 +318,6 @@ class ObjectRepositoryTool(UniqueObject,
             return ob, rev
         return self.copyRevision(docid, rev)
 
-    security.declarePrivate('getPermissionRole')
-    def getPermissionRole(self, perm):
-        """Get a special role mapping only to a permission.
-
-        Register it if it doesn't exist yet.
-        """
-        role = 'permission:' + perm.translate(name_trans)
-        if role not in self.__ac_roles__:
-            # Register the role and add its permission.
-            self._addRole(role)
-            for inhp in self.ac_inherited_permissions(1):
-                name, data = inhp[:2]
-                if name == perm:
-                    p = Permission(name, data, self)
-                    # Set permission to unacquire, and to Manager.
-                    p.setRoles(('Manager', role))
-                    break
-        return role
-
-    security.declarePrivate('_registerPermissionRolesFor')
-    def _registerPermissionRolesFor(self, ob):
-        """Register all special permission roles mentionned in ob."""
-        done = {}
-        lroles = getattr(ob, '__ac_local_roles__', None) or {}
-        lroles.update(getattr(ob, '__ac_local_group_roles__', None) or {})
-        for k, roles in lroles.items():
-            for role in roles:
-                if done.has_key(role):
-                    continue
-                done[role] = None
-                if role.startswith('permission:'):
-                    tperm = role[len('permission:'):]
-                    # Find corresponding untranslated permission name.
-                    perm = None
-                    for inhp in self.ac_inherited_permissions(1):
-                        name = inhp[0]
-                        if name.translate(name_trans) == tperm:
-                            perm = name
-                            break
-                    if perm is None:
-                        LOG('ObjectRepositoryTool', ERROR,
-                            '_registerPermissionRolesFor perm %s' % tperm)
-                        continue
-                    # Register if needed.
-                    self.getPermissionRole(perm)
-
-    security.declarePrivate('setRevisionSecurity')
-    def setRevisionSecurity(self, docid, rev, userperms):
-        """Set the security on an object.
-
-        userperms is a dict of {user: [sequence of permissions]}
-        user is user:uid or group:gid
-
-        Returns True if security was changed.
-
-        (Called by ProxyTool.)
-        """
-        LOG('ObjectRepositoryTool', TRACE,
-            'setRevisionSecurity docid=%s rev=%s perms=%s' %
-            (docid, rev, userperms))
-        ob = self.getObjectRevision(docid, rev)
-        lroles = {}
-        for user, perms in userperms.items():
-            roles = [self.getPermissionRole(perm) for perm in perms]
-            lroles[user] = roles
-        changed = setLocalRolesWithGroups(ob, lroles)
-        ##if changed:
-        ##    ob.reindexObjectSecurity()
-        return changed
-
     #
     # Staging
     #
@@ -415,8 +341,6 @@ class ObjectRepositoryTool(UniqueObject,
             where = aq_parent(aq_inner(where))
             connection = where._p_jar
         ob = connection.importFile(f)
-        # Register special permissions roles.
-        self._registerPermissionRolesFor(ob)
         # Set the object (this will recatalog).
         id = ob.getId()
         if self.hasObject(id):
