@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: ProxyBase.py 21936 2005-05-30 15:06:48Z atchertchian $
+# $Id: ProxyBase.py 30837 2005-12-20 15:30:52Z fguillaume $
 
 import os.path
 from zLOG import LOG, ERROR, DEBUG, TRACE
@@ -31,6 +31,7 @@ from ComputedAttribute import ComputedAttribute
 from Globals import InitializeClass, DTMLFile
 from AccessControl import ClassSecurityInfo
 from AccessControl import Unauthorized
+from AccessControl.PermissionRole import rolesForPermissionOn
 import Acquisition
 from Acquisition import aq_base, aq_parent, aq_inner
 from OFS.SimpleItem import Item
@@ -38,10 +39,15 @@ from OFS.Image import File
 from OFS.Traversable import Traversable
 from webdav.WriteLockInterface import WriteLockInterface
 
+import zope.interface
+from Products.CPSCore.interfaces import ICPSProxy
+
 from Products.CMFCore.utils import getToolByName
 from Products.CMFCore.permissions import View
 from Products.CMFCore.permissions import ModifyPortalContent
 from Products.CMFCore.permissions import ViewManagementScreens
+from Products.CPSCore.permissions import ViewArchivedRevisions
+from Products.CPSCore.permissions import ChangeSubobjectsOrder
 from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
 
 from Products.CPSUtil.timeoutcache import TimeoutCache, getCache
@@ -58,7 +64,6 @@ from Products.CPSCore.CPSBase import CPSBaseBTreeFolder
 from Products.CPSCore.IndexationManager import get_indexation_manager
 
 
-DOWNLOAD_AS_ATTACHMENT_FILES_SUFFIXES = ('.sxw', '.sxc')
 PROBLEMATIC_FILES_SUFFIXES = ('.exe', '.sxw', '.sxc')
 CACHE_ZIP_VIEW_KEY = 'CPS_ZIP_VIEW'
 CACHE_ZIP_VIEW_TIMEOUT = 7200           # time to cache in second
@@ -77,6 +82,8 @@ class ProxyBase(Base):
       the mapping language -> revision. It is used to provide branch
       lineage between proxies. """
 
+    zope.interface.implements(ICPSProxy)
+
     security = ClassSecurityInfo()
     use_mcat = 0
 
@@ -113,10 +120,8 @@ class ProxyBase(Base):
 
     security.declarePrivate('setDefaultLanguage')
     def setDefaultLanguage(self, default_language):
-        """Set the default language for this proxy.
-
-        (Called by ProxyTool.)
-        """
+        """Set the default language for this proxy."""
+        # Called by CPSWorkflow after creating an empty proxy.
         self._default_language = default_language
 
     # XXX was def getVersionInfos(self):
@@ -205,9 +210,21 @@ class ProxyBase(Base):
 
     security.declarePrivate('_getContent')
     def _getContent(self, lang=None, rev=None, editable=0):
-        """Get the content object, maybe editable."""
+        """Get the content object, maybe editable.
+
+        Returns an object acquisition-wrapped under the proxy itself,
+        so that security is correctly inferred.
+        """
         pxtool = getToolByName(self, 'portal_proxies')
-        return pxtool.getContent(self, lang=lang, rev=rev, editable=editable)
+        ob = pxtool.getContent(self, lang=lang, rev=rev, editable=editable)
+        # Rewrapping plays havoc with absolute_url, but that's not a
+        # problem as nobody should ever get the absolute_url of an
+        # object in the repository. In the previous implementation
+        # anyway you got URLs containing portal_repository/ which is bad
+        # too.
+        if ob is None:
+            return None
+        return aq_base(ob).__of__(self)
 
     security.declarePrivate('proxyChanged')
     def proxyChanged(self):
@@ -217,7 +234,6 @@ class ProxyBase(Base):
         utool = getToolByName(self, 'portal_url')
         rpath = utool.getRelativeUrl(self)
         pxtool._modifyProxy(self, rpath) # XXX or directly event ?
-        pxtool.setSecurity(self)
         evtool = getEventService(self)
         evtool.notify('sys_modify_object', self, {})
 
@@ -225,6 +241,7 @@ class ProxyBase(Base):
         """Transparent traversal of the proxy to the real subobjects.
 
         Used for skins that don't take proxies enough into account.
+        Returns an object wrapped in the acquisition context of the proxy.
 
         Parses URL revision switch of the form:
           mydoc/archivedRevision/n/...
@@ -266,17 +283,14 @@ class ProxyBase(Base):
                 res = ob[name]
             except (KeyError, IndexError, TypeError, AttributeError):
                 raise KeyError, name
-        if hasattr(res, '__of__'):
-            # XXX Maybe incorrect if complex wrapping.
-            res = aq_base(res).__of__(self)
         return res
 
     #
     # Freezing
     #
 
-    security.declarePrivate('freezeProxy')
-    def freezeProxy(self):
+    security.declareProtected(ModifyPortalContent, 'freezeProxy')
+    def freezeProxy(self, REQUEST=None):
         """Freeze the proxy.
 
         Freezing means that any attempt at modification will create a new
@@ -284,6 +298,8 @@ class ProxyBase(Base):
 
         (Called by CPSWorkflow.)
         """
+        if REQUEST is not None:
+            raise Unauthorized('Not allowed through the web')
         pxtool = getToolByName(self, 'portal_proxies')
         self._freezeProxy(self, pxtool)
 
@@ -376,34 +392,14 @@ class ProxyBase(Base):
     # Security
     #
 
-    def _setSecurity(self):
-        """Propagate security changes made on the proxy."""
-        pxtool = getToolByName(self, 'portal_proxies')
-        pxtool.setSecurity(self)
-
-    def _setSecurityRecursive(self, ob, pxtool=None):
-        """Propagate security changes made on the proxy."""
-        if not isinstance(ob, ProxyBase):
-            return
-        if pxtool is None:
-            pxtool = getToolByName(self, 'portal_proxies')
-        pxtool.setSecurity(ob)
-        for subob in ob.objectValues():
-            self._setSecurityRecursive(subob, pxtool=pxtool)
-
     def _reindexObject(self, idxs=[]):
         """Called to reindex when the object has changed."""
         LOG('ProxyBase', DEBUG, "reindex idxs=%s for %s"
             % (idxs, '/'.join(self.getPhysicalPath())))
-        if not idxs or 'allowedRolesAndUsers' in idxs:
-            # XXX should use an event for that
-            self._setSecurity()
         if 'allowedRolesAndUsers' in idxs:
             # Both must be updated
             idxs.append('localUsersWithRoles')
-        # Doesn't work if CMFCatalogAware isn't an ExtensionClass:
-        #return CMFCatalogAware.reindexObject(self, idxs=idxs)
-        return CMFCatalogAware.__dict__['reindexObject'](self, idxs=idxs)
+        return CMFCatalogAware.reindexObject(self, idxs=idxs)
 
     # overloaded
     def reindexObject(self, idxs=[]):
@@ -421,27 +417,17 @@ class ProxyBase(Base):
         """Called to security-related indexes."""
         LOG('ProxyBase', DEBUG, "reindex security for %s"
             % '/'.join(self.getPhysicalPath()))
-        # XXX should use an event for that
-        self._setSecurityRecursive(self)
         # Notify that this proxy's security has changed.
         # Listeners will have to recurse if necessary.
         # (The notification for the object repo is done by the repo.)
         evtool = getEventService(self)
         evtool.notify('sys_modify_security', self, {})
-        # Method definition is monkey-patched by PatchCMFCatalogAware
-        # to add the second argument.
-        return CMFCatalogAware.__dict__['reindexObjectSecurity'](self,
-                                                                 skip_self)
+        return CMFCatalogAware.reindexObjectSecurity(self, skip_self)
 
     # overloaded
     def reindexObjectSecurity(self):
         get_indexation_manager().push(self, with_security=True)
 
-    # XXX also call _setSecurity from:
-    #  manage_role
-    #  manage_acquiredPermissions
-    #  manage_changePermissions
-
     #
     # Helpers
     #
@@ -550,6 +536,19 @@ class ProxyBase(Base):
         else:
             return 1
 
+    security.declareProtected(View, 'isCPSFolderish')
+    def isCPSFolderish(self):
+        """Return True if the document is a structural folderish."""
+        if not self.isPrincipiaFolderish:
+            # structural document must be folderish
+            return False
+        ttool = getToolByName(self, 'portal_types')
+        if getattr(ttool[self.portal_type],
+                   'cps_display_as_document_in_listing', False):
+            # this folderish document is not structural
+            return False
+        return True
+
     #
     # Revision management
     #
@@ -684,8 +683,6 @@ class FileDownloader(Acquisition.Explicit):
     def __bobo_traverse__(self, request, name):
         state = self.state
         ob = self.ob
-        LOG('FileDownloader.getitem', DEBUG, "state=%s name=%s"
-            % (state, name))
         if state == 0:
             # First call, swallow attribute
             if not hasattr(aq_base(ob), name):
@@ -927,8 +924,8 @@ class LanguageViewer(Acquisition.Explicit):
 
     # Needed by brain.getObject in Zope >= 2.7.6
     getPhysicalRoot = Acquisition.Acquired
-    unrestrictedTraverse = Traversable.unrestrictedTraverse
-    restrictedTraverse = Traversable.restrictedTraverse
+    unrestrictedTraverse = Traversable.unrestrictedTraverse.im_func
+    restrictedTraverse = Traversable.restrictedTraverse.im_func
 
 InitializeClass(LanguageViewer)
 
@@ -996,7 +993,7 @@ class VirtualProxy(ProxyBase, CPSBaseDocument):
     """
 
     security = ClassSecurityInfo()
-    security.declareObjectPublic()
+    security.declareObjectProtected(ViewArchivedRevisions)
 
     # Never modifiable.
     _Modify_portal_content_Permission = ()
@@ -1022,7 +1019,9 @@ class VirtualProxy(ProxyBase, CPSBaseDocument):
             raise ValueError("Cannot specify lang for a virtual proxy")
         if rev is not None:
             raise ValueError("Cannot specify rev for a virtual proxy")
-        return self._ob
+        # Wrap in the virtual proxy, which is wrapped in the real proxy,
+        # to get proper security context.
+        return aq_base(self._ob).__of__(self)
 
     security.declareProtected(View, 'getLanguage')
     def getLanguage(self, lang=None):
@@ -1250,6 +1249,11 @@ class NotAProxy:
         """Is the proxy archived."""
         return 0
 
+    def isCPSFolderish(self):
+        """Return True if the document is a structural folderish."""
+        return self.isPrincipiaFolderish
+
+
 InitializeClass(NotAProxy)
 
 # Add all methods to Item class.
@@ -1330,6 +1334,8 @@ class ProxyFolder(ProxyBase, CPSBaseFolder):
 
     meta_type = 'CPS Proxy Folder'
 
+    security = ClassSecurityInfo()
+
     def __init__(self, id, **kw):
         CPSBaseFolder.__init__(self, id)
         ProxyBase.__init__(self, **kw)
@@ -1359,6 +1365,22 @@ class ProxyFolder(ProxyBase, CPSBaseFolder):
                       CPSBaseFolder.manage_options[1:]
                       )
 
+    # Changing security declarations of inherited methods from OFS.OrderSupport
+    # to gain finer granularity
+    security.declareProtected(ChangeSubobjectsOrder,
+            'moveObjectsByDelta',
+            'moveObjectsUp',
+            'moveObjectsDown',
+            'moveObjectsToTop',
+            'moveObjectsToBottom',
+            'orderObjects',
+            'moveObjectToPosition',
+            )
+
+    # Trying to remain consistent with Zope default behaviour
+    security.setPermissionDefault(ChangeSubobjectsOrder, ('Manager', 'Owner'))
+
+
 InitializeClass(ProxyFolder)
 
 
