@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: ProxyTool.py 3319 2005-04-18 16:13:25Z bdelbosc $
+# $Id: ProxyTool.py 30942 2005-12-22 20:45:41Z fguillaume $
 """Proxy Tool that manages proxy objects and their links to real documents.
 """
 
@@ -38,9 +38,28 @@ from Products.CMFCore.utils import mergedLocalRoles
 from Products.CMFCore.TypesTool import FactoryTypeInformation
 from Products.CMFCore.TypesTool import ScriptableTypeInformation
 
-from Products.CPSCore.CPSCorePermissions import ViewArchivedRevisions
+from Products.CPSCore.EventServiceTool import getEventService
+from Products.CPSCore.permissions import ViewArchivedRevisions
 from Products.CPSCore.ProxyBase import ProxyBase, SESSION_LANGUAGE_KEY, \
      REQUEST_LANGUAGE_KEY
+from Products.CPSUtil.session import sessionGet
+
+from zope.app.event.interfaces import IObjectModifiedEvent
+from zope.app.container.interfaces import IObjectMovedEvent
+from OFS.interfaces import IObjectWillBeMovedEvent
+from Products.CPSCore.interfaces import ICPSProxy
+
+
+def handleObjectEvent(ob, event):
+    """Notification from the event service.
+
+    This looks up the local utility ProxyTool and calls it.
+    Will be done with a real local utility later.
+    """
+    pxtool = getToolByName(ob, 'portal_proxies', None)
+    if pxtool is None:
+        return
+    pxtool.handleObjectEvent(ob, event)
 
 
 class ProxyTool(UniqueObject, SimpleItemWithProperties):
@@ -65,12 +84,16 @@ class ProxyTool(UniqueObject, SimpleItemWithProperties):
 
     id = 'portal_proxies'
     meta_type = 'CPS Proxies Tool'
-    use_portal_default_lang = 0
+    use_portal_default_lang = False
+    ignore_events = False
 
     _properties = SimpleItemWithProperties._properties + (
         {'id': 'use_portal_default_lang', 'type': 'boolean', 'mode': 'w',
-         'label': "Use translation_service default prior to proxies default "\
-                  "when current lang not found"},)
+         'label': "Use translation_service default prior to proxies default "
+                  "when current lang not found"},
+        {'id': 'ignore_events', 'type': 'boolean', 'mode': 'w',
+         'label': "Ignore events (DO NOT USE)"},
+        )
 
     security = ClassSecurityInfo()
 
@@ -262,11 +285,10 @@ class ProxyTool(UniqueObject, SimpleItemWithProperties):
                     else:
                         switcher = getattr(REQUEST, REQUEST_LANGUAGE_KEY, None)
                     # 1.2/ check SESSION
-                    if switcher is None and REQUEST.has_key('SESSION'):
+                    if switcher is None:
                         check += ' SESSION'
-                        SESSION = REQUEST.SESSION
-                        if SESSION.has_key(SESSION_LANGUAGE_KEY):
-                            switcher = SESSION[SESSION_LANGUAGE_KEY]
+                        switcher = sessionGet(REQUEST, SESSION_LANGUAGE_KEY,
+                                              None)
                     if switcher is not None:
                         utool = getToolByName(self, 'portal_url')
                         rpath = utool.getRelativeUrl(proxy)
@@ -358,7 +380,7 @@ class ProxyTool(UniqueObject, SimpleItemWithProperties):
     def getProxyInfosFromDocid(self, docid, workflow_vars=()):
         """Get the proxy infos from a docid.
 
-        Returns a list of info, of raises KeyError if there is no proxy
+        Returns a list of info, or raises KeyError if there is no proxy
         corresponding to docid.
 
         Info is a dict with:
@@ -528,7 +550,11 @@ class ProxyTool(UniqueObject, SimpleItemWithProperties):
 
     security.declarePrivate('_reindexProxiesForObject')
     def _reindexProxiesForObject(self, ob):
-        """Reindex all the proxies corresponding to an object in the repo."""
+        """Reindex the proxies corresponding to a repository object.
+
+        Also sends notification events (for these proxies).
+        """
+        evtool = getEventService(self)
         repotool = getToolByName(self, 'portal_repository')
         portal = aq_parent(aq_inner(self))
         docid, rev = repotool.getDocidAndRevisionFromObjectId(ob.getId())
@@ -550,6 +576,7 @@ class ProxyTool(UniqueObject, SimpleItemWithProperties):
             #LOG('ProxyTool', TRACE, '_reindexProxiesForObject reindexing '
             #    'rpath=%s' % rpath)
             proxy.reindexObject()
+            evtool.notify('sys_modify_object', proxy, {})
 
     # XXX implement this
     security.declarePublic('searchResults')
@@ -607,100 +634,6 @@ class ProxyTool(UniqueObject, SimpleItemWithProperties):
         repotool = getToolByName(self, 'portal_repository')
         self._unshareContentDoRecursion(proxy, repotool)
 
-    # XXX NOTE, the object may not be in the indexes yet...
-    # XXX _createObject -> _insertWorkflow -> _reindexWorkflowVariables
-    # XXX -> reindex() -> setSecurity
-    # XXX The CMF Add event is only sent after that...
-
-    security.declarePrivate('_setSecurityOnDocid')
-    def _setSecurityOnDocid(self, docid, skip_rpath=None):
-        """Set security to all revisions for a docid.
-
-        Also applies security to the archived revisions.
-
-        If skip_rpath, don't take that rpath into account (used when a
-        deletion is processed).
-
-        Returns the number of revisions whose security was changed.
-        """
-        portal = aq_parent(aq_inner(self))
-        repotool = getToolByName(self, 'portal_repository')
-        allperms = self._getRelevantPermissions()
-
-        all_revs = repotool.listRevisions(docid)
-
-        # Collect base security on pointed revisions,
-        # and collect archivers.
-        archivers_d = {}
-        rev_userperms = {}
-        for rev in all_revs:
-            userperms = {}
-            rev_userperms[rev] = userperms
-            rpaths = self._docid_rev_to_rpaths.get((docid, rev), ())
-            for rpath in rpaths:
-                if rpath == skip_rpath:
-                    continue
-                ob = portal.unrestrictedTraverse(rpath)
-                merged = mergedLocalRoles(ob, withgroups=1).items()
-                #LOG('_setSecurity', DEBUG, ' rpath=%s merged=%s' %
-                #    (rpath, merged))
-                for perm in allperms:
-                    proles = rolesForPermissionOn(perm, ob)
-                    for user, lroles in merged:
-                        for lr in lroles:
-                            if lr in proles:
-                                if perm == ViewArchivedRevisions:
-                                    archivers_d[user] = None
-                                else:
-                                    perms = userperms.setdefault(user, [])
-                                    if perm not in perms:
-                                        perms.append(perm)
-
-        # Update security for archivers, and apply it.
-        change_count = 0
-        archivers = archivers_d.keys()
-        for rev in all_revs:
-            # Update security for archivers.
-            userperms = rev_userperms[rev]
-            for user in archivers:
-                perms = userperms.setdefault(user, [])
-                if View not in perms:
-                    perms.append(View)
-            # Apply security.
-            changed = repotool.setRevisionSecurity(docid, rev, userperms)
-            if changed:
-                change_count += 1
-
-        return change_count
-
-    security.declarePrivate('setSecurity')
-    def setSecurity(self, proxy, skip_rpath=None):
-        """Reapply correct security info to the revisions of a proxy.
-
-        If skip_rpath, don't take that rpath into account (used when a
-        deletion is processed).
-
-        (Called by ProxyBase and self.) XXX but should use an event
-        """
-        # XXX should not get directly an object... or should it?
-
-        if not isinstance(proxy, ProxyBase):
-            return
-
-        self._setSecurityOnDocid(proxy.getDocid(), skip_rpath=skip_rpath)
-
-    def _getRelevantPermissions(self):
-        """Get permissions relevant to security info discovery.
-
-        Returns all the permissions managed by all the workflows,
-        and also 'View archived revisions'.
-        """
-        wftool = getToolByName(self, 'portal_workflow')
-        perms = list(wftool.getManagedPermissions())
-        if ViewArchivedRevisions not in perms:
-            perms.append(ViewArchivedRevisions)
-        return perms
-
     #
     # XXX
     # This does object construction like TypesTool but without security
@@ -755,8 +688,11 @@ class ProxyTool(UniqueObject, SimpleItemWithProperties):
         if final_type_name is None:
             final_type_name = type_name
         ob._setPortalTypeName(final_type_name)
-        ##ob.reindexObject(idxs=['portal_type', 'Type'])
-        # XXX should notify something
+        # Send a creation event with the correct portal_type set
+        evtool = getEventService(self)
+        evtool.notify('sys_add_object', ob, {})
+        # Object has been constructed without indexing, index it now.
+        ob.reindexObject()
         return ob
 
     #
@@ -768,6 +704,12 @@ class ProxyTool(UniqueObject, SimpleItemWithProperties):
         """Add knowledge about a new proxy.
 
         Maintains internal indexes.
+
+        We have to be flexible about receiving several adds for the same
+        rpath. Because when a proxy is initially created it is without a
+        portal_type, and after the portal_type is really set we have to
+        send another sys_add_object so that subscribers caring about the
+        portal_type see it created.
         """
         docid = proxy.getDocid()
         language_revs = proxy.getLanguageRevisions()
@@ -776,11 +718,6 @@ class ProxyTool(UniqueObject, SimpleItemWithProperties):
         if rpath not in rpaths:
             rpaths = rpaths + (rpath,)
             self._docid_to_rpaths[docid] = rpaths
-        else:
-            LOG('ProxyTool', ERROR,
-                'Index _docid_to_rpaths for %s already has rpath=%s: %s'
-                % (docid, rpath, rpaths))
-            raise ValueError, rpath
 
         self._rpath_to_infos[rpath] = (docid, language_revs)
 
@@ -793,11 +730,6 @@ class ProxyTool(UniqueObject, SimpleItemWithProperties):
             if rpath not in rpaths:
                 rpaths = rpaths + (rpath,)
                 self._docid_rev_to_rpaths[key] = rpaths
-            else:
-                LOG('ProxyTool', ERROR,
-                    'Index _docid_rev_to_rpaths for %s already has '
-                    'rpath=%s: %s' % (key, rpath, rpaths))
-                raise ValueError, rpath
 
 
     security.declarePrivate('_delProxy')
@@ -838,51 +770,47 @@ class ProxyTool(UniqueObject, SimpleItemWithProperties):
     # Event notification
     #
 
+    # BBB will be removed in CPS 3.5
     security.declarePrivate('notify_proxy')
-    def notify_proxy(self, event_type, object, infos):
+    def notify_proxy(self, event_type, ob, infos):
+        """Old event service tool subscriber method, now unused.
+        """
+        return
+
+    security.declarePrivate('handleObjectEvent')
+    def handleObjectEvent(self, ob, event):
         """Notification from the event service.
 
         Called when a proxy is added/deleted/modified.
-        Updates internal indexes.
-        """
-        # XXX Called when a document is modified. Notifies the proxies
-        # that they have implicitly been modified. (Would be used so
-        # that Title is reindexed for instance.)
 
-        if event_type in ('sys_add_object',
-                          'sys_del_object',
-                          'sys_modify_object',
-                          'modify_object'):
-            if isinstance(object, ProxyBase):
-                #LOG('ProxyTool', DEBUG, 'Got %s for proxy %s'
-                #    % (event_type, '/'.join(object.getPhysicalPath())))
-                rpath = infos['rpath']
-                dodel = 0
-                if event_type == 'sys_add_object':
-                    self._addProxy(object, rpath)
-                elif event_type == 'sys_modify_object':
-                    self._modifyProxy(object, rpath)
-                elif event_type == 'sys_del_object':
-                    dodel = 1
-                # Refresh security on the revisions.
-                if dodel:
-                    skip_rpath = rpath
-                else:
-                    skip_rpath = None
-                self.setSecurity(object, skip_rpath=skip_rpath)
-                if dodel:
-                    # Must be done last...
+        Updates internal indexes.
+
+        When a repository document is modified, reindexes the proxies
+        and notifies of their modification.
+        """
+        if self.ignore_events:
+            return
+
+        # Proxy events
+        if ICPSProxy.providedBy(ob):
+            pplen = len(self.getPhysicalPath()) - 1
+            rpath = '/'.join(ob.getPhysicalPath()[pplen:])
+            if IObjectWillBeMovedEvent.providedBy(event):
+                if event.oldParent is not None:
                     self._delProxy(rpath)
-                #LOG('ProxyTool', DEBUG, '  ... done')
-            else:
-                repotool = getToolByName(self, 'portal_repository')
-                if repotool.isObjectInRepository(object):
-                    if event_type in ('sys_modify_object', 'modify_object'):
-                        #LOG('ProxyTool', DEBUG, 'Got %s for repoob %s'
-                        #   % (event_type, '/'.join(object.getPhysicalPath())))
-                        # Repo object was modified, reindex all the proxies.
-                        self._reindexProxiesForObject(object)
-                        #LOG('ProxyTool', DEBUG, '  ... done')
+            elif IObjectMovedEvent.providedBy(event):
+                if event.newParent is not None:
+                    self._addProxy(ob, rpath)
+            elif IObjectModifiedEvent.providedBy(event):
+                self._modifyProxy(ob, rpath)
+
+        # Repository modification events
+        elif IObjectModifiedEvent.providedBy(event):
+            repotool = getToolByName(self, 'portal_repository', None)
+            if repotool is None:
+                return
+            if repotool.isObjectInRepository(ob):
+                self._reindexProxiesForObject(ob)
 
     #
     # Management
@@ -961,45 +889,6 @@ class ProxyTool(UniqueObject, SimpleItemWithProperties):
         for ob in portal.objectValues():
             self._recurse_rebuild(ob, utool)
 
-    security.declareProtected(ManagePortal, 'rebuildRepositorySecurity')
-    def rebuildRepositorySecurity(self, base_rpath=''):
-        """Rebuild security info on all repo objects.
-
-        If base_rpath is not None, do only objects impacted by proxies
-        under that rpath.
-
-        Return the number of docids impacted and the number
-        of revisions that changed.
-        """
-        LOG('rebuildRepositorySecurity', DEBUG, 'Starting')
-
-        # Find all docids in repo pointed by relevant proxies.
-        if base_rpath:
-            base_rpath_slash = base_rpath + '/'
-        else:
-            base_rpath_slash = ''
-        docids_d = {}
-        rpaths = [] # for debug
-        for rpath, infos in self._rpath_to_infos.items():
-            if rpath == base_rpath or rpath.startswith(base_rpath_slash):
-                rpaths.append(rpath) # for debug
-                docid, language_revs = infos
-                docids_d[docid] = None
-        docids = docids_d.keys()
-
-        LOG('rebuildRepositorySecurity', DEBUG,
-            " Impacted objects under '%s': %d proxies, %d docids"
-            % (base_rpath, len(rpaths), len(docids)))
-
-        # Update security on the docids.
-        change_count = 0
-        for docid in docids:
-            change_count += self._setSecurityOnDocid(docid)
-
-        LOG('rebuildRepositorySecurity', DEBUG,
-            'Done (%d docids, %d revs changed)' % (len(docids), change_count))
-        return (len(docids), change_count)
-
     #
     # ZMI
     #
