@@ -15,32 +15,43 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: TreesTool.py 3308 2005-04-14 10:04:04Z fguillaume $
+# $Id: TreesTool.py 32585 2006-02-03 14:59:04Z fguillaume $
 """Trees Tool, that caches some information about the site's hierarchies.
 """
 
-from zLOG import LOG, DEBUG, ERROR
-from Globals import InitializeClass, DTMLFile
-from Acquisition import aq_base, aq_parent, aq_inner
+import logging
+from ZODB.loglevels import TRACE, BLATHER
 from AccessControl import ClassSecurityInfo
 from AccessControl import Unauthorized
 from AccessControl import getSecurityManager
 from AccessControl.SecurityManagement import newSecurityManager
 from AccessControl.SecurityManagement import setSecurityManager
 from AccessControl.User import UnrestrictedUser as BaseUnrestrictedUser
-
+from Acquisition import aq_base, aq_parent, aq_inner
 from BTrees.OOBTree import OOBTree
+from Globals import InitializeClass, DTMLFile
 from OFS.Folder import Folder
 
-from Products.CMFCore.permissions \
-    import View, ManagePortal, ViewManagementScreens
-from Products.CMFCore.utils \
-    import SimpleItemWithProperties, UniqueObject, getToolByName
+from Products.CMFCore.utils import SimpleItemWithProperties
+from Products.CMFCore.utils import UniqueObject
+from Products.CMFCore.utils import getToolByName
+from Products.CMFCore.permissions import View
+from Products.CMFCore.permissions import ManagePortal
+from Products.CMFCore.permissions import ViewManagementScreens
 
+from Products.CPSUtil.text import truncateText
 from Products.CPSCore.utils import getAllowedRolesAndUsersOfUser
 from Products.CPSCore.utils import getAllowedRolesAndUsersOfObject
-from Products.CPSUtil.text import truncateText
+from Products.CPSCore.TreeCacheManager import get_treecache_manager
 
+from Products.CPSCore.treemodification import ADD, REMOVE, MODIFY
+from Products.CPSCore.treemodification import printable_op
+
+
+from zope.app.event.interfaces import IObjectModifiedEvent
+from zope.app.container.interfaces import IObjectMovedEvent
+from zope.app.container.interfaces import IContainerModifiedEvent
+from OFS.interfaces import IObjectWillBeMovedEvent
 
 def intersects(a, b):
     for v in a:
@@ -48,6 +59,10 @@ def intersects(a, b):
             return True
     return False
 
+from zope.interface import implements
+from Products.CPSCore.interfaces import ITreeTool
+from Products.CPSCore.interfaces import ITreeCache
+
 
 class UnrestrictedUser(BaseUnrestrictedUser):
     """Unrestricted user that still has an id."""
@@ -55,34 +70,100 @@ class UnrestrictedUser(BaseUnrestrictedUser):
         """Return the ID of the user."""
         return self.getUserName()
 
+logger = logging.getLogger('CPSCore.TreesTool')
+
 
 class TreesTool(UniqueObject, Folder):
     """Trees Tool that caches information about the site's hierarchies.
     """
 
+    implements(ITreeTool)
+
     id = 'portal_trees'
     meta_type = 'CPS Trees Tool'
 
     security = ClassSecurityInfo()
 
+    _properties = Folder._properties + (
+        {'id': 'ignore_events', 'type': 'boolean', 'mode': 'w',
+         'label': "Ignore events"},
+        )
+    ignore_events = False
+
     security.declarePrivate('notify_tree')
-    def notify_tree(self, event_type, ob, infos):
+    def notify_tree(self, event_type, ob, infos=None):
         """Notification method called by the event service.
 
-        Dispatches to the caches notification methods.
-        """
+        Dispatches to the accurate caches notification methods.
 
-        if event_type not in ('sys_add_cmf_object', # XXX ugh clean this up
+        infos is ignored.
+        """
+        if self.ignore_events:
+            return
+        if event_type not in ('sys_add_cmf_object',
+                              'sys_add_object',
                               'sys_del_object',
                               'sys_modify_object',
                               'sys_modify_security',
                               'sys_order_object',
                               'modify_object'):
             return
-        LOG('TreesTool', DEBUG, 'Got %s for %s'
-            % (event_type, '/'.join(ob.getPhysicalPath())))
-        for tree in self.objectValues():
-            tree.notify_tree(event_type, ob, infos)
+
+        path = ob.getPhysicalPath()
+        logger.log(TRACE, "Got %s for %s", event_type, '/'.join(path))
+        for cache in self.objectValues():
+            if cache.isCandidate(ob):
+                if event_type in ('sys_add_cmf_object', 'sys_add_object'):
+                    op = ADD
+                    info = None
+                elif event_type == 'sys_del_object':
+                    op = REMOVE
+                    info = None
+                elif event_type in ('sys_modify_object', 'modify_object'):
+                    op = MODIFY
+                    info = {'full': True}
+                elif event_type == 'sys_modify_security':
+                    op = MODIFY
+                    info = {'security': True}
+                elif event_type == 'sys_order_object':
+                    op = MODIFY
+                    info = {'order': True}
+                else:
+                    raise ValueError("Invalid event type %s" % event_type)
+                get_treecache_manager().push(cache, op, path, info)
+
+        """
+        LOG('TreesTool', DEBUG, "Got %s for %s" %
+            (event.__class__.__name__, '/'.join(path)))
+        for cache in self.objectValues():
+            if cache.isCandidate(ob):
+                op = None
+                info = None
+                if IObjectWillBeMovedEvent.providedBy(event):
+                    if event.oldParent is not None:
+                        op = REMOVE
+                elif IObjectMovedEvent.providedBy(event):
+                    if event.newParent is not None:
+                        op = ADD
+                elif IContainerModifiedEvent.providedBy(event):
+                    op = MODIFY
+                    info = {'order': True}
+                elif IObjectModifiedEvent.providedBy(event):
+                    # XXX check descriptions for security here
+                    # XXX also security is recursive!
+                    op = MODIFY
+                    info = {'full': True}
+                if op is None:
+                    continue
+                get_treecache_manager().push(cache, op, path, info)
+        """
+
+
+    security.declarePrivate('flushEvents')
+    def flushEvents(self):
+        """Flush tree cache manager, which executes the modifications.
+        """
+        get_treecache_manager()()
 
     #
     # ZMI
@@ -93,7 +174,8 @@ class TreesTool(UniqueObject, Folder):
                  'action': 'manage_addCPSTreeCacheForm',
                  'permission': ManagePortal},)
 
-    security.declareProtected(ViewManagementScreens, 'manage_addCPSTreeCacheForm')
+    security.declareProtected(ViewManagementScreens,
+                              'manage_addCPSTreeCacheForm')
     manage_addCPSTreeCacheForm = DTMLFile('zmi/tree_add', globals())
 
     security.declareProtected(ViewManagementScreens, 'manage_addCPSTreeCache')
@@ -109,9 +191,260 @@ class TreesTool(UniqueObject, Folder):
 InitializeClass(TreesTool)
 
 
+class TreeCacheUpdater(object):
+    """Get or update info about a cache.
+    """
+    def __init__(self, cache):
+        self.cache = cache
+        self.infos = cache._infos
+        self.portal = getToolByName(cache, 'portal_url').getPortalObject()
+        self.plen = len(self.portal.getPhysicalPath())
+        self.info_method = cache.info_method
+        tmp_user = UnrestrictedUser('manager', '', ['Manager'], '')
+        self.tmp_user = tmp_user.__of__(aq_inner(cache.acl_users))
+        self.root = cache.getRoot()
+
+    def getRpathFromPath(self, path):
+        return '/'.join(path[self.plen:])
+
+    def getRpath(self, ob):
+        return self.getRpathFromPath(ob.getPhysicalPath())
+
+    # Operations on objects
+
+    def isCandidate(self, ob):
+        """Return True if the object should be cached."""
+        # Check under root
+        if not self.root:
+            return False
+        rpath_slash = self.getRpath(ob)+'/'
+        if not rpath_slash.startswith(self.root+'/'):
+            return False
+        # Check excluded rpaths
+        for excluded_rpath in self.cache.excluded_rpaths:
+            if rpath_slash.startswith(excluded_rpath+'/'):
+                return False
+        # Check types
+        bob = aq_base(ob)
+        if (self.cache.meta_types and
+            getattr(bob, 'meta_type', None) not in self.cache.meta_types):
+            return False
+        type_names = self.cache.type_names or ()
+        if getattr(bob, 'portal_type', None) not in type_names:
+            return False
+
+        return True
+
+    def getNodeInfo(self, ob):
+        """Compute info about one object.
+        """
+        info = {}
+        if self.cache.info_method:
+            method = getattr(ob, self.cache.info_method, None)
+            if method is not None:
+                doc = ob.getContent(lang='default')
+
+                # Call the info method while being a temporary Manager
+                # so that it can access protected methods.
+                old_sm = getSecurityManager()
+                try:
+                    newSecurityManager(None, self.tmp_user)
+                    info = method(doc=doc)
+                finally:
+                    setSecurityManager(old_sm)
+
+                if not isinstance(info, dict):
+                    logger.error("getNodeInfo returned non-dict %r", info)
+                    info = {}
+        info.update({
+            'id': ob.getId(),
+            'rpath': self.getRpath(ob),
+            'portal_type': ob.portal_type,
+            })
+        info.update(self.getNodeSecurityInfo(ob))
+        return info
+
+    def getNodeSecurityInfo(self, ob):
+        """Get the security info about one object.
+        """
+        allowed_roles_and_users = getAllowedRolesAndUsersOfObject(ob)
+        local_roles = {}
+        for k, v in ob.get_local_roles():
+            local_roles['user:'+k] = v
+        for k, v in ob.get_local_group_roles():
+            local_roles['group:'+k] = v
+        return {
+            'allowed_roles_and_users': allowed_roles_and_users,
+            'local_roles': local_roles,
+            }
+
+    def updateNode(self, ob):
+        """Compute one node in the tree.
+
+        Keeps children info from previous node if available.
+        """
+        rpath = self.getRpath(ob)
+        old_info = self.infos.get(rpath)
+        info = self.getNodeInfo(ob)
+        if old_info is not None:
+            info['depth'] = old_info['depth']
+            info['children'] = old_info['children']
+            info['nb_children'] = old_info['nb_children']
+            self.infos[rpath] = info
+        else:
+            # Compute depth
+            root = self.root
+            depth = rpath.count('/') - root.count('/')
+            info['depth'] = depth
+            self.infos[rpath] = info
+            self.updateChildrenInfo(ob)
+
+    def updateChildrenInfo(self, ob):
+        """Recompute the list of children a node has.
+        """
+        rpath = self.getRpath(ob)
+        info = self.infos.get(rpath)
+        if info is None:
+            # Parent is outside of the tree
+            return
+        children = []
+        for subob in ob.objectValues():
+            if self.isCandidate(subob):
+                subrpath = self.getRpath(subob)
+                children.append(subrpath)
+        info['children'] = children
+        info['nb_children'] = len(children)
+        self.infos[rpath] = info
+
+    def makeTree(self, ob):
+        """Recompute the tree starting from ob."""
+        rpath = self.getRpath(ob)
+        root = self.root
+        depth = rpath.count('/') - root.count('/')
+        self._makeTree(ob, depth)
+
+    def _makeTree(self, ob, depth):
+        """Recompute the tree starting from ob.
+
+        Recursive method.
+        """
+        info = self.getNodeInfo(ob)
+        subdepth = depth+1
+        children = []
+        for subob in ob.objectValues():
+            if self.isCandidate(subob):
+                subrpath = self._makeTree(subob, subdepth)
+                children.append(subrpath)
+        info['depth'] = depth
+        info['children'] = children
+        info['nb_children'] = len(children)
+        rpath = info['rpath']
+        self.infos[rpath] = info
+        return rpath
+
+    def updateSecurityUnder(self, ob):
+        """Update security under an object.
+        """
+        rpath = self.getRpath(ob)
+        info = self.infos.get(rpath)
+        if info is not None:
+            info.update(self.getNodeSecurityInfo(ob))
+            self.infos[rpath] = info
+        # Recurse
+        for subob in ob.objectValues():
+            if self.isCandidate(subob):
+                self.updateSecurityUnder(subob)
+
+    # Operations on physical paths
+
+    def updateNodeAtPath(self, path):
+        """Update a node.
+        """
+        ob = self.portal.unrestrictedTraverse(path)
+        self.updateNode(ob)
+
+    def updateChildrenInfoAtPath(self, path):
+        """Update children info at a given physical path.
+        """
+        ob = self.portal.unrestrictedTraverse(path)
+        self.updateChildrenInfo(ob)
+
+    def addNodesUnderPath(self, path):
+        """Add a node and all its subnodes.
+        """
+        ob = self.portal.unrestrictedTraverse(path)
+        self.makeTree(ob)
+
+    def deleteNodesUnderPath(self, path):
+        """Delete all nodes at or under a given physical path.
+        """
+        rpath = self.getRpathFromPath(path)
+        for key in list(self.infos.keys(rpath+'/', rpath+'/\xFF')):
+            del self.infos[key]
+        if rpath in self.infos:
+            del self.infos[rpath]
+
+    def fixParentOfPathAfterDelete(self, path):
+        """Fix a parent's children info after a remove.
+        """
+        rpath = self.getRpathFromPath(path)
+        prpath = self.getRpathFromPath(path[:-1])
+        parent_info = self.infos.get(prpath)
+        if parent_info is None:
+            # Parent is outside of the tree
+            return
+        try:
+            parent_info['children'].remove(rpath)
+        except ValueError:
+            pass
+        else:
+            parent_info['nb_children'] -= 1
+            self.infos[prpath] = parent_info
+
+    def updateSecurityUnderPath(self, path):
+        """Update security info under a path.
+        """
+        ob = self.portal.unrestrictedTraverse(path)
+        self.updateSecurityUnder(ob)
+
+    # Operations on modificationt tree
+
+    def updateTree(self, tree):
+        """Replay modifications to a ModificationTree.
+
+        Here events have been compressed, and we have to recurse for ADD
+        and REMOVE.
+        """
+        for op, path, info in tree.get():
+            logger.log(TRACE, "  replaying %s %s %s",
+                       printable_op(op), '/'.join(path), info)
+            if op == ADD:
+                # First, delete old info about it
+                self.deleteNodesUnderPath(path)
+                # Then add new info
+                self.addNodesUnderPath(path)
+                # Fixup the parent because order is now unknown.
+                self.updateChildrenInfoAtPath(path[:-1])
+            elif op == REMOVE:
+                self.deleteNodesUnderPath(path)
+                # Fixup the parent
+                self.fixParentOfPathAfterDelete(path)
+            else: # op == MODIFY
+                if 'full' in info:
+                    self.updateNodeAtPath(path)
+                else:
+                    if 'security' in info:
+                        self.updateSecurityUnderPath(path)
+                    if 'order' in info:
+                        self.updateChildrenInfoAtPath(path)
+
+
 class TreeCache(SimpleItemWithProperties):
     """Tree cache object, caches information about one hierarchy.
     """
+
+    implements(ITreeCache)
+
     meta_type = 'CPS Tree Cache'
 
     security = ClassSecurityInfo()
@@ -122,17 +455,21 @@ class TreeCache(SimpleItemWithProperties):
          'label': 'Title'},
         {'id': 'root', 'type': 'string', 'mode': 'w',
          'label': 'Root'},
-        {'id': 'type_names', 'type': 'multiple selection', 'mode': 'w',
-         'select_variable': 'all_type_names', 'label': 'Portal Types'},
+        {'id': 'type_names', 'type': 'lines', 'mode': 'w',
+         'label': 'Portal Types'},
         {'id': 'meta_types', 'type': 'lines', 'mode': 'w',
          'label': 'Meta Types'},
+        {'id': 'excluded_rpaths', 'type': 'lines', 'mode': 'w',
+         'label': 'Excluded rpaths'},
         {'id': 'info_method', 'type': 'string', 'mode': 'w',
          'label': 'Info Method'},
         )
+
     title = ''
     root = ''
     type_names = ()
     meta_types = ()
+    excluded_rpaths = ()
     info_method = ''
 
     def __init__(self, id, **kw):
@@ -149,12 +486,10 @@ class TreeCache(SimpleItemWithProperties):
 
     def _upgrade(self):
         """Upgrade from the old format."""
-        LOG('TreeCache._upgrade', DEBUG, 'Upgrading tree %s' % self.getId())
-
+        logger.info("Upgrading tree %s", self.getId())
         delattr(self, '_tree')
         delattr(self, '_pointers')
         delattr(self, '_flat')
-
         self._clear()
         self.rebuild()
 
@@ -171,35 +506,10 @@ class TreeCache(SimpleItemWithProperties):
         res.sort()
         return res
 
-    security.declarePrivate('notify_tree')
-    def notify_tree(self, event_type, ob, infos):
-        """Notification method called when an event is received.
-
-        Called by the the trees tool's notify method.
-        """
-        self._maybeUpgrade()
-
-        urltool = getToolByName(self, 'portal_url')
-        plen = len(urltool.getPortalObject().getPhysicalPath())
-        rpath = '/'.join(ob.getPhysicalPath()[plen:])
-        if not self._isCandidate(ob, plen):
-            return
-        LOG('TreeCache.notify_tree', DEBUG, "%s: %s for %s"
-            % (self.getId(), event_type, rpath))
-
-        if event_type == 'sys_add_cmf_object':
-            self.updateNode(ob)
-            parent = aq_parent(aq_inner(ob))
-            self.updateChildrenInfo(parent)
-        elif event_type == 'sys_del_object':
-            self.deleteNode(ob)
-        elif event_type == 'sys_order_object':
-            self.updateChildrenInfo(ob)
-        elif event_type == 'sys_modify_security':
-            # This event has to do recursion by itself
-            self.makeTree(ob)
-        else: # event_type == 'modify_object' / 'sys_modify_object'
-            self.updateNode(ob)
+    security.declareProtected(View, 'isCandidate')
+    def isCandidate(self, ob):
+        """Return True if the object should be cached."""
+        return TreeCacheUpdater(self).isCandidate(ob)
 
     security.declarePrivate('rebuild')
     def rebuild(self):
@@ -211,165 +521,25 @@ class TreeCache(SimpleItemWithProperties):
             return
         root_ob = portal.unrestrictedTraverse(root, None)
         if root_ob is None:
-            LOG('TreeCache.rebuild', ERROR,
-                "%s: bad root '%s'" % (self.getId(), root))
+            # Root not present, actually legal (and common during installation)
+            logger.log(BLATHER, "Root %r not present when rebuilding %s",
+                       root, self.getId())
             return
-        self.makeTree(root_ob)
+        TreeCacheUpdater(self).makeTree(root_ob)
 
-    def _isCandidate(self, ob, plen):
-        """Return True if the object should be cached."""
-        #LOG('Tree', DEBUG, 'Is %s candidate?' % ob.getId())
-        bob = aq_base(ob)
-        if self.meta_types and getattr(bob, 'meta_type', None) not in self.meta_types:
-            #LOG('Tree', DEBUG, ' No, mt=%s' % getattr(bob, 'meta_type', None))
-            return False
-        type_names = self.type_names or [] # Stupid, may be ''.
-        if getattr(bob, 'portal_type', None) not in type_names:
-            #LOG('Tree', DEBUG,
-            #    ' No, pt=%s' % getattr(bob, 'portal_type', None))
-            return False
-        root = self.getRoot()
-        if not root:
-            return True
-        rpath = '/'.join(ob.getPhysicalPath()[plen:])
-        ok = (rpath+'/').startswith(root+'/')
-        #LOG('Tree', DEBUG, ' Returns ok=%s' % ok)
-        return ok
+    # Called by the TreeCacheManager
 
-    security.declarePrivate('getNodeInfo')
-    def getNodeInfo(self, ob, plen):
-        """Compute info about one object."""
-        info = {}
-        if self.info_method:
-            method = getattr(ob, self.info_method, None)
-            if method is not None:
-                doc = ob.getContent(lang='default')
-
-                # Call the info method while being a temporary Manager
-                # so that it can access protected methods.
-                old_sm = getSecurityManager()
-                tmp_user = UnrestrictedUser('manager', '', ['Manager'], '')
-                tmp_user = tmp_user.__of__(self.acl_users)
-                try:
-                    newSecurityManager(None, tmp_user)
-                    info = method(doc=doc)
-                finally:
-                    setSecurityManager(old_sm)
-
-                if not isinstance(info, dict):
-                    LOG('TreeCache', ERROR,
-                        "getNodeInfo returned non-dict %s" % `info`)
-                    info = {}
-        allowed_roles_and_users = getAllowedRolesAndUsersOfObject(ob)
-        local_roles = {}
-        for k, v in ob.get_local_roles():
-            local_roles['user:'+k] = v
-        for k, v in ob.get_local_group_roles():
-            local_roles['group:'+k] = v
-        ppath = ob.getPhysicalPath()
-        info.update({'id': ob.getId(),
-                     'rpath': '/'.join(ppath[plen:]),
-                     'portal_type': ob.portal_type,
-                     'allowed_roles_and_users': allowed_roles_and_users,
-                     'local_roles': local_roles,
-                     })
-        return info
-
-    security.declarePrivate('makeTree')
-    def makeTree(self, ob):
-        """Recompute the tree starting from ob."""
-        urltool = getToolByName(self, 'portal_url')
-        plen = len(urltool.getPortalObject().getPhysicalPath())
-        rpath = urltool.getRelativeUrl(ob)
-        root = self.getRoot()
-        depth = rpath.count('/') - root.count('/')
-        self._makeTree(ob, depth, plen)
-
-    security.declarePrivate('_makeTree')
-    def _makeTree(self, ob, depth, plen):
-        """Recompute the tree starting from ob.
-
-        Recursive method.
+    security.declarePrivate('updateTree')
+    def updateTree(self, tree):
+        """Replay modifications to a ModificationTree.
         """
-        info = self.getNodeInfo(ob, plen)
-        subdepth = depth+1
-        children = []
-        for subob in ob.objectValues():
-            if self._isCandidate(subob, plen):
-                subrpath = self._makeTree(subob, subdepth, plen)
-                children.append(subrpath)
-        info['depth'] = depth
-        info['children'] = children
-        info['nb_children'] = len(children)
-        rpath = info['rpath']
-        self._infos[rpath] = info
-        return rpath
+        self._maybeUpgrade()
+        TreeCacheUpdater(self).updateTree(tree)
 
-    def updateNode(self, ob):
-        """Compute one node in the tree.
-
-        Keeps children info from previous node if available.
+    def _getModificationTree(self):
+        """Debugging: get the current modification tree.
         """
-        urltool = getToolByName(self, 'portal_url')
-        plen = len(urltool.getPortalObject().getPhysicalPath())
-        rpath = urltool.getRelativeUrl(ob)
-        old_info = self._infos.get(rpath)
-        info = self.getNodeInfo(ob, plen)
-        if old_info is not None:
-            info['depth'] = old_info['depth']
-            children = old_info['children']
-            info['children'] = children
-            info['nb_children'] = len(children)
-            self._infos[rpath] = info
-        else:
-            # Compute depth
-            root = self.getRoot()
-            depth = rpath.count('/') - root.count('/')
-            info['depth'] = depth
-            self._infos[rpath] = info
-            self.updateChildrenInfo(ob)
-
-    def updateChildrenInfo(self, ob):
-        """Recompute info about children of a node."""
-        urltool = getToolByName(self, 'portal_url')
-        plen = len(urltool.getPortalObject().getPhysicalPath())
-        rpath = urltool.getRelativeUrl(ob)
-        info = self._infos.get(rpath)
-        if info is None:
-            # Parent is outside of the tree
-            return
-        children = []
-        for subob in ob.objectValues():
-            if self._isCandidate(subob, plen):
-                subrpath = urltool.getRelativeUrl(subob)
-                children.append(subrpath)
-        info['children'] = children
-        info['nb_children'] = len(children)
-        self._infos[rpath] = info
-
-    def deleteNode(self, ob):
-        """Delete a node from the tree."""
-        urltool = getToolByName(self, 'portal_url')
-        rpath = urltool.getRelativeUrl(ob)
-
-        # Update parent's children list
-        parent = aq_parent(aq_inner(ob))
-        prpath = urltool.getRelativeUrl(parent)
-        parent_info = self._infos.get(prpath)
-        if parent_info is not None:
-            try:
-                parent_info['children'].remove(rpath)
-            except ValueError:
-                pass
-            else:
-                parent_info['nb_children'] -= 1
-                self._infos[prpath] = parent_info
-
-        # Remove object info
-        try:
-            del self._infos[rpath]
-        except KeyError:
-            pass
+        return get_treecache_manager()._getModificationTree(self)
 
     def _localize(self, info, locale_keys, locale):
         """Localize info attributes specified in locale_keys into the locale
@@ -508,7 +678,10 @@ class TreeCache(SimpleItemWithProperties):
                     if done.has_key(rpath):
                         continue
                     done[rpath] = None
-                    info = infos[rpath]
+                    info = infos.get(rpath)
+                    if info is None:
+                        # Inconsistent tree, don't break completely
+                        continue
 
                     # Check depth
                     depth = info['depth']
