@@ -16,7 +16,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: ActionBox.py 7411 2005-04-15 15:19:45Z janguenot $
+# $Id: ActionBox.py 29825 2005-11-23 15:35:52Z jmorliaguet $
 
 """Backward compatibility;  see Products.CPSBoxes.ActionBox
 """
@@ -29,5 +29,6 @@ warn("The module, "
      "'Products.CPSDefault.ActionBox' "
      "is a deprecated compatiblity alias for "
      "'Products.CPSBoxes.ActionBox'; "
-     "please use the new module instead.",
+     "please use the new module instead. "
+     "This module will be removed in CPS 3.5.",
      DeprecationWarning)
