@@ -1,6 +1,6 @@
 ##parameters=lang, proxy=None, REQUEST=None
 
-# $Id: content_translate.py 7215 2005-03-03 13:58:20Z sfermigier $
+# $Id: content_translate.py 30186 2005-12-01 11:28:32Z bdelbosc $
 
 if proxy is None:
     proxy = context
@@ -10,9 +10,14 @@ if not hasattr(proxy, 'addLanguageToProxy'):
         raise ValueError("Not a proxy")
     psm = 'psm_translation_not_supported'
     url = proxy.absolute_url()
+    action = proxy.getTypeInfo().getActionById('view')
+    REQUEST.RESPONSE.redirect('%s/%s?portal_status_message=%s' % (
+        url, action, psm))
+    return
 
-localizer = context.Localizer
-if not lang in localizer.get_supported_languages():
+translationservice = context.translation_service
+
+if not lang in translationservice.getSupportedLanguages():
     if REQUEST is None:
         raise ValueError("Language '%s' not supported" % lang)
     psm = 'psm_translation_lang_not_supported'
