@@ -1,5 +1,5 @@
 ##parameters=ids=[], REQUEST=None
-# $Id: folder_delete.py 7215 2005-03-03 13:58:20Z sfermigier $
+# $Id: folder_delete.py 30246 2005-12-02 20:10:18Z fguillaume $
 """
 FIXME: add docstring.
 """
@@ -11,7 +11,9 @@ if ids:
         # XXX has to be called from here since the workflow doesn't handle the
         # deletion yet
         ob = getattr(context, id)
-        context.portal_eventservice.notifyEvent('workflow_delete', ob, {})
+        from Products.CPSCore.EventServiceTool import getPublicEventService
+        evtool = getPublicEventService(context)
+        evtool.notifyEvent('workflow_delete', ob, {})
     context.manage_delObjects(ids)
     message = 'portal_status_message=psm_item(s)_deleted'
 else:
