@@ -1,5 +1,5 @@
 ##parameters=
-#$Id: folder_paste.py 7215 2005-03-03 13:58:20Z sfermigier $
+#$Id: folder_paste.py 30246 2005-12-02 20:10:18Z fguillaume $
 """
 Paste objects from the clipboard. Used within the folder_contents template.
 """
@@ -16,9 +16,9 @@ try:
         result = context.manage_CPSpasteObjects(REQUEST['__cp'])
         for id in [ob['new_id'] for ob in result]:
             ob = getattr(context, id)
-            context.portal_eventservice.notifyEvent('workflow_cut_copy_paste',
-                                                    ob,
-                                                    {})
+            from Products.CPSCore.EventServiceTool import getPublicEventService
+            evtool = getPublicEventService(context)
+            evtool.notifyEvent('workflow_cut_copy_paste', ob, {})
         message = 'psm_item(s)_pasted'
     else:
         message = 'psm_copy_or_cut_at_least_one_document'
