@@ -1,5 +1,5 @@
 ##parameters=dt=None, fmt='medium'
-# $Id: getDateStr.py 7215 2005-03-03 13:58:20Z sfermigier $
+# $Id: getDateStr.py 23829 2005-06-10 17:54:55Z gdelaroch $
 """
 Return a string date using the current locale.
 """
@@ -11,8 +11,7 @@ from DateTime.DateTime import DateTimeError
 if not dt:
     return ''
 
-Localizer = context.Localizer
-mcat = Localizer.default
+mcat = context.translation_service
 
 if fmt in ('short', 'medium', 'long'):
     # This string will be used to retrieve format in the right .po file
