@@ -1,5 +1,5 @@
 ##parameters=root_id=None
-# $Id: getSectionsTree.py 7417 2005-04-18 13:33:03Z jmorliaguet $
+# $Id: getSectionsTree.py 23829 2005-06-10 17:54:55Z gdelaroch $
 """
 Get all sections info with allowed publishing transitions
 
@@ -11,7 +11,7 @@ if root_id is not None:
 else:
     sections_roots = context.getSectionsRoots()
 
-locale = context.Localizer.get_selected_language()
+locale = context.translation_service.getSelectedLanguage()
 ptree = context.portal_trees
 available_roots = ptree.objectIds()
 
