@@ -1,10 +1,10 @@
 ##parameters=key=None
-#$Id: getVocabularySearchReviewState.py 7354 2005-04-08 10:58:23Z bdelbosc $
+#$Id: getVocabularySearchReviewState.py 25294 2005-07-28 17:37:52Z atchertchian $
 """Return a list of review state, used as MethodVocabulary."""
 
 names = context.getWorkflowStateNames()
 l10n = context.translation_service
-res = [(name, l10n('label_' + name)) for name in names]
+res = [(name, l10n(name, default=name).capitalize()) for name in names]
 res.insert(0, ('', l10n('label_all')))
 if key is not None:
     res = [item[1] for item in res if item[0] == key][0]
