@@ -1,17 +1,18 @@
 ##parameters=object=None
 ##title=Is in a workspace
-#$Id: isInWorkspace.py 7322 2005-04-06 13:20:52Z madarche $
+#$Id: isInWorkspace.py 32458 2006-02-01 11:49:21Z div $
 """
 Return True if the context or the parameter 'object' is a workspace
 or inside a workspace.
 """
 
+ws = ['Workspace', 'Members Workspace']
 if not object:
     object = context
 
 try:
-    return object.portal_type == 'Workspace' \
-        or object.aq_parent.portal_type == 'Workspace'
+    return object.portal_type in ws \
+        or object.aq_parent.portal_type in ws
 except AttributeError:
     # FIXME: determine in which case it happens and how to react with the
     # portal's root
