@@ -1,7 +1,7 @@
 ##parameters
-# $Id: manageCPSLanguage.py 7273 2005-03-31 09:34:23Z bdelbosc $
+# $Id: manageCPSLanguage.py 31611 2006-01-13 15:32:54Z madarche $
 """
-Make available a new language in a CPS portal with Localizer
+Make available a new language in a CPS portal with TranslationService
 """
 
 # FIXME: localizergeddon
@@ -10,7 +10,7 @@ from Products.CPSDefault.utils import manageCPSLanguage
 
 request = context.REQUEST
 context_url = context.portal_url.getPortalObject().absolute_url()
-template = 'language_manage_form'
+template = 'config_form'
 psm = ''
 
 if request is not None:
