@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: BaseDirectory.py 21960 2005-05-30 16:15:38Z atchertchian $
+# $Id: BaseDirectory.py 33288 2006-02-22 13:59:11Z div $
 """BaseDirectory
 """
 
@@ -38,7 +38,8 @@ from Products.CMFCore.Expression import Expression
 from Products.CMFCore.Expression import getEngine
 from Products.PageTemplates.TALES import CompilerError
 
-from Products.CPSSchemas.PropertiesPostProcessor import PropertiesPostProcessor
+from Products.CPSUtil.PropertiesPostProcessor import PropertiesPostProcessor
+from Products.CPSSchemas.Schema import CPSSchema
 from Products.CPSSchemas.StorageAdapter import AttributeStorageAdapter
 from Products.CPSSchemas.DataModel import DataModel
 from Products.CPSSchemas.DataStructure import DataStructure
@@ -91,9 +92,9 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
     _propertiesBaseClass = SimpleItemWithProperties
     _properties = SimpleItemWithProperties._properties + (
         {'id': 'schema', 'type': 'string', 'mode': 'w',
-         'label': "Schema"},
+         'label': "Schemas"},
         {'id': 'schema_search', 'type': 'string', 'mode': 'w',
-         'label': "Schema for search"},
+         'label': "Schemas for search"},
         {'id': 'layout', 'type': 'string', 'mode': 'w',
          'label': "Layout"},
         {'id': 'layout_search', 'type': 'string', 'mode': 'w',
@@ -114,27 +115,30 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
          'label': 'Field for entry title'},
         {'id': 'search_substring_fields', 'type': 'tokens', 'mode': 'w',
          'label': 'Fields with substring search'},
+        {'id': 'is_hierarchical', 'type': 'boolean', 'mode': 'w',
+         'label': 'Directory has hierarchical support'},
         )
 
     schema = ''
     schema_search = ''
     layout = ''
     layout_search = ''
-    acl_directory_view_roles = 'Manager; Member'
+    acl_directory_view_roles = 'Manager'
     acl_entry_create_roles = 'Manager'
     acl_entry_delete_roles = 'Manager'
-    acl_entry_view_roles = 'Manager; Member'
+    acl_entry_view_roles = 'Manager'
     acl_entry_edit_roles = 'Manager'
     id_field = ''
     title_field = ''
     search_substring_fields = []
+    is_hierarchical = False
 
     entry_roles = []
 
-    acl_directory_view_roles_c = ['Manager', 'Member']
+    acl_directory_view_roles_c = ['Manager']
     acl_entry_create_roles_c = ['Manager']
     acl_entry_delete_roles_c = ['Manager']
-    acl_entry_view_roles_c = ['Manager', 'Member']
+    acl_entry_view_roles_c = ['Manager']
     acl_entry_edit_roles_c = ['Manager']
 
     _properties_post_process_split = (
@@ -185,7 +189,10 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
         """
         if entry is None:
             if id is not None:
-                entry = self._getEntry(id)
+                try:
+                    entry = self._getEntry(id)
+                except KeyError:
+                    entry = {}
             else:
                 entry = {}
         if id is None:
@@ -320,7 +327,7 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
     security.declarePrivate('listEntryIds')
     def listEntryIds(self):
         """List all the entry ids."""
-        raise NotImplementedError
+        return self._searchEntries()
 
     security.declarePrivate('listEntryIdsAndTitles')
     def listEntryIdsAndTitles(self):
@@ -339,7 +346,16 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
     security.declarePublic('hasEntry')
     def hasEntry(self, id):
         """Does the directory have a given entry?"""
-        raise NotImplementedError
+        self.checkViewEntryAllowed(id)
+        return self._hasEntry(id)
+
+    security.declarePrivate('_hasEntry')
+    def _hasEntry(self, id):
+        """Does the directory have a given entry?
+
+        This method doesn't do security checks.
+        """
+        return bool(self._searchEntries(**{self.id_field: [id]}))
 
     security.declarePublic('createEntry')
     def createEntry(self, entry):
@@ -368,8 +384,9 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
         raised if the entry does not exist.
         """
         try:
+            self.checkViewEntryAllowed()
             return self._getEntryKW(id)
-        except (KeyError, ValueError):
+        except (KeyError, ValueError, Unauthorized):
             if default is not _marker:
                 return default
             else:
@@ -470,7 +487,7 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
         """Edit an entry in the directory, unrestricted.
         """
         id = entry[self.id_field]
-        dm = self._getDataModel(id)
+        dm = self._getDataModel(id, check_acls=check_acls)
         dm_entry = self._getEntryFromDataModel(dm)
         if check_acls:
             self.checkEditEntryAllowed(id=id, entry=dm_entry)
@@ -487,14 +504,41 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
     def deleteEntry(self, id):
         """Delete an entry in the directory.
         """
-        raise NotImplementedError
+        self.checkDeleteEntryAllowed(id=id)
+        self._deleteEntry(id)
 
-    security.declarePublic('getNewEntry')
-    def getNewEntry(self, id):
-        """Return a new clean entry for entry_id.
+    security.declarePrivate('_deleteEntry')
+    def _deleteEntry(self, id):
+        """Delete an entry in the directory, unrestricted.
         """
         raise NotImplementedError
 
+    #
+    # Hierarchical support
+    #
+    security.declarePrivate('_isHierarchical')
+    def _isHierarchical(self):
+        """Return True if the directory support hierarchical methods."""
+        return self.is_hierarchical
+
+    security.declarePrivate('_listChildrenEntryIds')
+    def _listChildrenEntryIds(self, id, field_id=None):
+        """Return a children entries ids for entry 'id'.
+
+        Return a list of field_id if not None or self.id_field.
+        Available only if directory is hierarchical."""
+        raise NotImplementedError
+
+    security.declarePrivate('_getParentEntryId')
+    def _getParentEntryId(self, id, field_id):
+        """Return Parent Id of 'id'.
+
+        Return None if 'id' have no parent.
+        Return a field_id if not None or a self.id_field.
+        Available only if directory is hierarchical."""
+        raise NotImplementedError
+
+
     #
     # Rendering API
     #
@@ -598,7 +642,7 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
             # this hack works only if id is not computed by createEntry
             # XXX Hack, this should be done by field/schema... XXX
             id = dm.data[self.id_field]
-            if id and self.hasEntry(id):
+            if id and self._hasEntry(id):
                 ok = 0
                 ds.setError(self.id_field, 'cpsdir_err_entry_already_exists')
 
@@ -660,45 +704,39 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
                                                 layout_mode=layout_mode, **kw)
         else:
             ok = 1
+        formLayout = self._renderLayout(layout_structure, ds,
+                                        layout_mode=layout_mode, ok=ok, **kw)
+        rendered = ''
         if validate and ok:
             # Call callback.
             callback_func = getattr(self, callback, None)
             if callback_func is None:
                 raise ValueError("Unknown callback '%s'" % callback)
             rendered, ok = callback_func(self, ds, **kw)
-            if not rendered:
-                rendered = ''
-        else:
-            rendered = self._renderLayout(layout_structure, ds,
-                                          layout_mode=layout_mode, ok=ok, **kw)
+        rendered = formLayout + rendered
         return rendered, ok, ds
 
-    #
-    # Management API
-    #
-
-
     #
     # Internal
     #
 
     security.declarePrivate('_getSchemas')
-    def _getSchemas(self, search=0):
+    def _getSchemas(self, search=False):
         """Get the schemas for this directory.
 
-        If search=1, get the schemas for a search.
+        If search=True, get the schemas for a search.
 
         Returns a sequence of Schema objects.
         """
         stool = getToolByName(self, 'portal_schemas')
         schemas = []
         if not search:
-            schema_ids = [self.schema]
+            schema_ids = self.schema.split()
         else:
             if self.schema_search:
-                schema_ids = [self.schema_search]
+                schema_ids = self.schema_search.split()
             else:
-                schema_ids = [self.schema]
+                schema_ids = self.schema.split()
         for schema_id in schema_ids:
             schema = stool._getOb(schema_id, None)
             if schema is None:
@@ -707,13 +745,83 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
             schemas.append(schema)
         return schemas
 
+    security.declarePrivate('_getFieldItems')
+    def _getFieldItems(self, search=False):
+        """Get the schemas fields for this directory
+
+        If search=True, get the schemas keys for a search.
+
+        Returns a sequence of fields items (field_id, field)
+
+        Doesn't return duplicate field ids coming from differents schemas.
+        """
+        seen = {}
+        items = []
+        schemas = self._getSchemas(search=search)
+        for schema in schemas:
+            for field_id, field in schema.items():
+                if field_id not in seen:
+                    items.append((field_id, field))
+                    seen[field_id] = None
+        return items
+
     security.declarePrivate('_getFieldIds')
-    def _getFieldIds(self, search=0):
-        """Get the fields ids by querying the schemas."""
-        keys = []
-        for schema in self._getSchemas(search=search):
-            keys.extend(schema.keys())
-        return keys
+    def _getFieldIds(self, search=False):
+        """Get the schemas keys for this directory
+
+        If search=True, get the schemas keys for a search.
+
+        Returns a sequence of field ids.
+
+        Doesn't return duplicate field ids coming from differents schemas.
+        """
+        return [x[0] for x in self._getFieldItems(search=search)]
+
+    security.declarePrivate('_getUniqueSchema')
+    def _getUniqueSchema(self, search=False):
+        """Return a unique schema for this directory.
+
+        We have two cases here:
+
+         * Only one schema specified:
+
+           simply return it
+
+         * Several schemas specified:
+
+           Generate dynamicaly a *non* persistent CPSSchema instance
+           that aggregates all the fields from the different schemas of
+           this directory.
+
+           It the different schemas define fields with the same id, then
+           the field defined on the first schema will be kept.
+
+        This is used by the directory adapter storage because only one
+        exists right now for a given directory.
+        """
+        schemas = self._getSchemas(search=search)
+        # Do not generate a schema in this case but return the only
+        # specified schema on this directory.
+        if len(schemas) == 1:
+            return schemas[0]
+        # Generate a CPSSchema and add the aggregated fields on it.
+        # TODO : ensure it's not too costly to generate this
+        schema = CPSSchema(id=self.getId())
+        seen = {}
+        for field_id, field in self._getFieldItems(search=search):
+            if field_id not in seen:
+                schema.addSubObject(field)
+                seen[field_id] = None
+        return schema
+
+    security.declarePrivate('_getSchemaFieldById')
+    def _getSchemaFieldById(self, field_id, search=False):
+        """Return a field from the directory's schemas.
+        """
+        for id, field in self._getFieldItems(search=search):
+            if id == field_id:
+                return field
+        return None
 
     security.declarePrivate('_getSearchFields')
     def _getSearchFields(self, return_fields=None):
@@ -723,9 +831,7 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
         Also compute dependant fields.
         """
         res = []
-        # XXX AT: I dont get why only first schema is used
-        schema = self._getSchemas()[0]
-        all_field_ids = schema.keys()
+        all_field_ids = self._getFieldIds()
         field_ids_d = {self.id_field: None}
         if return_fields is not None:
             if return_fields == ['*']:
@@ -737,7 +843,10 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
             dep_ids = []
             for field_id in return_fields:
                 field_ids_d[field_id] = None
-                rpdf = schema[field_id].read_process_dependent_fields
+                field = self._getSchemaFieldById(field_id)
+                if field is None:
+                    continue
+                rpdf = list(field.read_process_dependent_fields)
                 dep_ids.extend(rpdf)
             for field_id in dep_ids:
                 field_ids_d[field_id] = None
@@ -869,7 +978,7 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
                     return {}
         mapping = {}
         # Put None values by default for all schema items.
-        for field_id in self._getSchemas()[0].keys():
+        for field_id in self._getFieldIds():
             mapping[field_id] = None
         # Put filled-in entry items in the namespace.
         mapping.update(entry)
@@ -976,7 +1085,7 @@ class BaseDirectory(PropertiesPostProcessor, SimpleItemWithProperties):
     def manage_getZMISearchFields(self):
         """Find which fields we can easily search from ZMI."""
         infos = []
-        for schema in self._getSchemas(search=1):
+        for schema in self._getSchemas(search=True):
             for field_id, field in schema.items():
                 mt = field.meta_type
                 if mt in ('CPS String Field',
