@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: LDAPBackingDirectory.py 21966 2005-05-30 19:36:41Z fguillaume $
+# $Id: LDAPBackingDirectory.py 31826 2006-01-19 17:40:04Z fguillaume $
 """LDAPBackingDirectory
 
 This is a simple but efficient version of an LDAP Directory.
@@ -32,6 +32,7 @@ import sys
 from Globals import InitializeClass
 from AccessControl import ClassSecurityInfo
 from OFS.Image import Image
+from OFS.Cache import Cacheable
 
 from Products.CPSSchemas.StorageAdapter import BaseStorageAdapter
 from Products.CPSSchemas.Field import ValidationError
@@ -41,12 +42,20 @@ from Products.CPSDirectory.BaseDirectory import AuthenticationFailed
 from Products.CPSDirectory.BaseDirectory import ConfigurationError
 from Products.CPSDirectory.BaseDirectory import _replaceProperty
 
+from Products.CPSDirectory.interfaces import IDirectory
+
+from zope.interface import implements
+
+
 # During testing?
+# XXX GR this isn't robust: depends on the frame number, which may change again
 from inspect import currentframe
-if currentframe(4).f_code.co_filename.endswith('test.py'):
+filename = currentframe(3).f_code.co_filename
+if '/ZopeTestCase/' in filename or filename.endswith("/testrunner.py"):
     _TESTING = True
 else:
     _TESTING = False
+del filename, currentframe
 
 # Import LDAP modules, maybe from testing code
 if not _TESTING:
@@ -121,7 +130,7 @@ def implodeRDN(avas):
     return '+'.join(avas)
 
 
-class LDAPBackingDirectory(BaseDirectory):
+class LDAPBackingDirectory(BaseDirectory, Cacheable):
     """LDAP Backing Directory.
 
     A directory that connects to an LDAP server.
@@ -134,10 +143,22 @@ class LDAPBackingDirectory(BaseDirectory):
       special ASCII chars are LDAP escaped (ex '\\3D' for '=').
 
     - 'base_dn' is read-only.
+
+
+    Hierarchical support requires to set is_hierarchical to True and to provide
+    a children_attr that holds the list of children for the current entry. The
+    list of children can be a list of 'cn' instead of 'dn', this can be set
+    using the children_id_attr property.
     """
+    implements(IDirectory)
 
     meta_type = 'CPS LDAP Backing Directory'
 
+    manage_options = (
+        BaseDirectory.manage_options +
+        Cacheable.manage_options
+        )
+
     security = ClassSecurityInfo()
 
     _properties = BaseDirectory._properties + (
@@ -154,6 +175,8 @@ class LDAPBackingDirectory(BaseDirectory):
          'label': 'LDAP use ssl'},
         {'id': 'ldap_base', 'type': 'string', 'mode': 'w',
          'label': 'LDAP base'},
+        {'id': 'ldap_base_creation', 'type': 'string', 'mode': 'w',
+         'label': 'LDAP base for creation'},
         {'id': 'ldap_scope', 'type': 'selection', 'mode': 'w',
          'label': 'LDAP scope', 'select_variable': 'all_ldap_scopes'},
         {'id': 'ldap_search_classes', 'type': 'string', 'mode': 'w',
@@ -168,6 +191,10 @@ class LDAPBackingDirectory(BaseDirectory):
          'label': 'LDAP rdn attribute (create)'},
         {'id': 'ldap_object_classes', 'type': 'string', 'mode': 'w',
          'label': 'LDAP object classes (create)'},
+        {'id': 'children_attr', 'type': 'string', 'mode': 'w',
+         'label': 'Field that contains a list of sub entries id for hierarchical directory'},
+        {'id': 'children_id_attr', 'type': 'string', 'mode': 'w',
+         'label': 'attr used as id for children_attr default is ldap_rdn_attr.'},
         )
     _properties = _replaceProperty(
         _properties, 'id_field',
@@ -183,6 +210,7 @@ class LDAPBackingDirectory(BaseDirectory):
     ldap_port = 389
     ldap_use_ssl = 0
     ldap_base = ''
+    ldap_base_creation = ''
     ldap_scope = 'ONELEVEL'
     ldap_search_classes = 'person'
     ldap_search_filter = ''
@@ -191,6 +219,9 @@ class LDAPBackingDirectory(BaseDirectory):
     ldap_rdn_attr = 'cn'
     ldap_object_classes = 'top, person'
 
+    children_id_attr = ldap_rdn_attr
+    children_attr = None
+
     ldap_scope_c = ldap.SCOPE_SUBTREE
     ldap_search_classes_c = ['person']
     ldap_search_classes_filter = '(objectClass=person)'
@@ -236,6 +267,7 @@ class LDAPBackingDirectory(BaseDirectory):
         self.ldap_search_classes_filter = filt
         # Reset connection (XXX problems with multiple threads)
         self._v_conn = None
+        self.ZCacheable_invalidate()
 
     security.declarePrivate('_getAdapters')
     def _getAdapters(self, id, search=0, **kw):
@@ -276,9 +308,10 @@ class LDAPBackingDirectory(BaseDirectory):
         res = self._searchEntriesFiltered(filter, attrs)
         return res
 
-    security.declarePublic('hasEntry')
-    def hasEntry(self, id):
+    security.declarePrivate('_hasEntry')
+    def _hasEntry(self, id):
         """Does the directory have a given entry?"""
+        # XXX should use base class implementation
         if not isCanonicalDN(id):
             raise KeyError("Entry DN '%s' is not canonical")
         try:
@@ -309,7 +342,7 @@ class LDAPBackingDirectory(BaseDirectory):
         if not id.strip() or not password.strip():
             # We don't want to cause an empty LDAP search.
             # Empty passwords are forbidden too  due to a
-            # strange behaviour of some LDAP servers (Sun iPlanet) 
+            # strange behaviour of some LDAP servers (Sun iPlanet)
             raise AuthenticationFailed
         return self._getEntryKW(id, password=password, **kw)
 
@@ -330,6 +363,8 @@ class LDAPBackingDirectory(BaseDirectory):
             ava = '%s=%s' % (self.ldap_rdn_attr, rdn)
             if self.ldap_scope_c == ldap.SCOPE_ONELEVEL:
                 base_dn = self.ldap_base
+            elif self.ldap_base_creation:
+                base_dn = self.ldap_base_creation
             else: # ldap.SCOPE_SUBTREE
                 if not entry.has_key('base_dn'):
                     raise ValueError("Missing base_dn in entry")
@@ -341,21 +376,91 @@ class LDAPBackingDirectory(BaseDirectory):
                                      (base_dn, self.ldap_base))
             dn = implodeDN((ava, base_dn))
 
-        if self.hasEntry(dn):
+        if self._hasEntry(dn):
             raise KeyError("Entry '%s' already exists" % dn)
 
         ldap_attrs['objectClass'] = list(self.ldap_object_classes_c)
         self.insertLDAP(dn, ldap_attrs)
         return dn
 
-    security.declarePublic('deleteEntry')
-    def deleteEntry(self, id):
+    security.declarePrivate('_deleteEntry')
+    def _deleteEntry(self, id):
         """Delete an entry in the directory."""
-        self.checkDeleteEntryAllowed(id=id)
-        if not self.hasEntry(id):
+        if not self._hasEntry(id):
             raise KeyError("No entry '%s'" % id)
         self.deleteLDAP(id)
 
+
+    #
+    # Hierarchical support
+    #
+    security.declarePrivate('_isHierarchical')
+    def _isHierarchical(self):
+        """Return True if the directory support hierarchical methods."""
+        if self.is_hierarchical and self.children_attr is not None:
+            return True
+        return False
+
+
+    security.declarePrivate('_listChildrenEntryIds')
+    def _listChildrenEntryIds(self, id, field_id=None):
+        """Return a children entries ids for entry 'id'.
+
+        Return a list of field_id if not None or self.id_field.
+        Available only if directory is hierarchical."""
+        if not self._isHierarchical():
+            raise ValueError(
+                "Directory [%s] is not hierarchical." % self.getId())
+        # get list of children
+        entry = self._getEntryFromLDAP(id, field_ids=[self.children_attr])
+        children = entry.get(self.children_attr, [])
+
+        # convert ids
+        field_id = field_id or self.id_field
+        if not children or field_id == self.children_id_attr:
+            return children
+        if field_id == 'dn' and self.children_id_attr == self.ldap_rdn_attr:
+            # fast rdn -> dn convertion
+            return ['%s=%s,%s' % (self.ldap_rdn_attr, child, self.ldap_base)
+                    for child in children]
+        # XXXX code to check and test
+        raise NotImplementedError('Convertion from %s to %s.' %  (
+            self.children_id_attr, field_id))
+        children_entries = [
+            self._searchEntries(return_fields=[field_id],
+                                **{self.children_id_attr: [child]})
+            for child in children]
+        return [entry[0][1].get(field_id) for entry in children_entires]
+
+    security.declarePrivate('_getParentEntryId')
+    def _getParentEntryId(self, id, field_id=None):
+        """Return Parent Id of 'id'.
+
+        Return None if 'id' have no parent.
+        Return a field_id if not None or a self.id_field.
+        Available only if directory is hierarchical."""
+        if not self._isHierarchical():
+            raise ValueError(
+                "Directory [%s] is not hierarchical." % self.getId())
+
+        field_id = field_id or self.id_field
+        # convert id to children_id
+        if self.id_field == self.children_id_attr:
+            c_id = id
+        else:
+            entry = self._getEntryFromLDAP(id,
+                                           field_ids=[self.children_id_attr])
+            c_id = entry.get(self.children_id_attr)
+        # search parent
+        parent = self._searchEntries(return_fields=[field_id],
+                                     **{self.children_attr: [c_id]})
+        parent = parent or None
+        if parent is not None:
+            # return the first available
+            parent = parent[0][1].get(field_id)
+
+        return parent
+
     #
     # Internal
     #
@@ -363,7 +468,7 @@ class LDAPBackingDirectory(BaseDirectory):
     def _getAdapterForPartialData(self, attrs):
         """Get an adapter for partial data."""
         dir = self
-        schema = self._getSchemas()[0] # XXX
+        schema = self._getUniqueSchema()
         adapter = LDAPBackingStorageAdapter(schema, None, dir, field_ids=attrs)
         return adapter
 
@@ -372,7 +477,10 @@ class LDAPBackingDirectory(BaseDirectory):
 
         Returns converted values.
         """
-        self.checkUnderBase(id)
+        try:
+            self.checkUnderBase(id)
+        except ValueError:
+            raise KeyError(id)
         filter = self.searchFilter()
         try:
             results = self.searchLDAP(id, ldap.SCOPE_BASE, filter,
@@ -403,7 +511,7 @@ class LDAPBackingDirectory(BaseDirectory):
         Returns a non-encoded LDAP filter. It will have to be UTF-8
         encoded before being passed to LDAP.
         """
-        all_field_ids = self._getSchemas()[0].keys()
+        all_field_ids = self._getFieldIds()
         filter_elems = [self.searchFilter()]
         for key, value in query.items():
             if not key in all_field_ids:
@@ -441,6 +549,7 @@ class LDAPBackingDirectory(BaseDirectory):
                 raise ValueError("Bad value %s for '%s'" % (`value`, key))
             if f:
                 filter_elems.append(f)
+        filter_elems.sort()
         filter = ''.join(filter_elems)
         if len(filter_elems) > 1:
             filter = '(&%s)' % filter
@@ -507,7 +616,7 @@ class LDAPBackingDirectory(BaseDirectory):
         Skips ignored_attrs. Keeps empty attributes if keep_empty.
         """
         ldap_attrs = {}
-        for field_id, field in self._getSchemas()[0].items(): # XXX
+        for field_id, field in self._getFieldItems():
             if field.write_ignore_storage:
                 continue
             if field_id in ('dn', 'base_dn'):
@@ -530,7 +639,7 @@ class LDAPBackingDirectory(BaseDirectory):
     def convertDataFromLDAP(self, dn, ldap_entry):
         """Convert LDAP values to a user data mapping."""
         entry = {}
-        for field_id, field in self._getSchemas()[0].items(): # XXX
+        for field_id, field in self._getFieldItems():
             if not ldap_entry.has_key(field_id):
                 continue
             values = ldap_entry[field_id]
@@ -553,13 +662,18 @@ class LDAPBackingDirectory(BaseDirectory):
                              (dn, self.ldap_base))
 
     security.declarePrivate('connectLDAP')
-    def connectLDAP(self, bind_dn=None, bind_password=None):
+    def connectLDAP(self, bind_dn=None, bind_password=None, retrying=False):
         """Get or initialize a connection to the LDAP server.
 
         If bind_dn and bind_password are provided, bind using them.
         Otherwise bind using the global bind dn and password.
         """
         conn = self._v_conn
+
+        # Saving for retrying
+        bind_dn_orig = bind_dn
+        bind_password_orig = bind_password
+
         if conn is None:
             if self.ldap_use_ssl:
                 proto = 'ldaps'
@@ -608,8 +722,14 @@ class LDAPBackingDirectory(BaseDirectory):
         try:
             conn.simple_bind_s(bind_dn, bind_password)
         except ldap.SERVER_DOWN:
-            raise ConfigurationError("Directory '%s': LDAP server is down"
-                                     % self.getId())
+            # retry with an empty connection only once
+            self._v_conn = None
+            if retrying:
+                raise ConfigurationError("Directory '%s': LDAP server is down"
+                                         % self.getId())
+            else:
+                return self.connectLDAP(bind_dn=bind_dn_orig,
+                        bind_password=bind_password_orig, retrying=True)
         except ldap.INVALID_CREDENTIALS:
             if not using_config_bind:
                 raise
@@ -642,6 +762,26 @@ class LDAPBackingDirectory(BaseDirectory):
 
         If password is provided, attempt to bind using it.
         """
+        if self.ZCacheable_isCachingEnabled():
+            attrs = list(attrs)
+            attrs.sort()
+            keyset = {
+                'base': base,
+                'scope': scope,
+                'filter': filter,
+                'attrs': attrs,
+                }
+            if password is not None:
+                keyset['password'] = password
+            LOG('searchLDAP', TRACE, "Searching cache for %s" % (keyset,))
+            ldap_entries = self.ZCacheable_get(keywords=keyset)
+            if ldap_entries is not None:
+                LOG('searchLDAP', TRACE, " -> results=%s" %
+                    (ldap_entries[:20],))
+                return ldap_entries
+        else:
+            keyset = None
+
         if password is None:
             conn = self.connectLDAP()
         else:
@@ -672,6 +812,13 @@ class LDAPBackingDirectory(BaseDirectory):
         LOG('searchLDAP', TRACE, " -> results=%s" % (ldap_entries[:20],))
         #except ldap.NO_SUCH_OBJECT:
         #except ldap.SIZELIMIT_EXCEEDED:
+        # XXX: might try to fetch partial results with ldap.async:
+        # http://python-ldap.sourceforge.net/doc/python-ldap/ldap.async-example.List.html
+
+        if keyset is not None:
+            LOG('searchLDAP', TRACE, "Putting in cache")
+            self.ZCacheable_set(ldap_entries, keywords=keyset)
+
         return ldap_entries
 
     def _insufficientAccess(self, e):
@@ -694,6 +841,7 @@ class LDAPBackingDirectory(BaseDirectory):
             conn.delete_s(dn)
         except ldap.INSUFFICIENT_ACCESS, e:
             raise self._insufficientAccess(e)
+        self.ZCacheable_invalidate()
 
     security.declarePrivate('insertLDAP')
     def insertLDAP(self, dn, ldap_attrs):
@@ -706,6 +854,7 @@ class LDAPBackingDirectory(BaseDirectory):
             conn.add_s(dn, attrs_list)
         except ldap.INSUFFICIENT_ACCESS, e:
             raise self._insufficientAccess(e)
+        self.ZCacheable_invalidate()
         # FIXME: except ldap.OBJECT_CLASS_VIOLATION:
         # {'info': "unrecognized objectClass 'evolutionPerson'", ...}
 
@@ -754,6 +903,7 @@ class LDAPBackingDirectory(BaseDirectory):
 
         LOG('modifyLDAP', TRACE, "modify_s dn=%s mod_list=%s" % (dn, mod_list))
         conn.modify_s(dn, mod_list)
+        self.ZCacheable_invalidate()
 
 
 InitializeClass(LDAPBackingDirectory)
@@ -782,7 +932,7 @@ class LDAPBackingStorageAdapter(BaseStorageAdapter):
             # Creation.
             return self.getDefaultData()
 
-        field_ids = self.getFieldIds()
+        field_ids = self.getReadableFieldIds()
         entry = self._dir._getEntryFromLDAP(id, field_ids,
                                             password=self._password)
         return self._getData(entry=entry)
