@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: LDAPDirectory.py 21960 2005-05-30 16:15:38Z atchertchian $
+# $Id: LDAPDirectory.py 31826 2006-01-19 17:40:04Z fguillaume $
 """LDAPDirectory
 """
 
@@ -34,6 +34,11 @@ from Products.CPSSchemas.BasicFields import CPSStringListField, CPSDateTimeField
 
 from Products.CPSDirectory.BaseDirectory import BaseDirectory
 
+from Products.CPSDirectory.interfaces import IDirectory
+
+from zope.interface import implements
+
+
 try:
     from Products.LDAPUserGroupsFolder.LDAPDelegate import \
          LDAPDelegate, ldap, to_utf8
@@ -61,6 +66,7 @@ class LDAPDirectory(BaseDirectory):
     - 'base_dn' is read-only except during creation where it can be used
     to specify a subbranch.
     """
+    implements(IDirectory)
 
     meta_type = 'CPS LDAP Directory'
 
@@ -203,8 +209,8 @@ class LDAPDirectory(BaseDirectory):
         - Keys with empty values are removed.
         - Keys with value '*' search for an existing field.
         """
-        schema = self._getSchemas()[0]
-        all_field_ids = schema.keys()
+        schema = self._getUniqueSchema()
+        all_field_ids = self._getFieldIds()
         # Find attrs needed to compute returned fields.
         attrsd, return_fields = self._getSearchFields(return_fields)
         attrs = attrsd.keys()
@@ -252,11 +258,6 @@ class LDAPDirectory(BaseDirectory):
             filter = '(&%s)' % filter
         return self._filteredSearchEntries(filter=filter, return_attrs=attrs)
 
-    security.declarePublic('hasEntry')
-    def hasEntry(self, id):
-        """Does the directory have a given entry?"""
-        return self._hasEntry(id)
-
     security.declarePrivate('hasEntryByDN')
     def hasEntryByDN(self, dn):
         """Does the directory have a given entry?
@@ -307,7 +308,7 @@ class LDAPDirectory(BaseDirectory):
             raise ValueError("Missing value for '%s' in entry" %
                              self.ldap_rdn_attr)
         id = entry[self.id_field]
-        if id and self.hasEntry(id):
+        if id and self._hasEntry(id):
             raise KeyError("Entry '%s' already exists" % id)
         rdn_attr = self.ldap_rdn_attr
         rdn = '%s=%s' % (rdn_attr, entry[rdn_attr])
@@ -336,10 +337,9 @@ class LDAPDirectory(BaseDirectory):
             id = rdn + ',' + base
             return id
 
-    security.declarePublic('deleteEntry')
-    def deleteEntry(self, id):
+    security.declarePrivate('_deleteEntry')
+    def _deleteEntry(self, id):
         """Delete an entry in the directory."""
-        self.checkDeleteEntryAllowed(id=id)
         user_dn = self._getLDAPEntry(id)['dn']
         msg = self._delegate.delete(dn=to_utf8(user_dn))
         if msg:
@@ -352,7 +352,7 @@ class LDAPDirectory(BaseDirectory):
     def _getAdapterForPartialData(self, attrs):
         """Get an adapter for partial data."""
         dir = self
-        schema = self._getSchemas()[0]
+        schema = self._getUniqueSchema()
         adapter = LDAPStorageAdapter(schema, None, dir, field_ids=attrs)
         return adapter
 
@@ -367,7 +367,7 @@ class LDAPDirectory(BaseDirectory):
     def _makeAttrsFromData(self, data, ignore_attrs=[], keep_empty=0):
         # Make attributes. Skip ignore_attrs.
         attrs = {}
-        for field_id, field in self._getSchemas()[0].items(): # XXX
+        for field_id, field in self._getFieldItems():
             if field.write_ignore_storage:
                 continue
             if field_id in ('dn', 'base_dn'):
