@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: MetaDirectory.py 8488 2005-04-20 21:46:46Z fguillaume $
+# $Id: MetaDirectory.py 33647 2006-03-01 15:11:18Z gracinet $
 """Meta Directory
 
 A directory redirects requests to other backing directories.
@@ -35,19 +35,29 @@ from Products.CMFCore.Expression import getEngine
 
 from Products.CPSSchemas.StorageAdapter import BaseStorageAdapter
 
+from Products.CPSDirectory.utils import QueryMatcher
+
 from Products.CPSDirectory.BaseDirectory import BaseDirectory
 from Products.CPSDirectory.BaseDirectory import AuthenticationFailed
 from Products.CPSDirectory.BaseDirectory import ConfigurationError
 
+from Products.CPSDirectory.interfaces import IMetaDirectory
+
+from zope.interface import implements
+
 
 class MetaDirectory(BaseDirectory):
     """Meta Directory
 
     A directory redirects requests to other backing directories.
     """
+    implements(IMetaDirectory)
 
     meta_type = 'CPS Meta Directory'
 
+    _properties = tuple(prop for prop in BaseDirectory._properties
+                        if prop['id'] != 'search_substring_fields')
+
     security = ClassSecurityInfo()
 
     backing_dir_infos = ()
@@ -150,7 +160,7 @@ class MetaDirectory(BaseDirectory):
         """Set the list of backing directories and their infos.
 
         Infos is a sequence of dicts with keys dir_id, id_conv,
-        field_rename, field_ignore.
+        field_rename, field_ignore, missing_entry_expr.
         """
         backing_dir_infos = []
         for info in infos:
@@ -225,12 +235,13 @@ class MetaDirectory(BaseDirectory):
         results = self._searchEntries(return_fields=[title_field])
         return [(id, entry[title_field]) for id, entry in results]
 
-    security.declarePublic('hasEntry')
-    def hasEntry(self, id):
+    security.declarePrivate('_hasEntry')
+    def _hasEntry(self, id):
         """Does the directory have a given entry?"""
+        # XXX should use base class implementation
         for info in self.getBackingDirectories():
             if info['missing_entry'] is None:
-                return info['dir'].hasEntry(id)
+                return info['dir']._hasEntry(id)
         return 0
 
     security.declarePublic('isAuthenticating')
@@ -261,7 +272,7 @@ class MetaDirectory(BaseDirectory):
     def _createEntry(self, entry):
         """Create an entry in the directory."""
         id = entry[self.id_field]
-        if self.hasEntry(id):
+        if self._hasEntry(id):
             raise KeyError("Entry '%s' already exists" % id)
 
         # Fetch a dm with default values
@@ -277,18 +288,17 @@ class MetaDirectory(BaseDirectory):
             adapter.setContextObject(id)
         dm._commit()
 
-    security.declarePublic('deleteEntry')
-    def deleteEntry(self, id):
+    security.declarePrivate('_deleteEntry')
+    def _deleteEntry(self, id):
         """Delete an entry in the directory."""
-        self.checkDeleteEntryAllowed(id=id)
-        if not self.hasEntry(id):
+        if not self._hasEntry(id):
             raise KeyError("Entry '%s' does not exist" % id)
         for info in self.getBackingDirectories():
             b_dir = info['dir']
             # Get id XXX maybe convert
             b_id = id
             try:
-                b_dir.deleteEntry(b_id)
+                b_dir._deleteEntry(b_id)
             except KeyError:
                 if info['missing_entry'] is None:
                     raise
@@ -307,8 +317,7 @@ class MetaDirectory(BaseDirectory):
 
         #print 'dir=%s rf=%s query=%s' % (self.id, return_fields, kw)
 
-        schema = self._getSchemas()[0]
-        all_field_ids = schema.keys()
+        all_field_ids = self._getFieldIds()
         id_field = self.id_field
 
         # Compute query.
@@ -326,7 +335,6 @@ class MetaDirectory(BaseDirectory):
         for info in self.getBackingDirectories():
             b_dir = info['dir']
             field_ignore = info['field_ignore']
-            field_rename = info['field_rename']
             field_rename_back = info['field_rename_back']
             b_field_ids = b_dir._getFieldIds()
             b_id_field = b_dir.id_field
@@ -337,6 +345,15 @@ class MetaDirectory(BaseDirectory):
                 b_key = field_rename_back.get(key, key)
                 if b_key in b_field_ids:
                     b_query[b_key] = value
+                    
+            # checks if there's a missing entry and it matches the query
+            missing_entry = info.get('missing_entry')
+            if missing_entry is not None:
+                matcher = QueryMatcher(b_query, accepted_keys=b_field_ids,
+                                   substring_keys=b_dir.search_substring_fields)
+                b_matched = matcher.match(missing_entry)
+            else:
+                b_matched = False
 
             # Get return fields
             if return_fields is None:
@@ -363,40 +380,52 @@ class MetaDirectory(BaseDirectory):
                         continue
                     b_return_fields.append(b_fid)
 
-            b_queries.append((info, b_return_fields, b_query))
+            b_queries.append((info, b_return_fields, b_query, b_matched))
 
         # Find in what order to do the searches.
-        # Search backing dirs with a missing_entry last so that we can intersect
-        # with a fixed base.
+        # criterium: (normal < empty queries < matching missing_entry)
+
+        # Search backing dirs with a matching missing_entry arelast
+        # so that we can intersect with a fixed base.
+
+        # Search backing dirs with empty queries are after normal ones, so
+        # that the filtering algorithm will see that they aren't needed
+        # to get a base list of entries to start the intersection process
         qs = []
         for qt in b_queries:
-            info, b_return_fields, b_query = qt
-            t = (int(info['missing_entry'] is not None), qt)
+            info, b_return_fields, b_query, b_matched = qt
+            if b_matched:
+                order = 2
+            elif not b_query:
+                order = 1
+            else:
+                order = 0
+            t = (order, qt)
             qs.append(t)
         qs.sort()
         b_queries = [t[1] for t in qs]
 
-        # Remove useless queries.
+        # Remove useless queries:
         qs = []
-        has_b_with_all_entries = 0
+        base_list_provides = False
         for qt in b_queries:
-            info, b_return_fields, b_query = qt
-            if not b_return_fields and not b_query and has_b_with_all_entries:
+            info, b_return_fields, b_query, b_matched = qt
+            if not b_return_fields and not b_query and base_list_provides:
                 # This query gets us no info and returns everything
-                # We must not skip it though if there would be no normal backing
-                # directories left to get a base list of entries.
+                # We must not skip it though if it's the first one to provide
+                # a base list of entries, see comment above about the ordering
                 #print ' ignoring query on %s' % info['dir_id']
                 continue
-            if info['missing_entry'] is None:
-                has_b_with_all_entries = 1
+            base_list_provides = base_list_provides or (not b_matched)
             qs.append(qt)
         b_queries = qs
 
         # Do searches
         acc_res = None
-        for info, b_return_fields, b_query in b_queries:
+        for info, b_return_fields, b_query, b_matched in b_queries:
             b_dir = info['dir']
             missing_entry = info['missing_entry']
+            b_field_rename = info['field_rename']
             has_missing_entries = missing_entry is not None
 
             # Do query
@@ -414,7 +443,7 @@ class MetaDirectory(BaseDirectory):
                 for id, b_entry in b_res:
                     entry = {}
                     for b_fid, value in b_entry.items():
-                        fid = field_rename.get(b_fid, b_fid)
+                        fid = b_field_rename.get(b_fid, b_fid)
                         entry[fid] = value
                     res.append((id, entry))
 
@@ -423,38 +452,39 @@ class MetaDirectory(BaseDirectory):
             if acc_res is None:
                 acc_res = res
             else:
-                if not has_missing_entries:
-                    # If we don't care about ordering, do intelligent intersection
+                if not b_matched:
+                    # We don't care about ordering, let's prepare for
+                    # quickest intersection by making acc_res the smaller one
                     if len(acc_res) > len(res):
-                        # Make acc_res be always the smaller one
                         acc_res, res = res, acc_res
+                
                 resids_d = {}
                 if return_fields is None:
-                    if has_missing_entries:
-                        # Keep all the ids we already have
-                        pass
+                    res_set = set(res)
+                    if b_matched:
+                        # previous results that matched the query or aren't in
+                        # current backing 
+                        acc_res = [id for id in acc_res
+                                   if id in res_set or not b_dir._hasEntry(id)]
                     else:
-                        # Intersect
-                        for id in res:
-                            resids_d[id] = None
-                        acc_res = [id for id in acc_res if resids_d.has_key(id)]
+                        acc_res = [id for id in acc_res if id in res_set]
                 else:
                     # Intersect and merge values for matching
-                    for id, d in res:
-                        resids_d[id] = d
-                    old_res = acc_res
-                    acc_res = []
-                    for id, old_d in old_res:
-                        new_d = resids_d.get(id)
-                        if new_d is None:
-                            if has_missing_entries:
-                                new_d = missing_entry
-                            else:
-                                # Not present in both sets.
-                                continue
-                        old_d.update(new_d)
-                        acc_res.append((id, old_d))
-            #print ' now acc_res=%s' % `acc_res`
+                    res_d = dict(res)
+                    if b_matched:
+                        # previous results that matched the query or aren't in
+                        # current backing
+                        acc_res = [(id,d) \
+                                    for (id,d) in acc_res
+                                    if id in res_d or not b_dir._hasEntry(id)]
+                        for (id,d) in acc_res:
+                            d.update(res_d.get(id, missing_entry))
+                    else:
+                        acc_res = [(id,d)
+                                   for (id,d) in acc_res
+                                   if id in res_d]
+                        for (id,d) in acc_res:
+                            d.update(res_d[id])
 
         if acc_res is None:
             # No directories entries contributed at all...
@@ -475,6 +505,49 @@ class MetaDirectory(BaseDirectory):
         #LOG('searchEntries', DEBUG, 'rf=%s idf=%s sidf=%s res=%s' % (return_fields, id_field, self.id_field, acc_res))
         return acc_res
 
+
+    #
+    # Hierarchical support
+    #
+    security.declarePrivate('_isHierarchical')
+    def _isHierarchical(self):
+        """Return true if one of the backing directory is hierarchical."""
+        for info in self.getBackingDirectories():
+            if info['missing_entry'] is None:
+                if info['dir']._isHierarchical():
+                    return True
+        return False
+
+    security.declarePrivate('_listChildrenEntryIds')
+    def _listChildrenEntryIds(self, id, field_id=None):
+        """Return a children entries ids for entry 'id'.
+
+        Return a list of field_id if not None or self.id_field.
+        Use the first hierarchical backing directory."""
+        field_id = field_id or self.id_field
+        for info in self.getBackingDirectories():
+            if info['missing_entry'] is None:
+                if info['dir']._isHierarchical():
+                    return info['dir']._listChildrenEntryIds(id, field_id)
+        raise ValueError(
+            "No Hierarchical backing directory for [%s] found." % self.getId())
+
+
+    security.declarePrivate('_getParentEntryId')
+    def _getParentEntryId(self, id):
+        """Return Parent Id of 'id'.
+
+        Return None if 'id' have no parent.
+        Return a field_id if not None or a self.id_field.
+        Use the first hierarchical backing directory."""
+        field_id = field_id or self.id_field
+        for info in self.getBackingDirectories():
+            if info['missing_entry'] is None:
+                if info['dir']._isHierarchical():
+                    return info['dir']._getParentEntryId(id, field_id)
+        raise ValueError(
+            "No Hierarchical backing directory for [%s] found." % self.getId())
+
     #
     # Internal
     #
