@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: SQLDirectory.py 21960 2005-05-30 16:15:38Z atchertchian $
+# $Id: SQLDirectory.py 32431 2006-01-31 00:05:21Z fguillaume $
 """SQLDirectory
 
 This is a directory backed by a table in an SQL database.
@@ -28,27 +28,39 @@ from Acquisition import aq_base, aq_parent, aq_inner
 from Globals import InitializeClass
 from AccessControl import ClassSecurityInfo
 from DateTime.DateTime import DateTime
+from OFS.Cache import Cacheable
 
 from Products.CPSSchemas.StorageAdapter import BaseStorageAdapter
 from Products.CPSDirectory.BaseDirectory import BaseDirectory
 from Products.CPSDirectory.BaseDirectory import ConfigurationError
 from Products.CPSDirectory.BaseDirectory import AuthenticationFailed
 
+from Products.CPSDirectory.interfaces import IDirectory
+
+from zope.interface import implements
+
 
 class SQLSyntaxError(ValueError):
     """SQL syntax error exception."""
     pass
 
 
-class SQLDirectory(BaseDirectory):
+class SQLDirectory(BaseDirectory, Cacheable):
     """SQL Directory.
 
     A directory that connects to an SQL database.
     """
+    implements(IDirectory)
+
     # XXX what about tables where the id is not a string, like an int ?
 
     meta_type = 'CPS SQL Directory'
 
+    manage_options = (
+        BaseDirectory.manage_options +
+        Cacheable.manage_options
+        )
+
     security = ClassSecurityInfo()
 
     _properties = BaseDirectory._properties + (
@@ -90,10 +102,10 @@ class SQLDirectory(BaseDirectory):
         paths.sort()
         return paths
 
-    #def _postProcessProperties(self):
-    #    """Post-processing after properties change."""
-    #    BaseDirectory._postProcessProperties(self)
-    #    # XXX sql_auxiliary_tables
+    def _postProcessProperties(self):
+        """Post-processing after properties change."""
+        BaseDirectory._postProcessProperties(self)
+        self.ZCacheable_invalidate()
 
     security.declarePrivate('_getAdapters')
     def _getAdapters(self, id, search=0, **kw):
@@ -135,45 +147,6 @@ class SQLDirectory(BaseDirectory):
     # API
     #
 
-    security.declarePrivate('listEntryIds')
-    def listEntryIds(self):
-        """List all the entry ids."""
-        sql = "SELECT %(id)s FROM %(table)s" % {
-            'id': self.getSQLField(self.id_field),
-            'table': self.sql_table,
-            }
-        items, data = self._execute(sql)
-        res = [t[0] for t in data]
-        return res
-
-    security.declarePrivate('listEntryIdsAndTitles')
-    def listEntryIdsAndTitles(self):
-        """List all the entry ids and titles."""
-        id_field = self.id_field
-        title_field = self.title_field
-        if title_field == id_field:
-            ids = self.listEntryIds()
-            return [(id, id) for id in ids]
-        sql = "SELECT %(id)s, %(title)s FROM %(table)s" % {
-            'id': self.getSQLField(id_field),
-            'title': self.getSQLField(title_field),
-            'table': self.sql_table,
-            }
-        items, data = self._execute(sql)
-        return data
-
-    security.declarePublic('hasEntry')
-    def hasEntry(self, id):
-        """Does the directory have a given entry?"""
-        sql = ("SELECT COUNT(%(idf)s) FROM %(table)s"
-               " WHERE %(idf)s = %(id)s") % {
-            'idf': self.getSQLField(self.id_field),
-            'table': self.sql_table,
-            'id': self.getSQLValue(id),
-            }
-        items, data = self._execute(sql)
-        return not not data[0][0]
-
     security.declarePublic('isAuthenticating')
     def isAuthenticating(self):
         """Check if this directory does authentication.
@@ -218,7 +191,7 @@ class SQLDirectory(BaseDirectory):
     def _createEntry(self, entry):
         """Create an entry in the directory."""
         id = entry[self.id_field]
-        if self.hasEntry(id):
+        if self._hasEntry(id):
             raise KeyError("Entry %s already exists" % `id`)
 
         sql_data = self._convertDataToQuotedSQL(entry)
@@ -233,11 +206,10 @@ class SQLDirectory(BaseDirectory):
         # XXX depends on SQL dialect
         return None
 
-    security.declarePublic('deleteEntry')
-    def deleteEntry(self, id):
+    security.declarePrivate('_deleteEntry')
+    def _deleteEntry(self, id):
         """Delete an entry in the directory."""
-        self.checkDeleteEntryAllowed(id=id)
-        if not self.hasEntry(id):
+        if not self._hasEntry(id):
             raise KeyError("No entry %s" % `id`)
         sql = "DELETE FROM %(table)s WHERE %(idf)s = %(id)s" % {
             'table': self.sql_table,
@@ -252,8 +224,8 @@ class SQLDirectory(BaseDirectory):
 
         See API in the base class.
         """
-        schema = self._getSchemas()[0]
-        all_field_ids = schema.keys()
+        schema = self._getUniqueSchema()
+        all_field_ids = self._getFieldIds()
 
         # Find field_ids needed to compute returned fields.
         attrsd, return_fields = self._getSearchFields(return_fields)
@@ -282,13 +254,16 @@ class SQLDirectory(BaseDirectory):
         res = []
         idix = field_ids.index(self.id_field)
         for result in data:
-            result = list(result)
             id = result[idix]
-            entry = {}
-            for field_id in field_ids:
-                entry[field_id] = result.pop(0)
-                # XXX conversions !
-            res.append((id, entry))
+            if return_fields is None:
+                res.append(id)
+            else:
+                result = list(result)
+                entry = {}
+                for field_id in field_ids:
+                    entry[field_id] = result.pop(0)
+                    # XXX conversions !
+                res.append((id, entry))
 
         # Now we must compute a partial datamodel for each result,
         # to get correct computed fields.
@@ -339,9 +314,21 @@ class SQLDirectory(BaseDirectory):
 
         Returns a tuple (items, data) or raises an exception.
         """
-        LOG('_execute', TRACE, "Execute:\n  %s" % sql)
+        keyset = None
+        if self.ZCacheable_isCachingEnabled():
+            if sql.startswith("SELECT"):
+                keyset = {'query': sql}
+                LOG('_execute', TRACE, "Searching cache for %s" % (keyset,))
+                res = self.ZCacheable_get(keywords=keyset)
+                if res is not None:
+                    LOG('_execute', TRACE, " -> results=%s" % (res[:20],))
+                    return res
+            else:
+                self.ZCacheable_invalidate()
+
         conn = self._getConnection()
         try:
+            LOG('_execute', TRACE, "Execute:\n  %s" % sql)
             res = conn.query(sql)
         except SyntaxError, e:
             # Gadlfy: invalid syntax
@@ -358,6 +345,11 @@ class SQLDirectory(BaseDirectory):
             raise
 
         LOG('_execute', TRACE, "Result:\n%s" % `res`)
+
+        if keyset is not None:
+            LOG('_execute', TRACE, "Putting in cache")
+            self.ZCacheable_set(res, keywords=keyset)
+
         return res
 
     def _getEntryFromSQL(self, id, field_ids, password=None):
@@ -408,7 +400,7 @@ class SQLDirectory(BaseDirectory):
         """
         quoter = self._getSQLQuoter()
         sql_data = {}
-        for field_id, field in self._getSchemas()[0].items(): # XXX
+        for field_id, field in self._getFieldItems():
             if field.write_ignore_storage:
                 continue
             if not data.has_key(field_id):
@@ -496,7 +488,7 @@ class SQLStorageAdapter(BaseStorageAdapter):
             # Creation.
             return self.getDefaultData()
 
-        field_ids = self.getFieldIds()
+        field_ids = self.getReadableFieldIds()
         entry = self._dir._getEntryFromSQL(id, field_ids,
                                            password=self._password)
         return self._getData(entry=entry)
