@@ -32,9 +32,7 @@ INSUFFICIENT_ACCESS = 'INSUFFICIENT_ACCESS'
 SCOPE_BASE = 0
 SCOPE_ONELEVEL = 1
 SCOPE_SUBTREE = 2
-MOD_ADD = 0
-MOD_DELETE = 1
-MOD_REPLACE = 2
+from fakeldap import MOD_ADD, MOD_DELETE, MOD_REPLACE
 OPT_REFERRALS = 8
 OPT_TIMELIMIT = 4
 OPT_SIZELIMIT = 3
