@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: testMetaDirectory.py 21960 2005-05-30 16:15:38Z atchertchian $
+# $Id: testMetaDirectory.py 32222 2006-01-25 10:47:58Z gracinet $
 
 
 import os, sys
@@ -23,61 +23,114 @@ if __name__ == '__main__':
     execfile(os.path.join(sys.path[0], 'framework.py'))
 
 import unittest
-from Testing import ZopeTestCase
-from CPSDirectoryTestCase import CPSDirectoryTestCase
+from Testing.ZopeTestCase import ZopeTestCase
 from AccessControl import Unauthorized
+from OFS.Folder import Folder
 
-class TestMetaDirectory(CPSDirectoryTestCase):
+from Products.CPSDirectory.tests.fakeCps import FakeField
+from Products.CPSDirectory.tests.fakeCps import FakeSchema
+from Products.CPSDirectory.tests.fakeCps import FakeSchemasTool
+from Products.CPSDirectory.tests.fakeCps import FakeDirectoryTool
+from Products.CPSDirectory.tests.fakeCps import FakeRoot
 
-    def afterSetUp(self):
-        CPSDirectoryTestCase.afterSetUp(self)
-        self.makeDirs()
+
+class TestMetaDirectory(ZopeTestCase):
+
+    def makeSite(self):
+        self.root = FakeRoot()
+        self.root.portal = Folder('portal')
+        utool = Folder('portal_url')
+        utool.getPortalObject = lambda : self.root.portal
+        self.root.portal.portal_url = utool
+        self.root.portal.portal_schemas = FakeSchemasTool()
+        self.root.portal.portal_directories = FakeDirectoryTool()
+        self.portal = self.root.portal
+        self.pd = self.portal.portal_directories
+
+    def makeSchemas(self):
+        stool = self.portal.portal_schemas
+        s = FakeSchema({
+            'idd': FakeField(),
+            'foo': FakeField(),
+            'pasglop': FakeField(),
+            })
+        stool._setObject('sfoo', s)
+        s = FakeSchema({
+            'id': FakeField(),
+            'bar': FakeField(),
+            'mail': FakeField(),
+            })
+        stool._setObject('sbar', s)
+        s = FakeSchema({
+            'id': FakeField(),
+            'foo': FakeField(),
+            'bar': FakeField(),
+            'email': FakeField(),
+            })
+        stool._setObject('smeta', s)
 
     def makeDirs(self):
-        stool = self.portal.portal_schemas
-        schema = stool.manage_addCPSSchema('sfoo')
-        schema.manage_addField('idd', 'CPS String Field') # different id
-        schema.manage_addField('foo', 'CPS String Field') # kept
-        schema.manage_addField('pasglop', 'CPS String Field') # ignored
+        from Products.CPSDirectory.ZODBDirectory import ZODBDirectory
+        from Products.CPSDirectory.MetaDirectory import MetaDirectory
+        class LoggerZODBDirectory(ZODBDirectory):
+            def _searchEntries(self, *args, **kwargs):
+                self.tests_called_search = True
+                return ZODBDirectory._searchEntries(self, *args, **kwargs)
 
-        schema = stool.manage_addCPSSchema('sbar')
-        schema.manage_addField('id', 'CPS String Field') # same id
-        schema.manage_addField('bar', 'CPS String Field') # kept
-        schema.manage_addField('mail', 'CPS String Field') # renamed
+        dtool = self.portal.portal_directories
+        dirfoo = ZODBDirectory('dirfoo', schema='sfoo', id_field='idd',
+                acl_directory_view_roles='test_role_1_',
+                acl_entry_create_roles='test_role_1_',
+                acl_entry_delete_roles='test_role_1_',
+                acl_entry_view_roles='test_role_1_',
+                acl_entry_edit_roles='test_role_1_',
+                )
+        dtool._setObject(dirfoo.getId(), dirfoo)
 
-        schema = stool.manage_addCPSSchema('smeta')
-        schema.manage_addField('id', 'CPS String Field')
-        schema.manage_addField('foo', 'CPS String Field')
-        schema.manage_addField('bar', 'CPS String Field')
-        schema.manage_addField('email', 'CPS String Field')
+        dirbar = LoggerZODBDirectory('dirbar', schema='sbar', id_field='id',
+                acl_directory_view_roles='test_role_1_',
+                acl_entry_create_roles='test_role_1_',
+                acl_entry_delete_roles='test_role_1_',
+                acl_entry_view_roles='test_role_1_',
+                acl_entry_edit_roles='test_role_1_',
+                )
+        dtool._setObject(dirbar.getId(), dirbar)
 
-        dirfoo = self.pd.manage_addCPSDirectory('dirfoo',
-                                                'CPS ZODB Directory')
-        dirfoo.manage_changeProperties(schema='sfoo', id_field='idd')
+        dirmeta = MetaDirectory('dirmeta', schema='smeta', id_field='id',
+                title_field='bar',
+                acl_directory_view_roles='test_role_1_',
+                acl_entry_create_roles='test_role_1_',
+                acl_entry_delete_roles='test_role_1_',
+                acl_entry_view_roles='test_role_1_',
+                acl_entry_edit_roles='test_role_1_',
+                )
+        dtool._setObject(dirmeta.getId(), dirmeta)
 
-        dirbar = self.pd.manage_addCPSDirectory('dirbar',
-                                                'CPS ZODB Directory')
-        dirbar.manage_changeProperties(schema='sbar', id_field='id')
-
-        dirmeta = self.pd.manage_addCPSDirectory('dirmeta',
-                                                 'CPS Meta Directory')
-        dirmeta.manage_changeProperties(schema='smeta', id_field='id',
-                                        title_field='bar')
-        dirmeta.setBackingDirectories(
+        dtool.dirmeta.setBackingDirectories(
             ({'dir_id': 'dirfoo',
-              'id_conv': None,
               'field_ignore': ('pasglop',),
               },
              {'dir_id': 'dirbar',
-              'id_conv': None,
-              'field_rename': {'mail': 'email'}, # back:meta
-              },
+              'field_rename': {'mail': 'email'}, # convention is back:meta
+                 },
              ))
 
-        self.dirfoo = dirfoo
-        self.dirbar = dirbar
-        self.dirmeta = dirmeta
+        self.dirfoo = dtool.dirfoo
+        self.dirbar = dtool.dirbar
+        self.dirmeta = dtool.dirmeta
 
+    def afterSetUp(self):
+        ZopeTestCase.afterSetUp(self)
+        self.makeSite()
+        self.makeSchemas()
+        self.makeDirs()
+
+class TestMetaDirectoryNoMissing(TestMetaDirectory):
+
+    def test_properties(self):
+        props = self.dirmeta.propertyIds()
+        self.assert_(props)
+        self.failIf('search_substring_fields' in props)
 
     def test_getEntry(self):
         id = '000'
@@ -270,31 +323,112 @@ class TestMetaDirectory(CPSDirectoryTestCase):
         res.sort()
         self.assertEquals(res, [('DDD', {}), ('EEE', {})])
 
-    def xxxtestSearchSubstrings(self):
-        dir = self.dir
+    def test_searchEntries_renames_1266(self):
+        # use-case of bug #1266: renames couldn't work in _searchEntries
+        # for two dirs at once
+        dir = self.dirmeta
+        dir.setBackingDirectories(
+            ({'dir_id': 'dirfoo',
+              'field_rename': {'pasglop': 'glop',}, # convention is back:meta
+              },
+             {'dir_id': 'dirbar',
+              'field_rename': {'mail': 'email'}, 
+                 },
+             ))
+
+        fooentry = {'idd': 'DDD', 'foo': 'ouah', 'pasglop': 'arg'}
+        barentry = {'id': 'DDD', 'bar': 'brr', 'mail': 'me@here'}
+        self.dirfoo.createEntry(fooentry)
+        self.dirbar.createEntry(barentry)
+        res = dir.searchEntries(id='DDD', return_fields=['glop', 'email', ])
+        self.assertEquals(res, [('DDD', {'glop' : 'arg', 'email': 'me@here'})])
+        
+    def test_searchEntries_renames_notinj(self):
+        # two different ids in the meta but same field ids in backings
+        # XXX an entire test case for this setting would be more appropriate
+
+        dtool = self.portal.portal_directories
+        stool = self.portal.portal_schemas
+
+        # we need a new backing        
+        from Products.CPSDirectory.ZODBDirectory import ZODBDirectory
+        from Products.CPSDirectory.MetaDirectory import MetaDirectory
+        dirbabar = ZODBDirectory('dirbabar', schema='sbar', id_field='id',
+                acl_directory_view_roles='test_role_1_',
+                acl_entry_create_roles='test_role_1_',
+                acl_entry_delete_roles='test_role_1_',
+                acl_entry_view_roles='test_role_1_',
+                acl_entry_edit_roles='test_role_1_',
+                )
+        dtool._setObject(dirbabar.getId(), dirbabar)
+        self.dirbabar = dtool.dirbabar
+
+        s = FakeSchema({
+            'id': FakeField(),
+            'foo': FakeField(),
+            'bar': FakeField(),
+            'email': FakeField(),
+            'email_babar' : FakeField(),
+            })
+        stool._setObject('smeta_babar', s)
+
+        dirmeta_babar = MetaDirectory('dirmeta_babar',
+                                     schema='smeta_babar', id_field='id',
+                title_field='bar',
+                acl_directory_view_roles='test_role_1_',
+                acl_entry_create_roles='test_role_1_',
+                acl_entry_delete_roles='test_role_1_',
+                acl_entry_view_roles='test_role_1_',
+                acl_entry_edit_roles='test_role_1_',
+                )
+        dtool._setObject(dirmeta_babar.getId(), dirmeta_babar)
+
+        dir = dtool.dirmeta_babar
+        dir.setBackingDirectories( 
+            ({'dir_id': 'dirbar',
+              'field_rename': {'mail': 'email'}, # convention is back:meta
+                 },
+             {'dir_id': 'dirbabar',
+              'field_rename': {'mail': 'email_babar'}, 
+                 },
+             ))
+
+        barentry = {'id': 'DDD', 'bar': 'brr', 'mail': 'me@here'}
+        babarentry = {'id': 'DDD', 'bar': 'sss', 'mail': 'babar@here'}
+        self.dirbar.createEntry(barentry)
+        self.dirbabar.createEntry(babarentry)
+        res = dir.searchEntries(id='DDD',
+                                return_fields=['email', 'email_babar'])
+        self.assertEquals(res, [('DDD', {'email_babar' : 'babar@here',
+                                         'email': 'me@here'})])
+
+    def testSearchSubstrings(self):
+        # most of this test is a copy/paste from the one in ZODB Directory
+        dir = self.dirmeta
+
+        ### allow substrings in backings
+        self.dirbar.search_substring_fields = ['bar']
+        self.dirfoo.search_substring_fields = ['foo']
 
         id1 = 'tree'
         foo1 = 'green'
         bar1 = ['a123', 'gra']
-        e1 = {'idd': id1, 'foo': foo1, 'bar': bar1}
+        e1 = {'id': id1, 'foo': foo1, 'bar': bar1}
         dir.createEntry(e1)
 
         id2 = 'sea'
         foo2 = 'blue'
         bar2 = ['812A', 'gra']
-        e2 = {'idd': id2, 'foo': foo2, 'bar': bar2}
+        e2 = {'id': id2, 'foo': foo2, 'bar': bar2}
         dir.createEntry(e2)
 
         ids = [id1, id2]
         ids.sort()
 
-        ### With substrings
-        dir.search_substring_fields = ['foo', 'bar']
-
         # Basic searches
-        res = dir.searchEntries(idd=id1)
+        res = dir.searchEntries(id=id1)
         self.assertEquals(res, [id1])
-        res = dir.searchEntries(idd=[id1])
+        res = dir.searchEntries(id=[id1])
         self.assertEquals(res, [id1])
         res = dir.searchEntries(foo=foo1)
         self.assertEquals(res, [id1])
@@ -318,7 +452,7 @@ class TestMetaDirectory(CPSDirectoryTestCase):
         self.assertEquals(res, [id1])
 
         # Multi-field searches
-        res = dir.searchEntries(idd=id1, foo=[foo1], bar='gra')
+        res = dir.searchEntries(id=id1, foo=[foo1], bar='gra')
         self.assertEquals(res, [id1])
         res = dir.searchEntries(foo=foo2, bar='gra')
         self.assertEquals(res, [id2])
@@ -328,9 +462,9 @@ class TestMetaDirectory(CPSDirectoryTestCase):
         self.assertEquals(res, [id1])
 
         # Substring searches
-        res = dir.searchEntries(idd='re')
+        res = dir.searchEntries(id='re')
         self.assertEquals(res, [])
-        res = dir.searchEntries(idd='TREE')
+        res = dir.searchEntries(id='TREE')
         self.assertEquals(res, [])
         res = dir.searchEntries(foo='e')
         self.assertEquals(res, ids)
@@ -343,43 +477,11 @@ class TestMetaDirectory(CPSDirectoryTestCase):
         res = dir.searchEntries(foo='E', bar='12')
         self.assertEquals(res, ids)
 
-class TestMetaDirectoryMissing(CPSDirectoryTestCase):
+class TestMetaDirectoryMissing(TestMetaDirectory):
 
     def afterSetUp(self):
-        CPSDirectoryTestCase.afterSetUp(self)
-        self.makeDirs()
-
-    def makeDirs(self):
-        stool = self.portal.portal_schemas
-        schema = stool.manage_addCPSSchema('sfoo')
-        schema.manage_addField('idd', 'CPS String Field') # different id
-        schema.manage_addField('foo', 'CPS String Field') # kept
-        schema.manage_addField('pasglop', 'CPS String Field') # ignored
-
-        schema = stool.manage_addCPSSchema('sbar')
-        schema.manage_addField('id', 'CPS String Field') # same id
-        schema.manage_addField('bar', 'CPS String Field') # kept
-        schema.manage_addField('mail', 'CPS String Field') # renamed
-
-        schema = stool.manage_addCPSSchema('smeta')
-        schema.manage_addField('id', 'CPS String Field')
-        schema.manage_addField('foo', 'CPS String Field')
-        schema.manage_addField('bar', 'CPS String Field')
-        schema.manage_addField('email', 'CPS String Field')
-
-        dirfoo = self.pd.manage_addCPSDirectory('dirfoo',
-                                                'CPS ZODB Directory')
-        dirfoo.manage_changeProperties(schema='sfoo', id_field='idd')
-
-        dirbar = self.pd.manage_addCPSDirectory('dirbar',
-                                                'CPS ZODB Directory')
-        dirbar.manage_changeProperties(schema='sbar', id_field='id')
-
-        dirmeta = self.pd.manage_addCPSDirectory('dirmeta',
-                                                 'CPS Meta Directory')
-        dirmeta.manage_changeProperties(schema='smeta', id_field='id',
-                                        title_field='bar')
-        dirmeta.setBackingDirectories(
+        TestMetaDirectory.afterSetUp(self)
+        self.dirmeta.setBackingDirectories(
             ({'dir_id': 'dirfoo',
               'field_ignore': ('pasglop',),
               'missing_entry_expr': "python:{'foo': 'defaultfoo'}"
@@ -389,11 +491,6 @@ class TestMetaDirectoryMissing(CPSDirectoryTestCase):
               },
              ))
 
-        self.dirfoo = dirfoo
-        self.dirbar = dirbar
-        self.dirmeta = dirmeta
-
-
     def test_getEntry(self):
         id = 'LDHL'
         self.assertRaises(KeyError, self.dirmeta.getEntry, id)
@@ -488,11 +585,7 @@ class TestMetaDirectoryMissing(CPSDirectoryTestCase):
         self.assertEquals(res, [('DDD', {'foo': 'ouah'}),
                                 ('EEE', {'foo': 'defaultfoo'})])
 
-    # XXX must have test with a search on a field from a missing entry
-
-    def XXXXXXXXXXXtest_searchEntries(self):
-        # TODO FIXME: the code doesn't treat the case where we
-        # do a search on an attribute that's in the missing entry.
+    def test_searchEntries(self):
         dir = self.dirmeta
 
         entry1 = {'id': 'AAA', 'foo': 'oof', 'bar': 'rab', 'email': 'lame@at'}
@@ -501,23 +594,49 @@ class TestMetaDirectoryMissing(CPSDirectoryTestCase):
         dir.createEntry(entry2)
         entry3 = {'id': 'CCC', 'foo': 'oo', 'bar': 'rab', 'email': 'yo@mama'}
         dir.createEntry(entry3)
+        barentry = {'id': 'DDD', 'bar': 'baronly', 'mail' : 'brian@spam'}
+        self.dirbar.createEntry(barentry)
+        self.failIf(self.dirfoo._hasEntry('DDD'))
 
         # Simple
         ids = dir.searchEntries()
         ids.sort()
-        self.assertEquals(ids, ['AAA', 'BBB', 'CCC'])
+        self.assertEquals(ids, ['AAA', 'BBB', 'CCC', 'DDD'])
 
         ids = dir.searchEntries(id='AAA')
         self.assertEquals(ids, ['AAA'])
+        
         ids = dir.searchEntries(foo='oo')
         ids.sort()
         self.assertEquals(ids, ['BBB', 'CCC'])
+
+        ids = dir.searchEntries(bar='baronly', foo='defaultfoo')
+        ids.sort()
+        self.assertEquals(ids, ['DDD'])
+        
+        # used to produce empty search in dirbar. Since query cannot match
+        # dirfoo's missing entry. search in dirbar shouldn't be done at all
+        # (see https://svn.nuxeo.org/trac/pub/ticket/995)
+        self.dirbar.tests_called_search = False
+        ids = dir.searchEntries(foo='oo')
+        ids.sort()
+        self.assertEquals(ids, ['BBB', 'CCC'])
+        self.failIf(self.dirbar.tests_called_search)
+        self.dirbar.tests_called_search = True
+        
+        ids = dir.searchEntries(foo='oo')
+        ids.sort()
+        self.assertEquals(ids, ['BBB', 'CCC'])
+        
         ids = dir.searchEntries(bar='rab')
         ids.sort()
         self.assertEquals(ids, ['AAA', 'CCC'])
+        
+        # unknown field: equivalent to an empty search
         ids = dir.searchEntries(hooooole='ohle')
         ids.sort()
-        self.assertEquals(ids, ['AAA', 'BBB', 'CCC'])
+        self.assertEquals(ids, ['AAA', 'BBB', 'CCC', 'DDD'])
+        
         ids = dir.searchEntries(email='evil@hell')
         self.assertEquals(ids, ['BBB'])
 
@@ -526,26 +645,39 @@ class TestMetaDirectoryMissing(CPSDirectoryTestCase):
         res.sort()
         self.assertEquals(res, [('AAA', {'email': 'lame@at'}),
                                 ('BBB', {'email': 'evil@hell'}),
-                                ('CCC', {'email': 'yo@mama'})])
+                                ('CCC', {'email': 'yo@mama'}),
+                                ('DDD', {'email': 'brian@spam'}),
+                                ])
+
         res = dir.searchEntries(foo='oo', return_fields=['email', 'pasglop'])
         res.sort()
         self.assertEquals(res, [('BBB', {'email': 'evil@hell'}),
                                 ('CCC', {'email': 'yo@mama'})])
+
         res = dir.searchEntries(email='lame@at', return_fields=['foo'])
         self.assertEquals(res, [('AAA', {'foo': 'oof'})])
 
+        res = dir.searchEntries(email='brian@spam', return_fields=['foo'])
+        self.assertEquals(res, [('DDD', {'foo': 'defaultfoo'})])
+
         res = dir.searchEntries(foo='oof', return_fields=['id'])
         self.assertEquals(res, [('AAA', {'id': 'AAA'})])
 
         # Not strictly defined but test it anyway
         res = dir.searchEntries(return_fields=['blort'])
         res.sort()
-        self.assertEquals(res, [('AAA', {}), ('BBB', {}), ('CCC', {})])
+        self.assertEquals(res, [('AAA', {}), ('BBB', {}),
+                                ('CCC', {}),('DDD', {}),
+                                ])
+
         res = dir.searchEntries(return_fields=['id'])
         res.sort()
+
         self.assertEquals(res, [('AAA', {'id':'AAA'}),
                                 ('BBB', {'id':'BBB'}),
-                                ('CCC', {'id':'CCC'})])
+                                ('CCC', {'id':'CCC'}),
+                                ('DDD', {'id':'DDD'})
+                                ])
 
         # Fields coming from both dirs
         res = dir.searchEntries(foo='oo', return_fields=['email', 'foo'])
@@ -553,6 +685,20 @@ class TestMetaDirectoryMissing(CPSDirectoryTestCase):
         self.assertEquals(res, [('BBB', {'foo': 'oo', 'email': 'evil@hell'}),
                                 ('CCC', {'foo': 'oo', 'email': 'yo@mama'})])
 
+        res = dir.searchEntries(foo='defaultfoo',
+                                return_fields=['email', 'foo'])
+        res.sort()
+        self.assertEquals(res, [('DDD', {'foo': 'defaultfoo',
+                                         'email': 'brian@spam'}),
+                                ])
+
+        # Substring on the missing
+        self.dirfoo.search_substring_fields = ['foo']
+        res = dir.searchEntries(bar='baronly', foo='default',
+                                return_fields=['foo']) 
+        res.sort()
+        self.assertEquals(res, [('DDD', {'foo': 'defaultfoo'})])
+
     def testBasicSecurity(self):
         self.assert_(self.dirmeta.isVisible())
         self.assert_(self.dirmeta.isCreateEntryAllowed())
@@ -564,7 +710,7 @@ class TestMetaDirectoryMissing(CPSDirectoryTestCase):
 
 def test_suite():
     suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(TestMetaDirectory))
+    suite.addTest(unittest.makeSuite(TestMetaDirectoryNoMissing))
     suite.addTest(unittest.makeSuite(TestMetaDirectoryMissing))
     return suite
 
