@@ -1,5 +1,5 @@
 ##parameters=type_name, datamodel
-# $Id: createCPSDocument_cb.py 9441 2004-04-08 10:24:48Z bdelbosc $
+# $Id: createCPSDocument_cb.py 27501 2005-09-21 16:19:35Z atchertchian $
 """
 Callback to create an empty object with the context as a container.
 
@@ -11,15 +11,13 @@ Returns the created object. In CPS, returns the proxy (which is
 the only thing the user sees).
 """
 
-folder = context
-
 id = datamodel.get('Title')
 if not id:
     id = 'my ' + type_name
 
 language = datamodel.get('Language')
 if not language:
-    language = context.Localizer.get_selected_language()
+    language = context.translation_service.getSelectedLanguage()
 
 id = context.computeId(compute_from=id)
 
