@@ -1,5 +1,5 @@
 ##parameters=loadcustom=1
-# $Id: getDocumentLayouts.py 21765 2005-05-23 09:10:25Z madarche $
+# $Id: getDocumentLayouts.py 31303 2006-01-04 14:21:17Z fguillaume $
 """
 Here are defined the list of layouts to be registred
 Please, follow the same pattern to add new layouts
@@ -66,6 +66,7 @@ metadata_layout = {
                 'is_i18n': 1,
                 'label_edit': 'label_contributors',
                 'label': 'label_contributors',
+                'readonly_layout_modes': ['create', 'edit'],
                 'help': 'help_dc_contirbutors',
             },
         },
@@ -975,7 +976,7 @@ link_layout = {
             'data': {
                 'label_edit': 'cpsdoc_Link_label_edit',
                 'is_i18n': 1,
-                'fields': [],
+                'fields': ['Relation', 'Title', 'Description'],
                 'widget_ids': ['Relation', 'Title', 'Description'],
             },
         },
@@ -1059,6 +1060,7 @@ image_layout = {
                 'display_width': 640,
                 'display_height': 600,
                 'size_max': 3*1024*1024,
+                'allow_resize': True,
             },
         },
     },
@@ -1102,10 +1104,10 @@ flash_animation_layout = {
                 'size_max': 80,
             },
         },
-        'preview': {
+        'flash_file': {
             'type': 'Flash Widget',
             'data': {
-                'fields': ['preview'],
+                'fields': ['flash_file'],
                 'is_i18n': 1,
                 'label_edit': 'cpsdoc_Flash_Animation_label_edit',
                 'label': '',
@@ -1121,7 +1123,7 @@ flash_animation_layout = {
         'rows': [
             [{'widget_id': 'Source'},
              {'widget_id': 'Rights'},],
-            [{'ncols': 2, 'widget_id': 'preview'},],
+            [{'ncols': 2, 'widget_id': 'flash_file'},],
             ],
         },
     }
@@ -1143,8 +1145,8 @@ imagegallery_layout = {
                 'description': '',
                 'css_class': '',
                 'is_limited': 1,
-                'min_value': 0.0,
-                'max_value': 65536.0,
+                'min_value': 0,
+                'max_value': 65536,
                 'thousands_separator': '',
             },
         },
@@ -1160,8 +1162,8 @@ imagegallery_layout = {
                 'description': '',
                 'css_class': '',
                 'is_limited': 1,
-                'min_value': 0.0,
-                'max_value': 65536.0,
+                'min_value': 0,
+                'max_value': 65536,
                 'thousands_separator': '',
             },
         },
@@ -1177,8 +1179,8 @@ imagegallery_layout = {
                 'description': '',
                 'css_class': '',
                 'is_limited': 1,
-                'min_value': 0.0,
-                'max_value': 65536.0,
+                'min_value': 0,
+                'max_value': 65536,
                 'thousands_separator': '',
             },
         },
@@ -1194,8 +1196,8 @@ imagegallery_layout = {
                 'description': '',
                 'css_class': '',
                 'is_limited': 1,
-                'min_value': 0.0,
-                'max_value': 128.0,
+                'min_value': 0,
+                'max_value': 128,
                 'thousands_separator': '',
             },
         },
@@ -1221,8 +1223,8 @@ imagegallery_layout = {
                 'description': '',
                 'css_class': '',
                 'is_limited': 1,
-                'min_value': 100.0,
-                'max_value': 1024.0,
+                'min_value': 100,
+                'max_value': 1024,
                 'thousands_separator': '',
             },
         },
@@ -1238,8 +1240,8 @@ imagegallery_layout = {
                 'description': '',
                 'css_class': '',
                 'is_limited': 1,
-                'min_value': 100.0,
-                'max_value': 1024.0,
+                'min_value': 100,
+                'max_value': 1024,
                 'thousands_separator': '',
             },
         },
@@ -1279,8 +1281,8 @@ book_layout = {
                 'description': '',
                 'css_class': '',
                 'is_limited': 1,
-                'min_value': 0.0,
-                'max_value': 65536.0,
+                'min_value': 0,
+                'max_value': 65536,
                 'thousands_separator': '',
             },
         },
@@ -1348,8 +1350,8 @@ chapter_layout = {
                 'css_class': '',
                 'widget_mode_expr': '',
                 'is_limited': 1,
-                'min_value': 0.0,
-                'max_value': 65536.0,
+                'min_value': 0,
+                'max_value': 65536,
                 'thousands_separator': '',
             },
         },
