@@ -1,9 +1,15 @@
 ##parameters=loadcustom=1
-#$Id: getDocumentTypes.py 21576 2005-05-16 01:05:39Z janguenot $
+#$Id: getDocumentTypes.py 30643 2005-12-14 19:10:36Z jmorliaguet $
 """
 Here are defined list of portal type created with CPSDocument
 """
 
+# permissions
+from Products.CMFCore.permissions import View
+from Products.CMFCore.permissions import ModifyPortalContent
+from Products.CMFCore.permissions import AddPortalContent
+ChangePermissions = 'Change permissions'
+
 section_type = {
     'title': 'portal_type_Section_title',
     'description': 'portal_type_Section_description',
@@ -25,32 +31,28 @@ section_type = {
     'actions': ({'id': 'view',
                  'name': 'action_view',
                  'action': 'folder_view',
-                 'permissions': ('View',)},
+                 'permissions': (View,)},
                 {'id': 'new_content',
                  'name': 'action_new_content',
                  'action': 'folder_factories',
-                 'permissions': ('Modify portal content',)},
+                 'permissions': (AddPortalContent,)},
                 {'id': 'contents',
                  'name': 'action_folder_contents',
                  'action': 'folder_contents',
-                 'permissions': ('Modify portal content',)},
+                 'permissions': (ModifyPortalContent,)},
                 {'id': 'edit',
                  'name': 'action_edit',
                  'action': 'cpsdocument_edit_form',
-                 'permissions': ('Modify portal content',)},
+                 'permissions': (ModifyPortalContent,)},
                 {'id': 'metadata',
                  'name': 'action_metadata',
                  'action': 'cpsdocument_metadata',
                  'condition': 'not:portal/portal_membership/isAnonymousUser',
-                 'permissions': ('View',)},
+                 'permissions': (View,)},
                 {'id': 'localroles',
                  'name': 'action_local_roles',
                  'action': 'folder_localrole_form',
-                 'permissions': ('Change permissions',)},
-                {'id': 'boxes',
-                 'name': 'action_boxes',
-                 'action': 'box_manage_form',
-                 'permissions': ('Manage Boxes',)},
+                 'permissions': (ChangePermissions,)},
                 ),
     }
 
@@ -75,37 +77,33 @@ workspace_type = {
     'actions': ({'id': 'view',
                  'name': 'action_view',
                  'action': 'folder_view',
-                 'permissions': ('View',)},
+                 'permissions': (View,)},
                 {'id': 'new_content',
                  'name': 'action_new_content',
                  'action': 'folder_factories',
-                 'permissions': ('Modify portal content',)},
+                 'permissions': (AddPortalContent,)},
                 {'id': 'import_documents',
                  'name': 'action_import_documents',
                  'action': 'cpsdocument_import_zip_form',
                  'visible': 0,
-                 'permissions': ('Modify portal content',)},
+                 'permissions': (ModifyPortalContent,)},
                 {'id': 'contents',
                  'name': 'action_folder_contents',
                  'action': 'folder_contents',
-                 'permissions': ('Modify portal content',)},
+                 'permissions': (ModifyPortalContent,)},
                 {'id': 'edit',
                  'name': 'action_edit',
                  'action': 'cpsdocument_edit_form',
-                 'permissions': ('Modify portal content',)},
+                 'permissions': (ModifyPortalContent,)},
                 {'id': 'metadata',
                  'name': 'action_metadata',
                  'action': 'cpsdocument_metadata',
                  'condition': 'not:portal/portal_membership/isAnonymousUser',
-                 'permissions': ('View',)},
+                 'permissions': (View,)},
                 {'id': 'localroles',
                  'name': 'action_local_roles',
                  'action': 'folder_localrole_form',
-                 'permissions': ('Change permissions',)},
-                {'id': 'boxes',
-                 'name': 'action_boxes',
-                 'action': 'box_manage_form',
-                 'permissions': ('Manage Boxes',)},
+                 'permissions': (ChangePermissions,)},
                 ),
     }
 
@@ -338,7 +336,7 @@ file_type = {
                      'condition': ('python:object is not None '
                                    'and object.getContent().file is not None '
                                    'and modules["Products.CPSUtil.integration"].isProductPresent("Products.ExternalEditor")'),
-                     'permissions': ('Modify portal content',)},
+                     'permissions': (ModifyPortalContent,)},
                 ),
     }
 
@@ -448,7 +446,7 @@ flash_animation_type = {
     'allow_discussion': 0,
     'cps_is_searchable': 1,
     'cps_proxy_type': 'document',
-    'schemas': ['metadata', 'common'],
+    'schemas': ['metadata', 'common', 'flash_animation'],
     'layouts': ['common', 'flash_animation'],
     'layout_clusters': ['metadata:metadata'],
     'flexible_layouts': [],
@@ -525,6 +523,11 @@ page_type = {
     'flexible_layouts': ['flexible_content:flexible_content'],
     'storage_methods': [],
     'use_content_status_history': 1,
+    'actions_add': ({'id': 'view',
+                     'name': 'action_view',
+                     'action': 'page_view',
+                     'permissions': (View,)},
+                ),
     }
 
 types = {}
