@@ -1,5 +1,5 @@
 ##parameters=type_name, datamodel
-# $Id: createCPSDocument_cb.py 9589 2004-09-08 23:40:21Z janguenot $
+# $Id: createCPSDocument_cb.py 23831 2005-06-10 17:55:02Z gdelaroch $
 """
 Callback to create an empty object with the context as a container.
 
@@ -19,7 +19,7 @@ if not id:
 
 language = datamodel.get('Language')
 if not language:
-    language = context.Localizer.get_selected_language()
+    language = context.translation_service.getSelectedLanguage()
 
 id = context.computeId(compute_from=id)
 
