@@ -1,12 +1,11 @@
 ##parameters=dt=None, fmt='medium'
-# $Id: getDateStr.py 9590 2004-09-08 23:41:22Z janguenot $
+# $Id: getDateStr.py 23831 2005-06-10 17:55:02Z gdelaroch $
 """ return a string date using the current locale """
 
 if not dt:
     return ''
 
-Localizer = context.Localizer
-mcat = Localizer.default
+mcat = context.translation_service
 
 if fmt not in ('short', 'medium', 'long', 'iso8601'):
     fmt = 'date_long'
