@@ -1,4 +1,5 @@
-# $Id: testDefaultDocuments.py 21645 2005-05-18 23:08:32Z janguenot $
+# -*- coding: iso-8859-15 -*-
+# $Id: testDefaultDocuments.py 32322 2006-01-27 14:26:20Z sfermigier $
 # TODO:
 # - don't depend on getDocumentSchemas / getDocumentTypes but is there
 #   an API for that ?
@@ -11,7 +12,8 @@ from pprint import pprint
 import unittest
 from DateTime import DateTime
 from Testing import ZopeTestCase
-import CPSDocumentTestCase
+from Products.CPSDefault.tests.CPSTestCase import CPSTestCase
+
 from Products.CPSSchemas.Widget import widgetname
 from Products.CMFCore.utils import _getViewFor
 
@@ -38,7 +40,7 @@ def randomText(max_len=10):
         [chr(random.randint(32, 128)) for i in range(0, max_len)])
 
 
-class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
+class TestDocuments(CPSTestCase):
     def afterSetUp(self):
         try:
             self.login('manager')
@@ -63,9 +65,6 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
     attr_values_1 = {'1 Subject': ['1 New Subject',],
                      '1 Title': '1 New Title',
                      '1 Description': '1 New Description'}
-    attr_values_2 = {'2 Subject': ['2 New Subject',],
-                     '2 Title': '2 New Title',
-                     '2 Description': '2 New Description'}
     def testCreateDocumentsInWorkspacesRoot(self):
         for doc_type in self.document_types.keys():
             if doc_type in ('Section',):
@@ -84,7 +83,7 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
             else:
                 proxy = getattr(self.ws, doc_id)
                 try:
-                    doc = proxy.getContent()
+                    doc = proxy.getEditableContent()
                 except AttributeError:
                     doc = proxy
 
@@ -96,7 +95,7 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
                 expected_invariable = {}
                 expected_invariable['Contributors'] = doc.contributors
                 expected_invariable['Creator'] = doc.Creator()
-                doc.edit(**self.attr_values_1)
+                doc.edit(proxy=proxy, **self.attr_values_1)
                 self._validateDocument(proxy, doc)
                 self._testAttributeValues(doc, self.attr_values_1)
                 self._testAttributeValues(doc, expected_invariable)
@@ -106,7 +105,6 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
 
 
     def _validateDocument(self, proxy, doc):
-        self._testInterfaces(doc)
         self._testDefaultAttributes(doc)
         self.assertEquals(doc.getAdditionalContentInfo(proxy), {})
         # Rendering / default view test (on the proxy)
@@ -116,7 +114,7 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
         # Normal View
         view = _getViewFor(proxy)
         self.assert_(view())
-        self.assert_(self.isValidXML(doc.exportAsXML(proxy=proxy)))
+        self.assertWellFormedXML(doc.exportAsXML(proxy=proxy), "exportAsXML")
 
     # Standard conversion to attributes for special metadata schema.
     field_to_attr = {
@@ -166,30 +164,20 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
                         attr_name, getattr(doc, attr_name),
                         attr_expected[prop_name]))
 
-    def _testInterfaces(self, doc):
-        from Interface.Verify import verifyObject
-        from Products.CMFCore.interfaces.Dynamic \
-            import DynamicType as IDynamicType
-        from Products.CMFCore.interfaces.Contentish \
-            import Contentish as IContentish
-        from Products.CMFCore.interfaces.DublinCore \
-            import DublinCore as IDublinCore
-
-        verifyObject(IDynamicType, doc)
-        verifyObject(IContentish, doc)
-        verifyObject(IDublinCore, doc)
-
     def _testRendering(self, doc, proxy):
         res = doc.render(proxy=proxy)
         self.assert_(res)
 
     def _testMetadataRendering(self, doc, proxy):
-        res = doc.renderEditDetailed(request=None, proxy=proxy,
-                                     layout_id='metadata')
+        res = doc.render(request=None, proxy=proxy, layout_mode='view',
+                         cluster='metadata')
+        self.assert_(res)
+        res = doc.render(request=None, proxy=proxy, layout_mode='edit',
+                         cluster='metadata')
         self.assert_(res)
 
     def _testEditRendering(self, doc, proxy):
-        res = doc.renderEditDetailed(request=None, proxy=proxy)
+        res = doc.render(request=None, proxy=proxy, layout_mode='edit')
         self.assert_(res)
 
 
@@ -209,7 +197,7 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
         self.ws.invokeFactory('News Item', id)
         proxy = getattr(self.ws, id)
         try:
-            doc = proxy.getContent()
+            doc = proxy.getEditableContent()
         except AttributeError:
             doc = proxy
 
@@ -245,20 +233,21 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
         request = self.portal.REQUEST
         request.form = form
         rendered, is_valid, ds = doc.renderEditDetailed(request=request,
+                                                        proxy=proxy,
                                                         layout_id='metadata')
-        self.assert_(is_valid, 'invalid input: ' + str(ds.getErrors()) +
+        self.assert_(is_valid, 'invalid input: ' + str(ds.errors) +
                      'ds = ' + str(ds))
         for k, v in data.items():
             self.assertEquals(ds[k], v)
 
     def testNews(self):
         self.ws.invokeFactory('News Item', 'news')
-        proxy = getattr(self.ws, 'news')
+        proxy = self.ws.news
 
         try:
-            doc = self.ws.news.getContent()
+            doc = proxy.getEditableContent()
         except AttributeError:
-            doc = self.ws.news
+            doc = proxy
 
         # Test doc has default values
         for prop_name in self.document_schemas['newsitem'].keys():
@@ -266,17 +255,26 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
             # getDocumentSchemas(). I consider this as a bug.
             self.assert_(hasattr(doc, prop_name))
 
-        doc.edit(Title='The title')
-        self.assertEquals(doc.Title(), 'The title')
+        TITLE = "Un titre accentu"
+        CONTENT = "L't est bientt termin"
 
-        doc.edit(content='The content')
-        self.assertEquals(doc.content, 'The content')
+        doc.edit(proxy=proxy, Title=TITLE)
+        self.assertEquals(doc.Title(), TITLE)
+
+        doc.edit(proxy=proxy, content=CONTENT)
+        self.assertEquals(doc.content, CONTENT)
         self.assertEquals(
-            doc.getAdditionalContentInfo(proxy)['summary'], 'The content')
+            doc.getAdditionalContentInfo(proxy)['summary'], CONTENT)
 
+        # Test view
+        view = proxy.cpsdocument_view()
+        self.assert_(TITLE in view)
+        self.assert_(CONTENT in view)
+
+        # Test summary for long content
         from Products.CPSDocument.CPSDocument import SUMMARY_MAX_LEN
         very_long_content = 'A very long content' * 100
-        doc.edit(content=very_long_content)
+        doc.edit(proxy=proxy, content=very_long_content)
         self.assertEquals(
             doc.getAdditionalContentInfo(proxy)['summary'],
             very_long_content[0:SUMMARY_MAX_LEN] + '...')
@@ -286,7 +284,7 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
 
         proxy = self.ws.file1
         try:
-            doc = proxy.getContent()
+            doc = proxy.getEditableContent()
         except AttributeError:
             doc = proxy
 
@@ -298,7 +296,7 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
 
         # Edit file as string
         text = randomText()
-        doc.edit(file=text)
+        doc.edit(proxy=proxy, file=text)
         self.assertEquals(doc.file, text)
         self.assertEquals(proxy['file'], text)
         #XXX self.assertEquals(doc.downloadFile('file'), text)
@@ -315,31 +313,31 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
         #self.assertEquals(response.headers['Content-Disposition'],
         #    "inline; filename=file")
 
-    if 0: # Don't know hown to do that
-        # Edit
-        class FieldStorage:
-            def __init__(self, **kw):
-                for k, v in kw.items():
-                    setattr(self, k, v)
-        from StringIO import StringIO
-        from ZPublisher.HTTPRequest import FileUpload
-        text = randomText()
-        file = StringIO(text)
-        fs = FieldStorage(file=file, headers={"Content-Type": "text/html"},
-            filename="filename")
-        fileupload = FileUpload(fs)
+        if 0: # XXX Don't know hown to do that
+            # Edit
+            class FieldStorage:
+                def __init__(self, **kw):
+                    for k, v in kw.items():
+                        setattr(self, k, v)
+            from StringIO import StringIO
+            from ZPublisher.HTTPRequest import FileUpload
+            text = randomText()
+            file = StringIO(text)
+            fs = FieldStorage(file=file, headers={"Content-Type": "text/html"},
+                filename="filename")
+            fileupload = FileUpload(fs)
 
-        doc.renderEdit(file=fileupload)
+            doc.renderEdit(file=fileupload)
 
-        response = DummyResponse()
-        doc.downloadFile('file', response)
-        self.assertEquals(response.data, text)
-        self.assertEquals(response.headers['Content-Type'],
-            'application/octet-stream')
-        self.assertEquals(response.headers['Content-Length'],
-            len(text))
-        self.assertEquals(response.headers['Content-Disposition'],
-            "inline; filename=filename")
+            response = DummyResponse()
+            doc.downloadFile('file', response)
+            self.assertEquals(response.data, text)
+            self.assertEquals(response.headers['Content-Type'],
+                'application/octet-stream')
+            self.assertEquals(response.headers['Content-Length'],
+                len(text))
+            self.assertEquals(response.headers['Content-Disposition'],
+                "inline; filename=filename")
 
     # XXX: is this correct ???
     def testFileCalledFile(self):
@@ -357,7 +355,10 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
             doc = self.ws.flex.getContent()
         except AttributeError:
             doc = self.ws.flex
-        # XXX: now do something!
+        tinfo = doc.getTypeInfo()
+        wid = tinfo.flexibleAddWidget(doc, 'flexible_content', 'link')
+        doc.edit(link_href_f0='http://test.nohost')
+        self.assertEquals(doc.link_href_f0, 'http://test.nohost')
 
     def testDocumentSearch(self):
         # The aim of this test is first to assert that documents can be queried.
@@ -491,6 +492,59 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
         # Check it's they are the same
         self.assertEqual(dm, dm2)
 
+    def test_renderEditObjectDetailed_pre_commit_hook(self):
+        # this no more mis-placed than the previous one :-)
+
+        def pre_hook(dm, proxy=None, **kw):
+            proxy.freezeProxy()
+
+        self.ws.invokeFactory('Workspace', 'test',
+                              Title='test', Description='Old')
+        proxy = self.ws.test
+        ob = proxy.getContent()
+        tinfo = ob.getTypeInfo()
+
+        request = self.portal.REQUEST
+        request.form = {widgetname('Description') : 'New descr'}
+        tinfo.renderEditObjectDetailed(ob, request=request,
+                                       proxy=proxy,
+                                       pre_commit_hook=pre_hook)
+
+        proxy = self.ws.test
+        # modification was done
+        self.assertEquals(proxy.getContent().Description(), 'New descr')
+        # hook was called
+        self.assertEquals(proxy.getRevision(), 2)
+        # and that was before the DM commit
+        self.assertEquals(proxy.getContent(rev=1).Description(), 'Old')
+
+    def test_renderEditObjectDetailed_post_commit_hook(self):
+        # a post-commit hook that stores info 
+        events = []
+        def post_hook(ob, proxy=None, **kw):
+            events.append((proxy.getId(), ob.Description()))
+
+        self.ws.invokeFactory('Workspace', 'test',
+                              Title='test', Description='Old')
+        proxy = self.ws.test
+        ob = proxy.getContent()
+        tinfo = ob.getTypeInfo()
+
+        request = self.portal.REQUEST
+        request.form = {widgetname('Description') : 'New descr'}
+        tinfo.renderEditObjectDetailed(ob, request=request,
+                                       proxy=proxy,
+                                       post_commit_hook=post_hook)
+
+        proxy = self.ws.test
+        # modification was done
+        self.assertEquals(proxy.getContent().Description(), 'New descr')
+        # hook was called, and that was after the DM commit
+        self.assertEquals(events, [('test', 'New descr')])
+
+
+        
+
     def testFlashDocument(self):
 
         doc_type = 'Flash Animation'
@@ -529,9 +583,10 @@ class TestDocuments(CPSDocumentTestCase.CPSDocumentTestCase):
         proxy = getattr(self.ws, doc_id)
 
         # edit Title
-        proxy.edit(Title='a title')
+        doc = proxy.getEditableContent()
+        doc.edit(proxy=proxy, Title='a title')
 
-        stext = proxy.getContent().SearchableText()
+        stext = doc.SearchableText()
 
         # It should return at least the title right now.
         self.assert_(stext)
