@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: install.py 10596 2005-03-16 10:11:49Z madarche $
+# $Id: install.py 30240 2005-12-02 20:07:21Z fguillaume $
 
 """
 CPSForum Installer
@@ -396,7 +396,8 @@ pid = state_change.kwargs.get('parent_id', '')
 kw = {'Title': subject,
       'Description': msg,
       'author': author,
-      'parent_id': pid}
+      'parent_id': pid,
+      'proxy': object}
 
 object.getEditableContent().edit(**kw)
 """
@@ -486,21 +487,21 @@ forum.getContent().postRejected(object.id, proxy=forum)
          'id': 'activate_comments',
          'name': 'action_activate_comments',
          'action': 'string:${object/absolute_url}/set_comment_mode?mode=1',
-         'condition': "python:getattr(object.getTypeInfo(),'cps_proxy_type','') in ('document', 'folderishdocument') and object.getContent().allow_discussion == 0",
+         'condition': "python:getattr(object.getTypeInfo(),'cps_proxy_type','') in ('document', 'folderishdocument') and getattr(object.getContent(), 'allow_discussion', None) == 0",
          'permission': 'Forum manage comments',
          'category': 'object',},
         {'tool':'portal_discussion',
          'id': 'deactivate_comments',
          'name': 'action_deactivate_comments',
          'action': 'string:${object/absolute_url}/set_comment_mode?mode=0',
-         'condition': "python:getattr(object.getTypeInfo(),'cps_proxy_type','') in ('document', 'folderishdocument') and object.getContent().allow_discussion == 1",
+         'condition': "python:getattr(object.getTypeInfo(),'cps_proxy_type','') in ('document', 'folderishdocument') and getattr(object.getContent(), 'allow_discussion', None) == 1",
          'permission': 'Forum manage comments',
          'category': 'object',},
         {'tool':'portal_discussion',
          'id': 'manage_comments',
          'name': 'action_manage_comments',
          'action': "string:${object/absolute_url}/manage_comments",
-         'condition': "python:getattr(object.getTypeInfo(),'cps_proxy_type','') in ('document', 'folderishdocument') and object.getContent().allow_discussion == 1",
+         'condition': "python:getattr(object.getTypeInfo(),'cps_proxy_type','') in ('document', 'folderishdocument') and getattr(object.getContent(), 'allow_discussion', None) == 1",
          'permission': 'Forum manage comments',
          'category': 'object',},
         {'tool':'portal_discussion',
@@ -516,8 +517,8 @@ forum.getContent().postRejected(object.id, proxy=forum)
     # New listeners for events
     ##############################################
     installer.log("Checking doc deletion listener (for commenting forum mapping sync)")
-    estool = installer.portal.portal_eventservice
-    objs = estool.objectValues()
+    evtool = installer.portal.portal_eventservice
+    objs = evtool.objectValues()
     subscribers = []
     for obj in objs:
         subscribers.append(obj.subscriber)
@@ -525,7 +526,7 @@ forum.getContent().postRejected(object.id, proxy=forum)
         installer.logOK()
     else:
         installer.log(" Creating portal_discussion subscriber")
-        estool.manage_addSubscriber(subscriber='portal_discussion',
+        evtool.manage_addSubscriber(subscriber='portal_discussion',
                                     action='document_deleted',
                                     meta_type='*',
                                     event_type='sys_del_object',
