@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: CMFInstaller.py 21669 2005-05-19 16:15:19Z madarche $
+# $Id: CMFInstaller.py 30672 2005-12-15 13:58:51Z jmorliaguet $
 
 import os
 from types import StringType
@@ -36,10 +36,12 @@ class CMFInstaller:
     def __init__(self, context, product_name=None):
         """CMFInstaller initialization
 
+        context is used to pass the portal root.
+
         product_name should be set as a class attribute when subclassing,
         but must be passed if you are not subclassing the installer.
 
-        is_main_installer should be se to 0 if this installer is called
+        _v_main_installer should be se to 0 if this installer is called
         from another installer to prevent multiple reindexing of catalogs
         and similar actions that only needs to be done once.
         """
@@ -170,6 +172,24 @@ class CMFInstaller:
         If No default is given, it will raise an error.
         """
         return getToolByName(self.portal, id, default)
+
+    #
+    # CMFSetup
+    #
+
+    def setupSetupTool(self):
+        self.log("Setting up Portal Setup Tool")
+        self.verifyTool('portal_setup', 'CMFSetup', 'Portal Setup Tool')
+
+    def importSetupProfile(self, profile_id, steps):
+        self.log("Importing profile %s, steps %s" % (profile_id, steps))
+        setup_tool = getToolByName(self.portal, 'portal_setup')
+        setup_tool.setImportContext('profile-%s' % profile_id)
+        for step in steps:
+            res = setup_tool.runImportStep(step)
+            for s in res['steps']:
+                self.log("%s: %s" % (s, res['messages'][s]))
+
     #
     # Methods to setup and manage actions
     #
@@ -303,6 +323,7 @@ class CMFInstaller:
                 # CPSIO skins on a Plone site when exporting a Plone site.
                 if skin_name not in  ['Basic',
                                       'CPSSkins',
+                                      'CPSSkins-macroless',
                                       'Plone Default',
                                       'Plone Tableless']:
                     continue
@@ -355,13 +376,15 @@ class CMFInstaller:
         manage_addLexicon(ct, id, title, elems)
         self.log(' Added')
 
-    def addPortalCatalogIndex(self, id, type, extra=None):
+    def addPortalCatalogIndex(self, id, type, extra=None, destructive=False):
         """Adds an index on portal_catalog."""
         self.log(' Portal_catalog verify index %s: %s' % (type, id))
         ct = self.portal.portal_catalog
         if id in ct.indexes():
             current_index_type = ct._catalog.getIndex(id).meta_type
-            if current_index_type == type:
+            if destructive:
+                pass
+            elif current_index_type == type:
                 self.logOK()
                 return
             elif type == 'ZCTextIndex' and current_index_type != 'TextIndex':
@@ -448,17 +471,26 @@ class CMFInstaller:
             self.log("  Type '%s'" % ptype)
             if ptype in ptypes_installed:
                 if destructive:
-                    self.log(" Deleting %s"  %ptype)
+                    self.log(" Deleting %s" % ptype)
                     ttool.manage_delObjects([ptype])
                 else:
                     self.logOK()
                     continue
 
             self.log("  Adding")
+            add_meta_type = typeinfo['add_meta_type']
+            typeinfo_name = typeinfo['typeinfo_name']
+            if not typeinfo_name.endswith(')'):
+                from warnings import warn
+                warn('Old typeinfo name is deprecated, got %r for type %r'
+                     % (typeinfo_name, ptype),
+                     DeprecationWarning, stacklevel=2)
+                ti_prod, ti_mt = [x.strip() for x in typeinfo_name.split(':')]
+                typeinfo_name = '%s (%s)' % (typeinfo_name, ti_mt)
             ttool.manage_addTypeInformation(
                 id=ptype,
-                add_meta_type=typeinfo['add_meta_type'],
-                typeinfo_name=typeinfo['typeinfo_name'])
+                add_meta_type=add_meta_type,
+                typeinfo_name=typeinfo_name)
             if typeinfo.has_key('properties'):
                 ttool[ptype].manage_changeProperties(
                     **typeinfo['properties'])
