@@ -15,13 +15,14 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: CPSInstaller.py 21925 2005-05-30 14:43:34Z madarche $
+# $Id: CPSInstaller.py 29589 2005-11-16 18:30:41Z fguillaume $
 
 import os
 from re import match
 from types import TupleType, ListType
 from ConfigParser import ConfigParser, NoOptionError, NoSectionError
 
+from OFS.Cache import isCacheable
 from App.Extensions import getPath
 from Acquisition import aq_base
 from zLOG import LOG, INFO, DEBUG
@@ -199,7 +200,7 @@ class CPSInstaller(CMFInstaller):
 
     def verifyLocalWorkflowChains(self, object, wfchains, destructive=0,
                                   under_sub_add=None):
-        """Sets up the local workflows on object.
+        """Set up the local workflows on object.
 
         wfchains = {
             '<Portal Type>': '<workflow_id>',
@@ -259,7 +260,35 @@ class CPSInstaller(CMFInstaller):
             if data.has_key('actions_add'):
                 self.log("    Adding actions")
                 for a in data['actions_add']:
-                    self.addAction(ti, a)
+                    # Remove previous with same id+category
+                    prevactions = ti.listActions()
+                    for i, prev in enumerate(prevactions):
+                        if (prev.id == a['id'] and
+                            prev.category == a.get('category', 'object')):
+                            # Change previous action
+                            # No API for that <sigh>
+                            properties = {}
+                            suffix = '_'+str(i)
+                            properties['visible'+suffix] = True
+                            for k, v in a.items():
+                                if k == 'permissions':
+                                    k = 'permission'
+                                if (k == 'action' and
+                                    v is not None and
+                                    ':' not in v):
+                                    v = 'string:${object_url}/'+v
+                                properties[k+suffix] = v
+                            if not k in properties:
+                                properties
+                            actions = list(prevactions)
+                            actions[i] = ti._extractAction(properties, i)
+                            ti._actions = tuple(actions)
+                            break
+                    else:
+                        self.addAction(ti, a)
+
+            if data.get('aliases'):
+                ti.setMethodAliases(data['aliases'])
 
             if data.get('display_in_cmf_calendar'):
                 display_in_cmf_calendar.append(ptype)
@@ -370,7 +399,7 @@ class CPSInstaller(CMFInstaller):
     # CPSSchemas installation
     #
     def verifySchemas(self, schemas):
-        """Adds schemas if they don't exist
+        """Add schemas if they don't exist
 
         The schemas parameter is a dictionary of schema definitions.
         The schema definition is what you get when you go to the 'Export'
@@ -397,32 +426,12 @@ class CPSInstaller(CMFInstaller):
                 schema.manage_addField(field_id, fieldinfo['type'],
                                     **fieldinfo['data'])
 
+    # BBB, will be removed in CPS 3.4.1
     def verifyWidgets(self, widgets):
-        """Adds widgets if they don't exist
-
-        The widgets parameter is a dictionary of widget definitions.
-        The widget definition is what you get when you go to the 'Export'
-        tab of a widget.
-        """
-        self.log("Verifying widgets")
-        wtool = self.portal.portal_widget_types
-        existing_widgets = wtool.objectIds()
-        for id, info in widgets.items():
-            self.log(" Adding widget %s" % id)
-            if id in existing_widgets:
-                ob = wtool[id]
-                if hasattr(ob, 'isUserModified') and ob.isUserModified():
-                    self.log('WARNING: The widget is modified and will not be '
-                             'changed. Delete manually if needed.')
-                    continue
-                else:
-                    self.log('   Deleting old definition')
-                    wtool.manage_delObjects([id])
-            widget = wtool.manage_addCPSWidgetType(id, info['type'])
-            widget.manage_changeProperties(**info['data'])
+        return
 
     def verifyLayouts(self, layouts):
-        """Adds layouts if they don't exist
+        """Add layouts if they don't exist
 
         The layouts parameter is a dictionary of layout definitions.
         The layout definition is what you get when you go to the 'Export'
@@ -451,7 +460,7 @@ class CPSInstaller(CMFInstaller):
             layout.manage_changeProperties(**info['layout'])
 
     def verifyVocabularies(self, vocabularies):
-        """Adds vocabularies if they don't exist
+        """Add vocabularies if they don't exist
 
         The vocabularies parameter is a dictionary of vocabulary definitions.
         The vocabulary definition is what you get when you go to the 'Export'
@@ -549,7 +558,7 @@ class CPSInstaller(CMFInstaller):
 
 
     def verifyMessageCatalog(self, catalog_id, title):
-        """Sets up a spezialized message catalog for your product"""
+        """Set up a specialized message catalog for your product"""
         self.log('Verifying message domain %s' % catalog_id)
         localizer = self.portal['Localizer']
         # MessageCatalog
@@ -578,9 +587,13 @@ class CPSInstaller(CMFInstaller):
         tree = self.portal.portal_trees[treename]
         old_type_names = list(tree.type_names)
         old_meta_types = list(tree.meta_types)
+        new_type_names = [x for x in list(type_names)
+                          if x not in old_type_names]
+        new_meta_types = [x for x in list(meta_types)
+                          if x not in old_meta_types]
         tree.manage_changeProperties(
-            type_names=old_type_names + list(type_names),
-            meta_types=old_meta_types + list(meta_types))
+            type_names=old_type_names + list(new_type_names),
+            meta_types=old_meta_types + list(new_meta_types))
         if (old_type_names != tree.type_names
             or old_meta_types != tree.meta_types):
             self.flagRebuildTreeCache(treename)
@@ -615,7 +628,7 @@ class CPSInstaller(CMFInstaller):
 
     def verifyBoxes(self, boxes, object=None):
         """Verify the existence of given boxes in the object's box container.
-        If not found, a box is instantied. Existing boxes are not affected.
+        If not found, a box is instantiated. Existing boxes are not affected.
 
         boxes is a dictionary with keys begins the box ids, and values being
         the dictionary given by the export tab.
@@ -637,12 +650,8 @@ class CPSInstaller(CMFInstaller):
 
             self.log("   Creation of box: %s" % box)
 
-            try:
-                apply(ttool.constructContent,
-                      (boxes[box]['type'], box_container,
-                       box, None), {})
-            except:
-                raise str(box)
+            __traceback_info__ = (box, boxes[box])
+            ttool.constructContent(boxes[box]['type'], box_container, box)
 
             ob = getattr(box_container, box)
             ob.manage_changeProperties(**boxes[box])
@@ -743,6 +752,7 @@ class CPSInstaller(CMFInstaller):
         dirtool = self.portal.portal_directories
         for id, info in directories.items():
             self.log(" Directory %s" % id)
+            cache_manager_id = None
             if id in dirtool.objectIds():
                 dir = dirtool[id]
                 if hasattr(dir, 'isUserModified') and dir.isUserModified():
@@ -750,6 +760,10 @@ class CPSInstaller(CMFInstaller):
                              'be changed. Delete manually if needed.')
                 else:
                     self.log('   Deleting old definition')
+                    if isCacheable(dir) and dir.ZCacheable_isCachingEnabled():
+                        # Store the associate ram cache manager
+                        cache_manager_id = dir.ZCacheable_getManagerId()
+                        dir.ZCacheable_invalidate()
                     dirtool.manage_delObjects([id])
             if id not in dirtool.objectIds():
                 dir = dirtool.manage_addCPSDirectory(id, info['type'])
@@ -779,6 +793,9 @@ class CPSInstaller(CMFInstaller):
                 dir.setBackingDirectories(bdi)
                 del info['data']['backing_dir_infos']
             dir.manage_changeProperties(**info['data'])
+            if cache_manager_id is not None:
+                # Restore the associate ram cache manager
+                dir.ZCacheable_setManagerId(cache_manager_id)
 
     def verifyEventSubscribers(self, subscribers):
         objs = self.portal.portal_eventservice.objectValues()
