@@ -1,5 +1,5 @@
 ##parameters=key=None
-#$Id: getVocabularyListingFilterTypes.py 11999 2004-04-27 15:25:32Z bdelbosc $
+#$Id: getVocabularyListingFilterTypes.py 23833 2005-06-10 17:55:08Z gdelaroch $
 """
 return the portal type vocabulary, used as MethodVocabulary
 """
@@ -14,7 +14,7 @@ def cmp_type(a, b):
     return cmp(aa, bb)
 
 def l10n(s):
-    cpsmcat = context.Localizer.default
+    cpsmcat = context.translation_service
     ret = cpsmcat(s)
     if same_type(ret, u''):
         return ret.encode('iso-8859-15', 'ignore')
