@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: OOoDocbookDocument.py 22009 2005-06-01 13:39:51Z fguillaume $
+# $Id: OOoDocbookDocument.py 30121 2005-11-30 14:20:51Z rspivak $
 
 import re
 import os
@@ -42,9 +42,16 @@ from Products.CPSDocument.CPSDocument import CPSDocument
 
 from App.Common import rfc1123_date
 
-from elementtree.ElementTree import XML, ElementTree
-from elementtree.ElementPath import findall as xpath_findall
-from elementtree.ElementPath import find as xpath_find
+# lxml should provide ElementTree API in case ElementTree is not installed
+# TODO: after dust has settled, remove dependence on ElementTree completely
+try:
+    from elementtree.ElementTree import XML, ElementTree
+    from elementtree.ElementPath import findall as xpath_findall
+    from elementtree.ElementPath import find as xpath_find
+except ImportError:
+    from lxml.etree import XML, ElementTree
+    from lxml._elementpath import findall as xpath_findall
+    from lxml._elementpath import find as xpath_find
 
 from zLOG import LOG, \
      TRACE, DEBUG, BLATHER, INFO, PROBLEM, WARNING, ERROR, PANIC
@@ -172,12 +179,12 @@ def nodeParentStyleNameMatchesStyleName(node, style_name):
     for styleElt in styleElts:
         automatic_style_name = styleElt.getAttribute('style:name')
         parent_style_name = styleElt.getAttribute('style:parent-style-name')
-        LOG(log_key, DEBUG, "node_style_name = %s" % repr(node_style_name))
-        LOG(log_key, DEBUG, "automatic_style_name = %s" % repr(automatic_style_name))
-        LOG(log_key, DEBUG, "parent_style_name = %s" % repr(parent_style_name))
+##         LOG(log_key, DEBUG, "node_style_name = %s" % repr(node_style_name))
+##         LOG(log_key, DEBUG, "automatic_style_name = %s" % repr(automatic_style_name))
+##         LOG(log_key, DEBUG, "parent_style_name = %s" % repr(parent_style_name))
         if (node_style_name == automatic_style_name
             and parent_style_name == style_name):
-            LOG(log_key, DEBUG, "True for style_name = %s" % repr(style_name))
+##             LOG(log_key, DEBUG, "True for style_name = %s" % repr(style_name))
             return True
     return False
 
@@ -208,7 +215,7 @@ def replaceKeywords(node, iter_context):
 
 
 def replaceContributors(contributors, document):
-    LOG(log_key, DEBUG, "contributors = %s" % str(contributors))
+##     LOG(log_key, DEBUG, "contributors = %s" % str(contributors))
     officeBodyElt = document.getElementsByTagName('office:body')[0]
     paraElts = officeBodyElt.getElementsByTagName('text:p')
     for elt in paraElts:
@@ -261,7 +268,7 @@ def replaceContributors(contributors, document):
 
         # Here we only want to split the contributor's fullname in 2 parts
         fullname = k.split(None, 1)
-        LOG(log_key, DEBUG, "fullname = %s" % str(fullname))
+##         LOG(log_key, DEBUG, "fullname = %s" % str(fullname))
         if fullname:
             if len(fullname) == 2:
                 firstname = fullname[0]
@@ -270,8 +277,8 @@ def replaceContributors(contributors, document):
                 firstname = ""
                 surname = fullname[0]
 
-            LOG(log_key, DEBUG, "firstname = %s" % firstname)
-            LOG(log_key, DEBUG, "surname = %s" % surname)
+##             LOG(log_key, DEBUG, "firstname = %s" % firstname)
+##             LOG(log_key, DEBUG, "surname = %s" % surname)
             if firstname:
                 spanElt = document.createElement('text:span')
                 spanElt.setAttribute('text:style-name', 'Firstname')
@@ -380,7 +387,7 @@ class OOoDocbookDocument(CPSDocument):
     metainfo_element_name = top_element_name + 'info'
 
     security.declareProtected(View, 'exportXmlDocbook')
-    def exportXmlDocbook(self, REQUEST=None, **kw):
+    def exportXmlDocbook(self):
         """Export XML, in the Docbook XML format, for this document.
         """
         tmpDirName = tempfile.mktemp()
@@ -393,7 +400,7 @@ class OOoDocbookDocument(CPSDocument):
         dbFileName = re.sub('\..+?$', '.docb.xml', self.file.title)
         dbFileName = re.sub('\s', '_', dbFileName)
         dbFilePath = os.path.join(tmpDirPath, dbFileName)
-        LOG(log_key, DEBUG, "DocBook file path = %s" % dbFilePath)
+##         LOG(log_key, DEBUG, "DocBook file path = %s" % dbFilePath)
         filePaths.append(dbFilePath)
         dbFile = open(dbFilePath, 'w+c')
         dbFile.write(str(self.file_xml))
@@ -402,22 +409,22 @@ class OOoDocbookDocument(CPSDocument):
         imageFileNames = self.file_xml_subfiles
         for imageFileName in imageFileNames:
             imageFilePath = os.path.join(tmpDirPath, imageFileName)
-            LOG(log_key, DEBUG, "Image file path = %s" % imageFilePath)
+##             LOG(log_key, DEBUG, "Image file path = %s" % imageFilePath)
             filePaths.append(imageFilePath)
-            imageFile = open(imageFilePath, 'w+c')
+            imageFile = open(imageFilePath, 'wb')
             imageFile.write(str(getattr(self, imageFileName)))
-            imageFile.flush()
+            imageFile.close()
 
         # Regexp to replace "xxx.sxw" by "xxx.zip"
         archiveFileName = re.sub('\..+?$', '.zip', self.file.title)
         archiveFileName = re.sub('\s', '_', archiveFileName)
         archiveFilePath = os.path.join(tmpDirPath, archiveFileName)
-        LOG(log_key, DEBUG, "Archive file path = %s" % archiveFilePath)
+##         LOG(log_key, DEBUG, "Archive file path = %s" % archiveFilePath)
         # Create a ZipFile object to write into
         archiveFile = ZipFile(archiveFilePath, 'w', ZIP_DEFLATED)
         archiveInternalSubDirName = dbFileName.split('.')[0]
         for filePath in filePaths:
-            LOG(log_key, DEBUG, "adding file to archive = %s" % filePath)
+##             LOG(log_key, DEBUG, "adding file to archive = %s" % filePath)
             if filePath != dbFilePath:
                 filePathInTheArchive = os.path.join(archiveInternalSubDirName,
                                                     'images', os.path.split(filePath)[1])
@@ -432,21 +439,22 @@ class OOoDocbookDocument(CPSDocument):
         archiveFile.close()
         dbFile.close()
 
-        archiveFile = open(archiveFilePath, 'r')
+        archiveFile = open(archiveFilePath, 'rb')
         out = archiveFile.read()
         archiveFile.close()
 
         shutil.rmtree(tmpDirPath)
 
-        resp = self.REQUEST.RESPONSE
-        resp.setHeader('Content-Type', 'application/zip')
-        resp.setHeader('Content-Disposition', 'filename=' + archiveFileName)
-        # The "no-cache" headers hit a bug when using MSIE on SSL so we need to use the
-        # approach of using the Last-Modified header.
-        # http://support.microsoft.com/default.aspx?scid=http://support.microsoft.com:80/support/kb/articles/q316/4/31.asp&NoWebContent=1&NoWebContent=&NoWebContent=1
-        #resp.setHeader('Pragma', 'no-cache')
-        #resp.setHeader('Cache-Control', 'no-cache')
-        resp.setHeader('Last-Modified', rfc1123_date())
+        if self.REQUEST:
+            resp = self.REQUEST.RESPONSE
+            resp.setHeader('Content-Type', 'application/zip')
+            resp.setHeader('Content-Disposition', 'filename=' + archiveFileName)
+            # The "no-cache" headers hit a bug when using MSIE on SSL so we
+            # need to use the approach of using the Last-Modified header.
+            # http://support.microsoft.com/default.aspx?scid=http://support.microsoft.com:80/support/kb/articles/q316/4/31.asp&NoWebContent=1&NoWebContent=&NoWebContent=1
+            #resp.setHeader('Pragma', 'no-cache')
+            #resp.setHeader('Cache-Control', 'no-cache')
+            resp.setHeader('Last-Modified', rfc1123_date())
 
         return out
 
@@ -468,8 +476,8 @@ class OOoDocbookDocument(CPSDocument):
         CPSDocument.inheritedAttribute('postCommitHook')(
             self, datamodel=datamodel)
 
-        LOG(log_key, DEBUG, 'postCommitHook: dm.dirty=%s' %
-            datamodel.dirty.keys())
+##         LOG(log_key, DEBUG, 'postCommitHook: dm.dirty=%s' %
+##             datamodel.dirty.keys())
 
         if datamodel.isDirty('file'):
             self._updateMetadataFromDocument(datamodel)
@@ -479,18 +487,17 @@ class OOoDocbookDocument(CPSDocument):
 
     def _updateMetadataFromDocument(self, datamodel):
         """Update the object's metadata from the document."""
-
-        LOG(log_key, DEBUG, "_updateMetadataFromDocument")
+##         LOG(log_key, DEBUG, "_updateMetadataFromDocument")
 
         if not self.file_xml:
-            LOG(log_key, PROBLEM,
-                "_updateMetadataFromDocument no file_xml -> no update")
+##             LOG(log_key, PROBLEM,
+##                 "_updateMetadataFromDocument no file_xml -> no update")
             return
 
         file_xml_string = str(self.file_xml)
         if not file_xml_string:
-            LOG(log_key, PROBLEM,
-                "_updateMetadataFromDocument no file_xml_string -> no update")
+##             LOG(log_key, PROBLEM,
+##                 "_updateMetadataFromDocument no file_xml_string -> no update")
             return
         doc = ElementTree(XML(file_xml_string))
         rootElt = doc.getroot()
@@ -505,7 +512,7 @@ class OOoDocbookDocument(CPSDocument):
                 abstract = '%s%s%s' % (abstract, delimiter, toLatin9(elt.text))
                 delimiter = '\n\n'
 
-        LOG(log_key, DEBUG, "_updateMetadataFromDocument abstract = %s" % abstract)
+##         LOG(log_key, DEBUG, "_updateMetadataFromDocument abstract = %s" % abstract)
 
         year = findText(metainfoElt, 'copyright/year')
         holder = findText(metainfoElt, 'copyright/holder')
@@ -514,9 +521,9 @@ class OOoDocbookDocument(CPSDocument):
             rights = "Copyright  " + rights
 
         coverage = findFirstText(metainfoElt, 'bibliocoverage')
-        LOG(log_key, DEBUG, "_updateMetadataFromDocument coverage = %s" % coverage)
+##         LOG(log_key, DEBUG, "_updateMetadataFromDocument coverage = %s" % coverage)
         source = findFirstText(metainfoElt, 'bibliosource')
-        LOG(log_key, DEBUG, "_updateMetadataFromDocument source = %s" % source)
+##         LOG(log_key, DEBUG, "_updateMetadataFromDocument source = %s" % source)
         relation = ''
         relationElts = xpath_findall(metainfoElt,
                                      'bibliorelation/ulink')
@@ -536,7 +543,7 @@ class OOoDocbookDocument(CPSDocument):
 
         keywordElts = xpath_findall(metainfoElt, 'keywordset/keyword')
         keywords = [toLatin9(x.text) for x in keywordElts or ()]
-        LOG(log_key, DEBUG, "_updateMetadataFromDocument keywords = %s" % ", ".join(keywords))
+##         LOG(log_key, DEBUG, "_updateMetadataFromDocument keywords = %s" % ", ".join(keywords))
 
         # Do not use unfilled data from an OOo document on to the
         # portal. Unfilled data is marked up as <xxxxx>.
@@ -576,7 +583,7 @@ class OOoDocbookDocument(CPSDocument):
 
     def _updateDocumentFromMetadata(self, datamodel):
         """Update the document from the object's metadata."""
-        LOG(log_key, DEBUG, '_updateDocumentFromMetadata')
+##         LOG(log_key, DEBUG, '_updateDocumentFromMetadata')
 
         file = getattr(aq_base(self), 'file', None)
         if file is None:
@@ -594,7 +601,7 @@ class OOoDocbookDocument(CPSDocument):
             try:
                 z = ZipFile(StringIO(str(file)), 'r')
             except BadZipfile:
-                LOG(log_key, DEBUG, "Attached file is not a zipfile")
+##                 LOG(log_key, DEBUG, "Attached file is not a zipfile")
                 return
             for zfilename in z.namelist():
                 filename = zfilename.replace('/', '_')
@@ -611,8 +618,7 @@ class OOoDocbookDocument(CPSDocument):
             z.close()
 
             if content_xml is None:
-                LOG(log_key, DEBUG,
-                    "Document does not have any content.xml")
+##                 LOG(log_key, DEBUG, "Document does not have any content.xml")
                 return
 
             # Parse XML into DOM and modify it.
@@ -717,12 +723,11 @@ def addOOoDocbookDocumentInstance(container, id, REQUEST=None, **kw):
     container._setObject(id, instance)
 
     # It's mandatory then after to get the object through its parent, for the
-    # object to have a reference on its parent. Having the object know about its
-    # parent is mandatory if one wants to be able to call some methods like
+    # object to have a reference on its parent. Having the object know about
+    # its parent is mandatory if one wants to be able to call some methods like
     # manage_addProduct() on it.
     #object = getattr(container, id)
 
     if REQUEST:
         object = container._getOb(id)
-        LOG(log_key, DEBUG, "object = %s" % object)
         REQUEST.RESPONSE.redirect(object.absolute_url() + '/manage_main')
