@@ -17,32 +17,13 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: testGlobalInstall.py 12429 2005-04-15 08:28:23Z madarche $
-
-import os, sys
-if __name__ == '__main__':
-    execfile(os.path.join(sys.path[0], 'framework.py'))
+# $Id: testGlobalInstall.py 31728 2006-01-17 20:07:56Z fguillaume $
 
 import unittest
 import CPSOOoTestCase
 
 class TestGlobalInstall(CPSOOoTestCase.CPSOOoTestCase):
 
-    def afterSetUp(self):
-        self.login('manager')
-
-    def beforeTearDown(self):
-        self.logout()
-
-    # CPSOOo is now installed by default so this doesn't work
-    def XXXtestInstallerScript(self):
-        from Products.ExternalMethod.ExternalMethod import ExternalMethod
-        installer = ExternalMethod('cpsooo_installer', 'CPSOOo Install',
-                                   'CPSOOo.install', 'install')
-        self.portal._setObject('cpsooo_installer', installer)
-        self.assert_('cpsooo_installer' in self.portal.objectIds())
-        self.portal.cpsooo_installer()
-
     def testOOoDocbookDocumentPortalTypes(self):
         ttool = self.portal.portal_types
         self.failUnless('OOoDocbookDocument' in ttool.objectIds())
