@@ -18,7 +18,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-# $Id: CPSPortlet.py 21625 2005-05-18 09:33:11Z jmorliaguet $
+# $Id: CPSPortlet.py 32835 2006-02-14 17:23:52Z jmorliaguet $
 
 __author__ = "Julien Anguenot <mailto:ja@nuxeo.com>"
 
@@ -35,19 +35,22 @@ from App.Common import rfc1123_date
 from Globals import InitializeClass, DTMLFile
 from Acquisition import aq_inner, aq_parent, aq_base
 from AccessControl import ClassSecurityInfo
+from zLOG import LOG, INFO
 
+from Products.PageTemplates.TALES import CompilerError
 from Products.CMFCore.utils import getToolByName, _getViewFor
-try:
-    from Products.CMFCore.permissions import View, ModifyPortalContent
-except ImportError:
-    from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
-
+from Products.CMFCore.permissions import View, ModifyPortalContent
+from Products.CPSCore.ProxyBase import FileDownloader
 from Products.CPSDocument.CPSDocument import CPSDocument
 
 from CPSPortletsPermissions import ManagePortlets
 from PortletGuard import PortletGuard
+from CPSPortletCatalogAware import CPSPortletCatalogAware
 from cpsportlets_utils import html_slimmer
 
+from zope.interface import implements
+from Products.CPSPortlets.interfaces import ICPSPortlet
+
 _marker = []
 
 # Edge-Side-Includes
@@ -65,12 +68,15 @@ ESI_CODE = """
 """
 
 VISIBILITY_VOC = 'cpsportlets_visibility_range_voc'
+KEYWORD_DOWNLOAD_FILE = 'downloadFile'
 
-class CPSPortlet(CPSDocument):
+class CPSPortlet(CPSPortletCatalogAware, CPSDocument):
     """ CPS Portlet
     This is a CPSPortlet child base class for portlets
     """
 
+    implements(ICPSPortlet)
+
     meta_type = 'CPS Portlet'
     portal_type = meta_type
 
@@ -83,6 +89,39 @@ class CPSPortlet(CPSDocument):
 
     guard = None
 
+    def __getitem__(self, name):
+        """Transparent traversal of the proxy to the real subobjects.
+
+        Used for skins that don't take proxies enough into account.
+
+        Parses URL revision switch of the form:
+          mydoc/archivedRevision/n/...
+
+        Parses URL translation switch of the form:
+          mydoc/switchLanguage/<lang>/...
+
+        Parses URLs for download of the form:
+          mydoc/downloadFile/attrname/mydocname.pdf
+        """
+        ob = self
+        if ob is None:
+            raise KeyError(name)
+        if name == KEYWORD_DOWNLOAD_FILE:
+            downloader = FileDownloader(ob, self)
+            return downloader.__of__(self)
+        try:
+            res = getattr(aq_base(ob), name)
+        except AttributeError:
+            try:
+                #res = ob[name]
+                res = None
+            except (KeyError, IndexError, TypeError, AttributeError):
+                raise KeyError, name
+        if hasattr(res, '__of__'):
+            # XXX Maybe incorrect if complex wrapping.
+            res = aq_base(res).__of__(self)
+        return res
+
     security.declarePublic('getGuard')
     def getGuard(self):
         return self.guard
@@ -92,23 +131,45 @@ class CPSPortlet(CPSDocument):
         """Used only by the time of using guard management form."""
         return PortletGuard().__of__(self)  # Create a temporary guard.
 
+    security.declarePublic('renderGuardForm')
+    def renderGuardForm(self):
+        """Render the guard form"""
+        guard = self.getGuard() or self.getTempGuard()
+        if guard is not None:
+            return guard.guardForm()
+        return ''
+
+    security.declarePublic('guardExprDocs')
+    def guardExprDocs(self):
+        """Render the guard expression documentation"""
+        return self.cpsportlet_guard_tales_help()
+
     security.declareProtected(ManagePortlets, 'setGuardProperties')
-    def setGuardProperties(self, props={}, REQUEST=None):
+    def setGuardProperties(self, props=None, REQUEST=None):
         """Postprocess guard values."""
         if REQUEST is not None:
             # XXX Using REQUEST itself should work.
             # but update is complaining it is not a dictionary
+            if props is None:
+                props = {}
             props.update(REQUEST.form)
 
         # XXX found we must create a new instance every time.
         # not that much resource friendly...
         self.guard = PortletGuard()
-        self.guard.changeFromProperties(props)
+
+        psm = err = ''
+        try:
+            self.guard.changeFromProperties(props)
+        except CompilerError, e:
+            LOG('CPSPortlet:setGuardProperties', INFO, e)
+            psm = 'cpsportlet_psm_guard_error'
+            err = e
 
         if REQUEST is not None:
-            return self.manage_guardForm(REQUEST,
-                management_view='Guard',
-                manage_tabs_message='Guard setting changed.')
+            psm = psm or 'cpsportlet_psm_settings_updated'
+            return self.cpsportlet_guard(REQUEST, portal_status_message=psm,
+                                         err=err)
 
     security.declarePublic('SearchableText')
     def SearchableText(self):
@@ -272,12 +333,15 @@ class CPSPortlet(CPSDocument):
             res = []
             for o in p.split(':')[1].split(','):
                 if o[0] == '(' and o[-1] == ')':
-                    o = getattr(self, o[1:-1], None)
+                    o = getattr(aq_base(self), o[1:-1], None)
                     if o is None:
                         continue
                     if isinstance(o, (list, tuple)):
                         res.extend(o)
                         continue
+                    if isinstance(o, int):
+                        res.append(o)
+                        continue
                 res.append(str(o))
             return res
 
@@ -291,6 +355,13 @@ class CPSPortlet(CPSDocument):
             if param == 'no-cache':
                 return None, data
 
+            # disable the cache is a field value is True
+            elif param.startswith('no-cache:'):
+                opts = getOptions(param)
+                for opt in opts:
+                    if opt:
+                        return None, data
+
             # random integer
             elif param.startswith('random:'):
                 opts = getOptions(param)
@@ -321,7 +392,7 @@ class CPSPortlet(CPSDocument):
 
             # current user
             elif param == 'user':
-                index_string = str(REQUEST.get('AUTHENTICATED_USER'))
+                index_string = str(REQUEST.get('AUTHENTICATED_USER', ''))
 
             # current language
             elif param == 'current_lang':
@@ -335,6 +406,13 @@ class CPSPortlet(CPSDocument):
             elif param == 'baseurl':
                 index_string = REQUEST.get('cpsskins_base_url')
 
+            # protocol
+            elif param == 'protocol':
+                url = REQUEST.get('SERVER_URL', '')
+                pos = url.find('://')
+                if pos > 0:
+                    index_string = url[:pos]
+
             # CMF Actions by categories
             elif param.startswith('actions:'):
                 prefix = 'actions'
@@ -469,7 +547,7 @@ class CPSPortlet(CPSDocument):
 
             # current user if the cache entry is user-dependent
             if 'user' in self.getCacheParams():
-                user = REQUEST.get('AUTHENTICATED_USER')
+                user = str(REQUEST.get('AUTHENTICATED_USER', ''))
             else:
                 user = ''
 
@@ -743,6 +821,7 @@ class CPSPortlet(CPSDocument):
             self._interesting_events += ((event_ids,
                                           folder_paths,
                                           portal_types), )
+            self.reindexObject()
             return 0
         return 1
 
@@ -900,7 +979,11 @@ class CPSPortlet(CPSDocument):
         self.resetCacheTimeout()
 
         # reset the javascript methods
-        javascript_dict = self.getCPSPortletJavaScript()
+        try:
+            javascript_dict = self.getCPSPortletJavaScript()
+        except AttributeError:
+            javascript_dict = {}
+
         if javascript_dict.has_key(ptype_id):
             self._setJavaScript(javascript_dict[ptype_id])
 
@@ -914,9 +997,7 @@ class CPSPortlet(CPSDocument):
 
         if ptype_id is None:
             return
-        cache_params_dict = self.getCPSPortletCacheParams()
-        if not cache_params_dict.has_key(ptype_id):
-            return
+        ptltool = getToolByName(self, 'portal_cpsportlets')
 
         def getOptions(p):
             """extract cache parameter options
@@ -938,17 +1019,17 @@ class CPSPortlet(CPSDocument):
         event_ids = ()
         folder_paths = ()
         portal_types = ()
-        for param in cache_params_dict[ptype_id]:
+        for param in ptltool.getCacheParametersFor(ptype_id):
             # event ids
             if param.startswith('event_ids:'):
                 event_ids = getOptions(param)
 
             # folders in which the event occurs
-            if param.startswith('event_in_folders:'):
+            elif param.startswith('event_in_folders:'):
                 folder_paths = getOptions(param)
 
             # portal types on which the event occurs
-            if param.startswith('event_on_types:'):
+            elif param.startswith('event_on_types:'):
                 portal_types = getOptions(param)
 
         self.clearEvents()
@@ -962,12 +1043,10 @@ class CPSPortlet(CPSDocument):
     # ZMI
     #################################################################
 
-    security.declareProtected(ManagePortlets, 'manage_guardForm')
-    manage_guardForm = DTMLFile('zmi/manage_guardForm', globals())
+    security.declareProtected(ManagePortlets, 'manage_exportForm')
     manage_exportForm = DTMLFile('zmi/manage_exportForm', globals())
 
     manage_options = (CPSDocument.manage_options + (
-                      {'label': 'Guard', 'action': 'manage_guardForm'},
                       {'label': 'Export', 'action': 'manage_exportForm'},
                       )
                      )
