@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: CPSPortletsPermissions.py 13029 2005-03-16 10:50:09Z jmorliaguet $
+# $Id: CPSPortletsPermissions.py 31165 2005-12-31 00:32:00Z fguillaume $
 """CPS Portlets Permissions
 
 Here it's defined the specific permissions for the CPS Portlets
@@ -26,11 +26,7 @@ Here it's defined the specific permissions for the CPS Portlets
 'Manage Portlets': Permission to add new portlet at global scope (within portal)
 """
 
-# fallback to CMF 1.4
-try:
-    from Products.CMFCore.permissions import setDefaultRoles
-except ImportError:
-    from Products.CMFCore.CMFCorePermissions import setDefaultRoles
+from Products.CMFCore.permissions import setDefaultRoles
 
 ManagePortlets = 'Manage Portlets'
 setDefaultRoles(ManagePortlets, ('Manager', 'Owner'))
