@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-# $Id: PortletsTool.py 21994 2005-06-01 08:50:43Z jmorliaguet $
+# $Id: PortletsTool.py 32654 2006-02-08 15:35:34Z jmorliaguet $
 
 __author__ = "Julien Anguenot <mailto:ja@nuxeo.com>"
 
@@ -26,22 +26,19 @@ __author__ = "Julien Anguenot <mailto:ja@nuxeo.com>"
 
 from zLOG import LOG, DEBUG, ERROR
 
-from Globals import InitializeClass, DTMLFile
-from Globals import  PersistentMapping
 from AccessControl import ClassSecurityInfo, getSecurityManager, Unauthorized
 from Acquisition import aq_base, aq_parent, aq_inner
+from Globals import InitializeClass, DTMLFile
+from Globals import  PersistentMapping
 
-from Products.BTreeFolder2.CMFBTreeFolder import CMFBTreeFolder
-
-# Fallback to CMF 1.4
-try:
-    from Products.CMFCore.permissions import View
-except ImportError:
-    from Products.CMFCore.CMFCorePermissions import View
-
-from Products.CMFCore.utils import UniqueObject, getToolByName,\
-                                  _checkPermission
+from Products.CMFCore.CMFBTreeFolder import CMFBTreeFolder
+from Products.CMFCore.permissions import View
+from Products.CMFCore.utils import UniqueObject
+from Products.CMFCore.utils import getToolByName
+from Products.CMFCore.utils import _checkPermission
 
+from zope.interface import implements
+from Products.CPSPortlets.interfaces import IPortletTool
 from PortletRAMCache import RAMCache, SimpleRAMCache
 from PortletsContainer import PortletsContainer
 from CPSPortletsPermissions import ManagePortlets
@@ -53,7 +50,7 @@ PORTLET_RAMCACHE_ID = 'portlets'
 # Icons
 ICON_RAMCACHE_ID = 'icons'
 ACTIONICON_RAMCACHE_ID = 'actioncons'
-IMG_TAG = '<img src="%s" width="%s" height="%s" alt="%s" border="0" />'
+IMG_TAG = '<img src="%s" width="%s" height="%s" alt="%s" />'
 
 # FTI
 FTI_RAMCACHE_ID = 'fti'
@@ -66,6 +63,8 @@ class PortletsTool(UniqueObject, PortletsContainer):
     """ Portlets Tool
     """
 
+    implements(IPortletTool)
+
     id = 'portal_cpsportlets'
     meta_type = 'CPS Portlets Tool'
 
@@ -74,6 +73,12 @@ class PortletsTool(UniqueObject, PortletsContainer):
     # RAM Cache
     caches = {}
 
+    _properties = PortletsContainer._properties + (
+        {'id': 'ignore_events', 'type': 'boolean', 'mode': 'w',
+         'label': "Ignore events"},
+        )
+    ignore_events = False
+
     def __init__(self):
         self.initializeCacheParameters()
         CMFBTreeFolder.__init__(self, self.id)
@@ -81,35 +86,37 @@ class PortletsTool(UniqueObject, PortletsContainer):
     #
     # Catalog
     #
+
     security.declarePrivate('_getPortletCatalog')
     def _getPortletCatalog(self):
         """Return the portlet catalog
         """
-
-        # XXX change later to a dedicated catalog
-        catalog = getToolByName(self, 'portal_catalog')
+        catalog = getToolByName(self, 'portal_cpsportlets_catalog', None)
+        if catalog is None:
+            # BBB for installer
+            catalog = getToolByName(self, 'portal_catalog')
         return catalog
 
     security.declarePublic('listAllPortlets')
-    def listAllPortlets(self):
+    def listAllPortlets(self, event_ids=[]):
         """List all the portlets over the portal
 
-        Use the catalog.
+        Use the portlets catalog and the dedicated indexes for the
+        query.
         """
 
-        # Use the catalog to get all the portlets and their identifiers
-        catalog = self._getPortletCatalog()
-        portal_types = self.listPortletTypes()
+        query = {
+            'portal_type' : self.listPortletTypes(),
+            }
 
-        # Lookup through catalog to get the portlets
-        # We ask the tool to know all the portal_types
+        if event_ids:
+            query['eventIds'] = event_ids
 
         portlets = []
-        for res in catalog.searchResults(portal_type=portal_types):
+        for res in self._getPortletCatalog()(**query):
             portlet = res.getObject()
-            if portlet is None:
-                continue
-            portlets.append(portlet)
+            if portlet is not None:
+                portlets.append(portlet)
         return portlets
 
     security.declarePublic('listPortlets')
@@ -222,7 +229,9 @@ class PortletsTool(UniqueObject, PortletsContainer):
     def getPortletContainer(self, context=None, create=0, local=0):
         """Returns the portlet container within the given context if it
         exists. Otherwise return the tool
-        If the 'local' parameter is 1 then only local containers will be returned.
+
+        If the 'local' parameter is 1 then only local containers will
+        be returned.
         """
         if context is not None:
             container_id = self.getPortletContainerId()
@@ -324,7 +333,7 @@ class PortletsTool(UniqueObject, PortletsContainer):
 
     security.declarePublic('getPortlets')
     def getPortlets(self, context=None, slot=None, sort=1, override=1,
-                    visibility_check=1, **kw):
+                    visibility_check=1, guard_check=1, **kw):
         """Return a list of portlets.
         """
 
@@ -335,6 +344,8 @@ class PortletsTool(UniqueObject, PortletsContainer):
         bmf = self.getBottomMostFolder(context=context)
 
         # get portlets from the root to current path
+        # XXX This is badly reinventing traversal.
+        # XXX Use aq_parent(aq_inner(ob)) the other way round instead.
         utool = getToolByName(self, 'portal_url')
         rpath = utool.getRelativeContentPath(bmf)
         obj = utool.getPortalObject()
@@ -344,21 +355,22 @@ class PortletsTool(UniqueObject, PortletsContainer):
         for elem in ('',) + rpath:
             if not elem:
                 continue
-            obj = getattr(obj, elem)
+            obj = getattr(obj, elem, None)
+            if obj is None:
+                break
             allportlets.extend(self._getFolderPortlets(folder=obj, slot=slot))
 
         # list of portlets that will not be displayed
         remove_list = []
 
         # portlet guard and visibility range check
-        if visibility_check:
-            for portlet in allportlets:
-                if portlet.getGuard() and \
-                not portlet.getGuard().check(
-                    getSecurityManager(),
-                    portlet,
-                    context) or \
-                not self._isPortletVisible(portlet, context):
+        for portlet in allportlets:
+            if visibility_check:
+                if not self._isPortletVisible(portlet, context):
+                    remove_list.append(portlet)
+            if guard_check:
+                if portlet.getGuard() and not portlet.getGuard().check(
+                    getSecurityManager(), portlet, context):
                     remove_list.append(portlet)
 
         # portlet override
@@ -516,7 +528,11 @@ class PortletsTool(UniqueObject, PortletsContainer):
         if dest_slot is None:
             dest_slot = portlet.getSlot()
 
-        if not _checkPermission(ManagePortlets, src_folder) or \
+        if leave:
+            src_perm = View
+        else:
+            src_perm = ManagePortlets
+        if not _checkPermission(src_perm, src_folder) or \
             not _checkPermission(ManagePortlets, dest_folder):
             raise Unauthorized(
                 "You are not allowed to move portlets from %s to %s" %(
@@ -583,10 +599,12 @@ class PortletsTool(UniqueObject, PortletsContainer):
             return cache_parameters[ptype_id][:]
         return ['no-cache']
 
+    security.declareProtected(ManagePortlets, 'updateCacheParameters')
     def updateCacheParameters(self, params={}):
         """update the cache parameters
         """
         self.cache_parameters.update(params)
+        self._p_changed = 1
         # rebuild portlets
         self.rebuild_portlets()
 
@@ -775,17 +793,33 @@ class PortletsTool(UniqueObject, PortletsContainer):
         if img_tag is None:
             ttool = getToolByName(self, 'portal_types')
             ti = ttool.getTypeInfo(portal_type)
-            if ti is not None:
-                icon_path = ti.getIcon()
-                img = self.unrestrictedTraverse(icon_path, default=None)
-                if img is None:
-                    return None
-                img_tag = IMG_TAG % (base_url + icon_path,
-                                     getattr(img, 'width', 0),
-                                     getattr(img, 'height', 0),
-                                     alt)
-                if cache is not None:
-                    cache.setEntry(index, img_tag)
+            if ti is None:
+                return None
+            icon_path = ti.getIcon()
+
+            # see if an icon with the same name is present in the current
+            # theme's icon folder. The path must be relative and cannot contain
+            # any '/'.
+            if '/' not in icon_path:
+                icon_name = icon_path
+                tmtool = getToolByName(self, 'portal_themes')
+                utool = getToolByName(self, 'portal_url')
+                theme, page = tmtool.getEffectiveThemeAndPageName()
+                theme_container = tmtool.getThemeContainer(theme)
+
+                icon_folder = theme_container.icons
+                if icon_name in icon_folder.objectIds():
+                    icon_path = utool.getRpath(icon_folder) + '/' + icon_name
+
+            img = self.unrestrictedTraverse(icon_path, default=None)
+            if img is None:
+                return None
+            img_tag = IMG_TAG % (base_url + icon_path,
+                                 getattr(img, 'width', 0),
+                                 getattr(img, 'height', 0),
+                                 alt)
+            if cache is not None:
+                cache.setEntry(index, img_tag)
         return img_tag
 
     #
@@ -814,6 +848,22 @@ class PortletsTool(UniqueObject, PortletsContainer):
                 action_id=action_id)
             if not icon_path:
                 return
+
+            # see if an icon with the same name is present in the current
+            # theme's icon folder. The path must be relative and cannot contain
+            # any '/'.
+            if '/' not in icon_path:
+                icon_name = icon_path
+                tmtool = getToolByName(self, 'portal_themes')
+                utool = getToolByName(self, 'portal_url')
+                theme, page = tmtool.getEffectiveThemeAndPageName()
+                theme_container = tmtool.getThemeContainer(theme)
+
+                icon_folder = theme_container.icons
+                if icon_name in icon_folder.objectIds():
+                    icon_path = utool.getRpath(icon_folder) + '/' + icon_name
+
+            # fall back to the icon path specified in CMFActionIcons
             img = self.unrestrictedTraverse(icon_path, default=None)
             if img is None:
                 return None
@@ -1129,6 +1179,9 @@ class PortletsTool(UniqueObject, PortletsContainer):
     def notify_event(self, event_type, object, infos):
         """Standard event hook
         """
+        if self.ignore_events:
+            return
+
         ##LOG(":: CPS Portlets Tool :: ", DEBUG, 'notify_event()')
         ##LOG(":: EVENT TYPE ::", DEBUG, event_type)
         ##LOG(":: OBJECT ::", DEBUG, repr(object))
@@ -1143,9 +1196,9 @@ class PortletsTool(UniqueObject, PortletsContainer):
 
         # expire the portlets interested in the event
         # ZEO-aware invalidation.
-        for portlet in self.listPortletsInterestedInEvent(event_id=event_type,
-                                                          folder_path=object_path,
-                                                          portal_type=portal_type):
+        for portlet in self.listPortletsInterestedInEvent(
+            event_id=event_type, folder_path=object_path,
+            portal_type=portal_type):
             portlet.expireCache()
 
         # remove all cache entries of a user who logs out.
@@ -1158,14 +1211,22 @@ class PortletsTool(UniqueObject, PortletsContainer):
         """Return the list of all portlets interested in an event given its
         event_id
         """
-
         returned = []
-        portlets = self.listAllPortlets()
+        portlets = self.listAllPortlets(event_ids=[event_id])
+
+        # Here the portlets are already filtered by events right.
         for portlet in portlets:
-            if portlet.isInterestedInEvent(event_id=event_id,
-                                           folder_path=folder_path,
-                                           portal_type=portal_type):
-                returned.append(portlet)
+            try:
+                # XXX event_id is given as parameter but is useless.
+                interested = portlet.isInterestedInEvent(
+                    event_id=event_id, folder_path=folder_path,
+                    portal_type=portal_type)
+            except AttributeError:
+                # Not a portlet even if it claimed to be one in its FTI
+                continue
+            else:
+                if interested:
+                    returned.append(portlet)
         return returned
 
 InitializeClass(PortletsTool)
