@@ -14,10 +14,12 @@ cache_params = {
         'object:published_path',
         'objects:relative_path',
         'timeout:300',
+        'request:breadcrumb_set',
         ],
     'Actions Portlet': ['actions:(categories)', 'current_lang'],
-    'Content Portlet': ['user',
-        'event_ids:workflow_create,workflow_publish,workflow_modify,sys_del_object,workflow_cut_copy_paste,workflow_copy_submit',
+    'Content Portlet': ['no-cache:(contextual)',
+        'user',
+        'event_ids:workflow_create,workflow_accept,workflow_reject,workflow_publish,workflow_modify,sys_del_object,workflow_cut_copy_paste,workflow_copy_submit',
         'event_in_folders:(folder_path)',
         'event_on_types:(searchable_types)',
         'baseurl',
