@@ -1,6 +1,7 @@
 mtool = context.portal_membership
 utool = context.portal_url
-base_url = context.cpsskins_getBaseUrl()
+# Note: this will not work in CMF
+base_url = utool.getBaseUrl()
 
 rpath = utool.getRelativeContentPath(context)
 parent = context
