@@ -1,5 +1,5 @@
 # (c) 2003 Nuxeo SARL <http://nuxeo.com>
-# $Id: testRSS.py 21660 2005-05-19 13:13:41Z tziade $
+# $Id: testRSS.py 28712 2005-10-27 14:33:52Z tziade $
 
 import unittest, os.path
 import CPSRSSTestCase
@@ -30,12 +30,6 @@ class TestRSSTool(CPSRSSTestCase.CPSRSSTestCase):
     def testChannelNotLazy(self):
         self._testChannel(lazy_refresh=0)
 
-    def test_timeout_picker(self):
-        import sys
-        python_version = sys.version_info[0], sys.version_info[1]
-        if python_version[0] == 2 and python_version[1] >= 3:
-            self.assert_('timeoutsocket' not in sys.modules.keys())
-
 def test_suite():
     suites = [unittest.makeSuite(TestRSSTool)]
     return unittest.TestSuite(suites)
