@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: BasicFields.py 21920 2005-05-30 12:59:57Z madarche $
+# $Id: BasicFields.py 31669 2006-01-16 13:18:16Z jmorliaguet $
 """BasicFields
 
 Definition of standard field types.
@@ -23,10 +23,7 @@ Definition of standard field types.
 
 from zLOG import LOG, DEBUG, WARNING
 import sys
-from types import IntType, FloatType, LongType, BooleanType, \
-                  StringType, UnicodeType, \
-                  ListType, DictType, TupleType
-
+import warnings
 from Globals import InitializeClass
 from DateTime.DateTime import DateTime
 
@@ -41,14 +38,16 @@ from Products.CPSSchemas.FileUtils import convertFileToHtml
 from Products.CPSSchemas.FileUtils import convertFileToText
 from Products.CPSSchemas.DiskFile import DiskFile
 
+from zope.interface import implements
+from Products.CPSSchemas.interfaces import IFileField
+from Products.CPSSchemas.interfaces import IFieldNodeIO
+
 
 def _isinstance(ob, cls):
-    try:
-        return isinstance(ob, cls)
-    except TypeError:
-        # In python 2.1 isinstance() raises TypeError
-        # instead of returning 0 for ExtensionClasses.
-        return 0
+    warnings.warn("_isinstance() is deprecated and will be removed in "
+                  "CPS 3.4.1. Use isinstance() instead.",
+                  DeprecationWarning, stacklevel=2)
+    return isinstance(ob, cls)
 
 #
 # UTF-8
@@ -59,7 +58,7 @@ if default_encoding == 'ascii':
     default_encoding = 'iso-8859-15'
 
 def toUTF8(s):
-    if not isinstance(s, UnicodeType):
+    if not isinstance(s, unicode):
         s = unicode(s, default_encoding)
     return s.encode('utf-8')
 
@@ -71,13 +70,17 @@ class CPSIntField(CPSField):
     """Integer field."""
     meta_type = "CPS Int Field"
 
+    implements(IFieldNodeIO)
+
     default_expr = 'python:0'
     default_expr_c = Expression(default_expr)
 
     # XXX this is never called yet.
     def validate(self, value):
-        if isinstance(value, IntType):
+        if isinstance(value, int):
             return value
+        if isinstance(value, bool):
+            return int(value)
         raise ValidationError('Not an integer: %s' % repr(value))
 
     def convertToLDAP(self, value):
@@ -94,6 +97,17 @@ class CPSIntField(CPSField):
             raise ValidationError("Incorrect Int value from LDAP: %s"
                                   % `values`)
 
+    def setNodeValue(self, node, value, context):
+        """See IFieldNodeIO.
+        """
+        context.setNodeValue(node, str(int(value)))
+
+    def getNodeValue(self, node, context):
+        """See IFieldNodeIO.
+        """
+        text = context.getNodeValue(node)
+        return int(text)
+
 InitializeClass(CPSIntField)
 
 
@@ -101,12 +115,14 @@ class CPSBooleanField(CPSField):
     """Boolean field."""
     meta_type = "CPS Boolean Field"
 
+    implements(IFieldNodeIO)
+
     default_expr = 'python:False'
     default_expr_c = Expression(default_expr)
 
     # XXX this is never called yet.
     def validate(self, value):
-        if isinstance(value, BooleanType):
+        if isinstance(value, bool):
             return value
         raise ValidationError('Not a boolean: %s' % repr(value))
 
@@ -124,35 +140,31 @@ class CPSBooleanField(CPSField):
             raise ValidationError("Incorrect Boolean value from LDAP: %s"
                                   % `values`)
 
+    def setNodeValue(self, node, value, context):
+        """See IFieldNodeIO.
+        """
+        context.setNodeValue(node, str(bool(value)))
+
+    def getNodeValue(self, node, context):
+        """See IFieldNodeIO.
+        """
+        text = context.getNodeValue(node)
+        return context._convertToBoolean(text)
+
 InitializeClass(CPSBooleanField)
 
 
-class CPSLongField(CPSField):
-    """Long field."""
+class CPSLongField(CPSIntField):
+    """Long field
+
+    This field is DEPRECATED, use the identical Int Field instead.
+    """
     meta_type = "CPS Long Field"
-
-    default_expr = 'python:0L'
-    default_expr_c = Expression(default_expr)
-
-    # XXX this is never called yet.
-    def validate(self, value):
-        if isinstance(value, LongType):
-            return value
-        raise ValidationError('Not an long integer: %s' % repr(value))
-
-    def convertToLDAP(self, value):
-        """Convert a value to LDAP attribute values."""
-        return [str(value)]
-
-    def convertFromLDAP(self, values):
-        """Convert a value from LDAP attribute values."""
-        try:
-            if len(values) != 1:
-                raise ValueError
-            return long(values[0])
-        except (ValueError, TypeError):
-            raise ValidationError("Incorrect Long value from LDAP: %s"
-                                  % `values`)
+    def __init__(self, id, **kw):
+        warnings.warn("The Long Field (%s) is deprecated and will be "
+                      "removed in CPS 3.5.0. Use a Int Field instead" %
+                      id, DeprecationWarning)
+        CPSIntField.__init__(self, id, **kw)
 
 InitializeClass(CPSLongField)
 
@@ -161,12 +173,14 @@ class CPSFloatField(CPSField):
     """Float field."""
     meta_type = "CPS Float Field"
 
+    implements(IFieldNodeIO)
+
     default_expr = 'python:0.0'
     default_expr_c = Expression(default_expr)
 
     # XXX this is never called yet.
     def validate(self, value):
-        if isinstance(value, FloatType):
+        if isinstance(value, float):
             return value
         raise ValidationError('Not an real number: %s' % repr(value))
 
@@ -184,6 +198,23 @@ class CPSFloatField(CPSField):
             raise ValidationError("Incorrect Float value from LDAP: %s"
                                   % `values`)
 
+    def setNodeValue(self, node, value, context):
+        """See IFieldNodeIO.
+        """
+        if value is None:
+           value = ""
+        else:
+           value = str(value)
+        context.setNodeValue(node, value)
+
+    def getNodeValue(self, node, context):
+        """See IFieldNodeIO.
+        """
+        text = context.getNodeValue(node)
+        if text:
+            return float(text)
+        return None
+
 InitializeClass(CPSFloatField)
 
 
@@ -191,12 +222,14 @@ class CPSStringField(CPSField):
     """String field."""
     meta_type = "CPS String Field"
 
+    implements(IFieldNodeIO)
+
     default_expr = 'string:'
     default_expr_c = Expression(default_expr)
 
     # XXX this is never called yet.
     def validate(self, value):
-        if isinstance(value, StringType):
+        if isinstance(value, str):
             return value
         raise ValidationError('Not a string: %s' % repr(value))
 
@@ -223,6 +256,17 @@ class CPSStringField(CPSField):
             pass
         return value
 
+    def setNodeValue(self, node, value, context):
+        """See IFieldNodeIO.
+        """
+        context.setNodeValue(node, value.decode(default_encoding))
+
+    def getNodeValue(self, node, context):
+        """See IFieldNodeIO.
+        """
+        text = context.getNodeValue(node)
+        return text.encode(default_encoding)
+
 InitializeClass(CPSStringField)
 
 
@@ -232,6 +276,7 @@ class CPSPasswordField(CPSStringField):
 
 InitializeClass(CPSPasswordField)
 
+
 class CPSListField(CPSField):
     """Meta list Field"""
     meta_type = "CPS List Field"
@@ -242,7 +287,7 @@ class CPSListField(CPSField):
 
     # XXX this is never called yet.
     def validate(self, value):
-        if isinstance(value, ListType):
+        if isinstance(value, list):
             for v in value:
                 if not self.verifyType(v):
                     raise ValidationError(self.validation_error_msg +
@@ -268,15 +313,18 @@ class CPSListField(CPSField):
 
 InitializeClass(CPSListField)
 
+
 class CPSStringListField(CPSListField):
     """String List field."""
     meta_type = "CPS String List Field"
 
+    implements(IFieldNodeIO)
+
     validation_error_msg = 'Not a string list: '
 
     def verifyType(self, value):
         """Verify the type of the value"""
-        return isinstance(value, StringType)
+        return isinstance(value, str)
 
     def convertToLDAP(self, value):
         """Convert a value to LDAP attribute values."""
@@ -296,26 +344,51 @@ class CPSStringListField(CPSListField):
             res.append(v)
         return res
 
+    def setNodeValue(self, node, value, context):
+        """See IFieldNodeIO.
+        """
+        for v in value:
+            child = context.createStrictTextElement('e')
+            v = v.decode(default_encoding)
+            context.setNodeValue(child, v)
+            node.appendChild(child)
+
+    def getNodeValue(self, node, context):
+        """See IFieldNodeIO.
+        """
+        res = []
+        for child in node.childNodes:
+            if child.nodeName != 'e':
+                continue
+            v = context.getNodeValue(child)
+            v = v.encode(default_encoding)
+            res.append(v)
+        return res
+
 InitializeClass(CPSStringListField)
 
 class CPSListListField(CPSListField):
-    """List of List field."""
+    """List of List field.
+
+    This doesn't give enough type information so is not really
+    registered as a field type. This is an abstract base class.
+    """
     meta_type = "CPS List List Field"
 
     validation_error_msg = 'Not a list of list: '
 
     # XXX this is never called yet.
     def validate(self, value):
-        if isinstance(value, ListType):
-            for list in value:
-                if isinstance(list, ListType):
-                    for e in list:
+        if isinstance(value, list):
+            for l in value:
+                if isinstance(l, list):
+                    for e in l:
                         if not self.verifyType(e):
                             raise ValidationError(self.validation_error_msg +
                                             repr(value) + '=>' + repr(e))
                 else:
                     raise ValidationError(self.validation_error_msg +
-                                          repr(value) + '=>' + repr(list))
+                                          repr(value) + '=>' + repr(l))
             return value
         raise ValidationError(self.validation_error_msg + repr(value))
 
@@ -339,7 +412,7 @@ class CPSIntListListField(CPSListListField):
 
     def verifyType(self, value):
         """Verify the type of the value"""
-        return isinstance(value, IntType)
+        return isinstance(value, int)
 
 InitializeClass(CPSIntListListField)
 
@@ -347,6 +420,8 @@ class CPSDateTimeField(CPSField):
     """DateTime field."""
     meta_type = "CPS DateTime Field"
 
+    implements(IFieldNodeIO)
+
     default_expr = 'nothing'
     default_expr_c = Expression(default_expr)
 
@@ -409,6 +484,26 @@ class CPSDateTimeField(CPSField):
             raise ValidationError("Incorrect DateTime value from LDAP: %s" %
                                   `values`)
 
+    def setNodeValue(self, node, value, context):
+        """See IFieldNodeIO.
+        """
+        v = self.convertToLDAP(value)
+        if v:
+            v = v[0]
+        else:
+            v = ''
+        context.setNodeValue(node, v)
+
+    def getNodeValue(self, node, context):
+        """See IFieldNodeIO.
+        """
+        v = context.getNodeValue(node)
+        if v:
+            value = self.convertFromLDAP([v])
+        else:
+            value = None
+        return value
+
 InitializeClass(CPSDateTimeField)
 
 
@@ -416,6 +511,8 @@ class CPSFileField(CPSField):
     """File field."""
     meta_type = "CPS File Field"
 
+    implements(IFieldNodeIO, IFileField)
+
     default_expr = 'nothing'
     default_expr_c = Expression(default_expr)
 
@@ -490,7 +587,7 @@ class CPSFileField(CPSField):
     def validate(self, value):
         if not value:
             return None
-        if _isinstance(value, File):
+        if isinstance(value, File):
             return value
         raise ValidationError('Not a file: %s' % repr(value))
 
@@ -509,6 +606,17 @@ class CPSFileField(CPSField):
                                   "(%d-element list)" % len(values))
         return File(self.getFieldId(), '', values[0])
 
+    def setNodeValue(self, node, value, context):
+        """See IFieldNodeIO.
+        """
+        return
+
+    def getNodeValue(self, node, context):
+        """See IFieldNodeIO.
+        """
+        # Will be later initialized as a subobject.
+        return None
+
 InitializeClass(CPSFileField)
 
 class CPSDiskFileField(CPSFileField):
@@ -540,7 +648,7 @@ class CPSDiskFileField(CPSFileField):
         """
         field_id = self.getFieldId()
         file = data[field_id] # May be None.
-        if _isinstance(file, File) and not _isinstance(file, DiskFile):
+        if isinstance(file, File) and not isinstance(file, DiskFile):
             file = DiskFile(file.getId(), file.title, file.data,
                             file.content_type, self.getStoragePath())
             data[field_id] = file
@@ -582,7 +690,7 @@ class CPSDiskFileField(CPSFileField):
     def validate(self, value):
         if not value:
             return None
-        if _isinstance(value, File):
+        if isinstance(value, File):
             return value
         raise ValidationError('Not a file: %s' % repr(value))
 
@@ -639,7 +747,7 @@ class CPSSubObjectsField(CPSField):
     def validate(self, value):
         if value is None:
             return None
-        if _isinstance(value, DictType):
+        if isinstance(value, dict):
             return value
         raise ValidationError('Not SubObjects: %s' % repr(value))
 
@@ -651,6 +759,8 @@ class CPSImageField(CPSField):
     """Image field."""
     meta_type = "CPS Image Field"
 
+    implements(IFieldNodeIO, IFileField)
+
     default_expr = 'nothing'
     default_expr_c = Expression(default_expr)
 
@@ -658,7 +768,7 @@ class CPSImageField(CPSField):
     def validate(self, value):
         if not value:
             return None
-        if _isinstance(value, Image):
+        if isinstance(value, Image):
             return value
         raise ValidationError('Not an image: %s' % repr(value))
 
@@ -677,6 +787,17 @@ class CPSImageField(CPSField):
                                   "(%d-element list)" % len(values))
         return Image(self.getFieldId(), '', values[0])
 
+    def setNodeValue(self, node, value, context):
+        """See IFieldNodeIO.
+        """
+        return
+
+    def getNodeValue(self, node, context):
+        """See IFieldNodeIO.
+        """
+        # Will be later initialized as a subobject.
+        return None
+
 InitializeClass(CPSImageField)
 
 class CPSRangeListField(CPSListField):
@@ -687,9 +808,9 @@ class CPSRangeListField(CPSListField):
 
     # XXX this is never called yet.
     def validate(self, value):
-        if isinstance(value, ListType):
+        if isinstance(value, list):
             for v in value:
-                if not isinstance(v, TupleType):
+                if not isinstance(v, tuple):
                     raise ValidationError(self.validation_error_msg +
                                           "%s is not a tuple in %s" %
                                           (repr(v),
@@ -710,19 +831,21 @@ class CPSRangeListField(CPSListField):
 
     def verifyType(self, value):
         """Verify the type of the value"""
-        return isinstance(value, IntType)
+        return isinstance(value, int)
 
 InitializeClass(CPSRangeListField)
 
 class CPSCoupleField(CPSListField):
     """CPS Couple Field
 
-    Holds two values within a list (whatever types)
-    The order matters in a couple for the comparaison.
-    """
+    Holds two non-negative integers within a list. The order matters.
 
+    XXX should be relaxed to allow a tuple (safer), see CPSPortlets.
+    """
     meta_type = "CPS Couple Field"
 
+    implements(IFieldNodeIO)
+
     validation_error_message = "Not a couple : "
 
     def _getValidationErrorMessage(self, value):
@@ -735,26 +858,43 @@ class CPSCoupleField(CPSListField):
         Has to be a list of 2 integer values within a list
         """
 
-        if (isinstance(value, ListType) and
+        if (isinstance(value, list) and
             len(value) == 2):
             return value
 
         # Default case
-        elif (isinstance(value, ListType) and
+        elif (isinstance(value, list) and
               len(value) == 0):
             return value
 
         raise ValidationError(self._getValidationErrorMessage(value))
 
+    def setNodeValue(self, node, value, context):
+        """See IFieldNodeIO.
+        """
+        context.setNodeValue(node, '%d-%d' % (value[0], value[1]))
+
+    def getNodeValue(self, node, context):
+        """See IFieldNodeIO.
+        """
+        text = context.getNodeValue(node)
+        v0, v1 = text.split('-', 1)
+        return [int(v0), int(v1)]
+
+InitializeClass(CPSCoupleField)
+
+
+
 # Register field classes
 
 FieldRegistry.register(CPSBooleanField)
 FieldRegistry.register(CPSStringField)
 FieldRegistry.register(CPSPasswordField)
+#FieldRegistry.register(CPSListListField) # Not registered on purpose
 FieldRegistry.register(CPSIntListListField)
 FieldRegistry.register(CPSStringListField)
 FieldRegistry.register(CPSIntField)
-FieldRegistry.register(CPSLongField)
+FieldRegistry.register(CPSLongField) # deprecated
 FieldRegistry.register(CPSFloatField)
 FieldRegistry.register(CPSDateTimeField)
 FieldRegistry.register(CPSFileField)
