@@ -16,7 +16,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: Layout.py 15161 2005-03-25 18:47:13Z ogrisel $
+# $Id: Layout.py 30974 2005-12-26 16:20:47Z fguillaume $
 """Layout
 
 A layout describes how to render a set of widgets.
@@ -35,11 +35,16 @@ from Products.CMFCore.permissions import ViewManagementScreens
 from Products.CMFCore.utils import SimpleItemWithProperties
 from Products.CMFCore.utils import getToolByName
 from Products.CMFCore.Expression import getEngine
+from Products.CMFCore.Expression import SecureModuleImporter
 
+from Products.CPSUtil.PropertiesPostProcessor import PropertiesPostProcessor
 from Products.CPSSchemas.FolderWithPrefixedIds import FolderWithPrefixedIds
-from Products.CPSSchemas.WidgetTypesTool import WidgetTypeRegistry
-from Products.CPSSchemas.PropertiesPostProcessor import PropertiesPostProcessor
 from Products.CPSSchemas.DataModel import ReadAccessError
+from Products.CPSSchemas.Widget import widgetRegistry
+
+from zope.interface import implements
+from Products.CPSSchemas.interfaces import ILayout
+
 
 class LayoutContainer(Folder):
     """Layout Tool
@@ -120,26 +125,25 @@ class Layout(PropertiesPostProcessor,
     """
 
     _properties = (
-        {'id': 'style_prefix', 'type': 'string', 'mode': 'w',
-         'label': 'Prefix for zpt'},
         {'id': 'layout_create_method', 'type': 'string', 'mode': 'w',
-         'label': 'ZPT in charge of layout when in create mode'},
+         'label': 'Layout method for create mode'},
         {'id': 'layout_edit_method', 'type': 'string', 'mode': 'w',
-         'label': 'ZPT in charge of layout when in edit mode'},
+         'label': 'Layout method for edit mode'},
         {'id': 'layout_view_method', 'type': 'string', 'mode': 'w',
-         'label': 'ZPT in charge of layout when in view mode'},
+         'label': 'Layout method for view mode'},
+        {'id': 'style_prefix', 'type': 'string', 'mode': 'w',
+         'label': 'Layout method prefix for default'},
         {'id': 'flexible_widgets', 'type': 'tokens', 'mode': 'w',
          'label': 'Allowed widgets in flexible'},
         {'id': 'validate_values_expr', 'type': 'text', 'mode': 'w',
-         'label': 'Validates layout values with the given expression, \
-             see documentation'},
+         'label': 'Layout validation expression'},
         )
 
     style_prefix = 'layout_default_'
     layout_create_method = ''
     layout_edit_method = ''
     layout_view_method = ''
-    flexible_widgets = []
+    flexible_widgets = ()
     validate_values_expr = ''
     validate_values_expr_c = None
 
@@ -278,6 +282,7 @@ class Layout(PropertiesPostProcessor,
          - widget_css_class
          - widget_rendered
          - widget_javascript
+         - widget_input_area_id
         (In addition to widget_id and ncols of the standard data.)
         """
         layout_structure = self.getLayoutDefinition() # get a copy
@@ -291,11 +296,19 @@ class Layout(PropertiesPostProcessor,
                 mode = widget.getModeFromLayoutMode(layout_mode, datamodel)
                 css_class = widget.getCssClass(layout_mode, datamodel)
                 js_code = widget.getJavaScriptCode(layout_mode, datamodel)
+                # Information about a potential input area is important for
+                # accessibility: it is used to associate the widget label with a
+                # potential input area.
+                if widget.has_input_area and mode != 'view':
+                    input_area_id = widget.getHtmlWidgetId()
+                else:
+                    input_area_id = None
                 widgets[widget_id] = {
                     'widget': widget,
                     'widget_mode': mode,
                     'widget_css_class': css_class,
                     'widget_javascript': js_code,
+                    'widget_input_area_id': input_area_id,
                     }
             else:
                 widgets[widget_id] = {
@@ -356,6 +369,7 @@ class Layout(PropertiesPostProcessor,
             'kw': kw,
             'nothing': None,
             'portal': portal,
+            'modules': SecureModuleImporter,
             }
         return getEngine().getContext(data)
 
@@ -387,6 +401,8 @@ class Layout(PropertiesPostProcessor,
                           **kw):
         """Applies the layout style method to the rendered widgets.
 
+        ``context`` is used to find the layout method.
+
         Returns the rendered string. The rendered string is computed according
         to the following properties and in decreasing order of precedence:
         "layout_create_method", "layout_edit_method", "layout_view_method"
@@ -438,16 +454,14 @@ class Layout(PropertiesPostProcessor,
                                 **kw)
         return rendered
 
-    def __repr__(self):
-        return '<Layout %s>' % `self.getLayoutDefinition()`
-
-
 InitializeClass(Layout)
 
 
 class CPSLayout(Layout):
     """Persistent Layout."""
 
+    implements(ILayout)
+
     meta_type = "CPS Layout"
 
     security = ClassSecurityInfo()
@@ -457,11 +471,10 @@ class CPSLayout(Layout):
         Layout.__init__(self, **kw)
 
     security.declarePrivate('addWidget')
-    def addWidget(self, id, wtid, **kw):
+    def addWidget(self, id, meta_type, **kw):
         """Add a new widget instance."""
-        wtool = getToolByName(self, 'portal_widget_types')
-        widget_type = wtool[wtid]
-        widget = widget_type.makeInstance(id, **kw)
+        class_ = widgetRegistry.getClass(meta_type)
+        widget = class_(id, **kw)
         return self.addSubObject(widget)
 
     #
@@ -471,19 +484,18 @@ class CPSLayout(Layout):
     def all_meta_types(self):
         # List of meta types contained in Folder, for copy/paste support.
         return [
-            {'name': WidgetTypeRegistry.getClass(wt).meta_type,
+            {'name': mt,
              'action': '',
              'permission': ManagePortal}
-            for wt in WidgetTypeRegistry.listWidgetTypes()]
+            for mt in widgetRegistry.listWidgetMetaTypes()]
 
     def filtered_meta_types(self):
         # List of types available in Folder menu.
-        wtool = getToolByName(self, 'portal_widget_types')
         return [
-            {'name': wtid,
-             'action': 'manage_addCPSWidgetForm/'+wtid.replace(' ', ''),
+            {'name': mt,
+             'action': 'manage_addCPSWidgetForm/'+mt.replace(' ', ''),
              'permission': ManagePortal}
-            for wtid in wtool.objectIds()]
+            for mt in widgetRegistry.listWidgetMetaTypes()]
 
     manage_options = (
         {'label': 'Widgets',
@@ -507,26 +519,24 @@ class CPSLayout(Layout):
     security.declareProtected(ManagePortal, 'manage_addCPSWidgetForm')
     manage_addCPSWidgetForm = DTMLFile('zmi/widget_addform', globals())
 
-    security.declareProtected(ManagePortal, 'getUnstrippedWidgetTypeId')
-    def getUnstrippedWidgetTypeId(self, swtid):
-        """Get an unstripped version of a widget type id."""
-        wtool = getToolByName(self, 'portal_widget_types')
-        twtid = swtid.replace(' ', '')
-        for wtid in wtool.objectIds():
-            if wtid.replace(' ', '') == twtid:
-                return wtid
-        raise ValueError(swtid)
+    security.declareProtected(ManagePortal, 'getUnstrippedWidgetMetaType')
+    def getUnstrippedWidgetMetaType(self, smt):
+        """Get an unstripped version of a widget meta type."""
+        smt = smt.replace(' ', '')
+        for mt in widgetRegistry.listWidgetMetaTypes():
+            if mt.replace(' ', '') == smt:
+                return mt
+        raise ValueError(smt)
 
     security.declareProtected(ManagePortal, 'manage_addCPSWidget')
-    def manage_addCPSWidget(self, id, swtid, REQUEST=None, **kw):
+    def manage_addCPSWidget(self, id, meta_type, REQUEST=None, **kw):
         """Add a widget, called from the ZMI."""
         if REQUEST is not None:
             kw.update(REQUEST.form)
-            for key in ('id', 'swtid'):
-                if kw.has_key(key):
+            for key in ('id', 'meta_type'):
+                if key in kw:
                     del kw[key]
-        wtid = self.getUnstrippedWidgetTypeId(swtid)
-        widget = self.addWidget(id, wtid, **kw)
+        widget = self.addWidget(id, meta_type, **kw)
         if REQUEST is not None:
             REQUEST.RESPONSE.redirect(widget.absolute_url()+
                                       '/manage_workspace')
