@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: SchemasTool.py 13712 2003-09-08 17:49:36Z fguillaume $
+# $Id: SchemasTool.py 30964 2005-12-26 00:43:09Z fguillaume $
 """Schemas Tool
 """
 
@@ -26,6 +26,9 @@ from Products.CMFCore.utils import UniqueObject
 
 from Products.CPSSchemas.Schema import SchemaContainer
 
+from zope.interface import implements
+from Products.CPSSchemas.interfaces import ISchemaTool
+
 
 class SchemasTool(UniqueObject, SchemaContainer):
     """Schemas Tool
@@ -34,6 +37,8 @@ class SchemasTool(UniqueObject, SchemaContainer):
     A schema describes a set of fields that can store data.
     """
 
+    implements(ISchemaTool)
+
     id = 'portal_schemas'
     meta_type = 'CPS Schemas Tool'
 
