@@ -1,11 +1,5 @@
 ##parameters=proxy, fileName, widgetId
-# $Id: getExternalEditorPath.py 14871 2004-11-05 13:06:05Z atchertchian $
-
-from zLOG import LOG, DEBUG
-
-log_key = 'getExternalEditorPath'
-
-LOG(log_key, DEBUG, "proxy=%s, fileName = %s, widgetId = %s" % (proxy, fileName, widgetId))
+# $Id: getExternalEditorPath.py 30782 2005-12-19 16:01:10Z fguillaume $
 
 portal = context.portal_url.getPortalObject()
 portalAbsoluteUrl = portal.absolute_url()
@@ -24,6 +18,4 @@ external_editor_path = ('%s/%sexternalEdit_?path=%s/downloadFile/%s/%s' %
                         (portalAbsoluteUrl, prefix, proxyPhysicalPath,
                          widgetId, fileName))
 
-LOG(log_key, DEBUG, "external_editor_path = %s" % external_editor_path)
-
 return external_editor_path
