@@ -1,5 +1,5 @@
 ##parameters=mode, datastructure
-# $Id: widget_dummy_prepare_validate.py 14036 2003-11-21 14:00:35Z bdelbosc $
+# $Id: widget_dummy_prepare_validate.py 29531 2005-11-15 11:07:47Z fguillaume $
 
 datamodel = datastructure.getDataModel()
 
@@ -9,7 +9,8 @@ field_id = context.fields[0]
 if mode == 'prepare':
     datastructure.set(widget_id, datamodel[field_id])
     return
-else: # Validate
+
+if mode == 'validate':
     value = datastructure.get(widget_id, '')
     if not value.startswith('dummy'):
         datastructure.setError(widget_id,
