@@ -1,5 +1,8 @@
 ##parameters=mode, datastructure
-# $Id: widget_my_prepare_validate.py 14044 2003-11-21 15:31:56Z bdelbosc $
+# $Id: widget_my_prepare_validate.py 29531 2005-11-15 11:07:47Z fguillaume $
+"""
+XXX add docstring
+"""
 
 datamodel = datastructure.getDataModel()
 
@@ -9,7 +12,8 @@ field_id = context.fields[0]
 if mode == 'prepare':
     datastructure.set(widget_id, datamodel[field_id])
     return
-else: # Validate
+
+if mode == 'validate':
     value = datastructure.get(widget_id, '')
     if not value.startswith('my'):
         datastructure.setError(widget_id,
