@@ -1,12 +1,15 @@
 # -*- coding: iso-8859-15 -*-
 # (c) 2003 Nuxeo SARL <http://nuxeo.com>
-# $Id: testWidgetsValidation.py 21572 2005-05-16 00:37:28Z janguenot $
+# $Id: testWidgetsValidation.py 32985 2006-02-17 19:32:41Z fguillaume $
 
 import os
 import unittest
-from Testing.ZopeTestCase import ZopeLite
 
+from Acquisition import Implicit
 from OFS.Image import File
+from OFS.Folder import Folder
+
+from ZPublisher.HTTPRequest import FileUpload
 
 from Products.CPSSchemas.DataStructure import DataStructure
 from Products.CPSSchemas.BasicWidgets import CPSStringWidget, \
@@ -16,9 +19,62 @@ from Products.CPSSchemas.ExtendedWidgets import CPSRangeListWidget, \
      CPSTextWidget
 
 from Products.CPSSchemas.ExtendedWidgets import CPSFlashWidget
+from Products.CPSSchemas import tests as cpsschemas_tests
+TEST_SWF = os.path.join(cpsschemas_tests.__path__[0], 'test.swf')
 
-from Products.CPSSchemas import tests
-TEST_SWF = os.path.join(tests.__path__[0], 'test.swf')
+from Products.CPSSchemas.ExtendedWidgets import CPSDateTimeWidget
+
+
+class FakePortal(Implicit):
+    pass
+fakePortal = FakePortal()
+
+class FakeTranslationService:
+    def getSelectedLanguage(self):
+        return 'fr'
+fakePortal.translation_service = FakeTranslationService()
+
+class FakeMimeTypeRegistry(Implicit):
+    def lookupExtension(self, name):
+        mimetypes = {}
+        mimetypes['SWF'] = 'application/x-shockwave-flash'
+        extension = name[name.rfind('.')+1:].upper()
+        if extension in mimetypes:
+            return mimetypes[extension]
+        else:
+            return 'testlookup/'+name[name.rfind('.')+1:].upper()
+
+class FakeFieldStorage:
+    def __init__(self, file, filename, headers=None):
+        self.file = file
+        self.filename = filename
+        self.headers = headers or {}
+    def read(self, n):
+        return self.file.read(n)
+    def seek(self, n, m):
+        return self.file.seek(n, m)
+
+class FakeAdapter(object):
+    def __init__(self, schema):
+        self.schema = schema
+    def getSchema(self):
+        return self.schema
+    def _getContentUrl(self, a, b, c=None):
+        return 'http://url for %s %s %s' % (a, b, c)
+
+class FakeDataModel(dict):
+    _adapters = None
+    proxy = None
+    context = None
+    def __init__(self, dm=None):
+        if dm is not None:
+            self.update(dm)
+    def getProxy(self):
+        return self.proxy
+    def getObject(self):
+        return self.proxy
+    def getContext(self):
+        return self.context
 
 class WidgetValidationTest(unittest.TestCase):
     """Tests validate method of widgets"""
@@ -30,8 +86,7 @@ class WidgetValidationTest(unittest.TestCase):
         data = {id: value}
         ds = DataStructure(data, datamodel=data)
         properties.update({'fields': (id,)})
-        widget = self.widget_type(id, self.default_value)
-        widget.manage_changeProperties(**properties)
+        widget = self.widget_type(id, **properties).__of__(fakePortal)
 
         ret = widget.validate(ds)
         err = ds.getError(id)
@@ -63,8 +118,10 @@ class FloatWidgetValidationTest(WidgetValidationTest):
         self.assert_(ret, err)
 
     def test_float_nok_1(self):
-        ret, err, ds = self._validate({}, '12345,803')
-        self.assert_(ret, err)
+        # this would work with locale fr_FR et etc/zope.conf...
+        # ret, err, ds = self._validate({}, '12345,803')
+        ret, err, ds = self._validate({}, '12345;803')
+        self.assert_(err)
 
     def test_float_nok_2(self):
         ret, err, ds = self._validate({'decimals_separator': ','}, '12345.803')
@@ -151,30 +208,37 @@ class BooleanWidgetValidationTest(WidgetValidationTest):
         ret, err, ds = self._validate({}, 1)
         self.assert_(ret, err)
 
-    def test_boolean_nok_1(self):
+    def test_boolean_ok_3(self):
+        ret, err, ds = self._validate({}, False)
+        self.assert_(ret, err)
+
+    def test_boolean_ok_4(self):
+        ret, err, ds = self._validate({}, True)
+        self.assert_(ret, err)
+
+    def test_boolean_ok_5(self):
         ret, err, ds = self._validate({}, 2)
-        self.assertEquals(err, 'cpsschemas_err_boolean')
+        self.assert_(ret, err)
 
-    def test_boolean_nok_2(self):
+    def test_boolean_ok_6(self):
         ret, err, ds = self._validate({}, -1)
-        self.assertEquals(err, 'cpsschemas_err_boolean')
+        self.assert_(ret, err)
 
-# XXX bug to fix
-#    def test_boolean_nok_3(self):
-#        ret, err, ds = self._validate({}, '')
-#        self.assertEquals(err, 'cpsschemas_err_boolean')
+    def test_boolean_ok_7(self):
+        ret, err, ds = self._validate({}, '')
+        self.assert_(ret, err)
 
-    def test_boolean_nok_4(self):
+    def test_boolean_ok_8(self):
         ret, err, ds = self._validate({}, None)
-        self.assertEquals(err, 'cpsschemas_err_boolean')
+        self.assert_(ret, err)
 
-    def test_boolean_nok_5(self):
+    def test_boolean_ok_9(self):
         ret, err, ds = self._validate({}, 'foo')
-        self.assertEquals(err, 'cpsschemas_err_boolean')
+        self.assert_(ret, err)
 
-    def test_boolean_nok_5(self):
+    def test_boolean_ok_10(self):
         ret, err, ds = self._validate({}, {'foo': 'sk'})
-        self.assertEquals(err, 'cpsschemas_err_boolean')
+        self.assert_(ret, err)
 
 class TextWidgetValidationTest(WidgetValidationTest):
     widget_type = CPSTextWidget
@@ -424,7 +488,18 @@ class PasswordWidgetValidationTest(WidgetValidationTest):
         pass
 
     def test_password_ok_required_1(self):
-        ret, err, ds = self._validate({'is_required': 0}, '')
+        # default value for size_min is 5
+        ret, err, ds = self._validate({'is_required': 0, 'size_min': 0}, '')
+        self.assertEquals(err, None)
+
+    def test_password_ok_required_2(self):
+        # default value for size_min is 5
+        ret, err, ds = self._validate({'is_required': 0, 'size_min': 0}, 'here')
+        self.assertEquals(err, None)
+
+    def test_password_ok_required_3(self):
+        # Even if there is a size_min, this should not fail
+        ret, err, ds = self._validate({'is_required': 0, 'size_min': 5}, '')
         self.assertEquals(err, None)
 
     def test_password_nok_required_1(self):
@@ -439,6 +514,12 @@ class PasswordWidgetValidationTest(WidgetValidationTest):
         ret, err, ds = self._validate({'size_max': 8}, 'foobarfo')
         self.assertEquals(err, None)
 
+    def test_password_ok_size_3(self):
+        # Non-required password fields should not raise an error if empty,
+        # even if min size is set.
+        ret, err, ds = self._validate({'size_min': 5}, '')
+        self.assertEquals(err, None)
+
     def test_password_nok_size_1(self):
         ret, err, ds = self._validate({'size_min': 5}, 'foob')
         self.assertEquals(err, 'cpsschemas_err_password_size_min')
@@ -447,6 +528,10 @@ class PasswordWidgetValidationTest(WidgetValidationTest):
         ret, err, ds = self._validate({'size_max': 8}, 'foobarfoo')
         self.assertEquals(err, 'cpsschemas_err_string_too_long')
 
+    def test_password_nok_size_3(self):
+        ret, err, ds = self._validate({'size_min': 5}, 'four')
+        self.assertEquals(err, 'cpsschemas_err_password_size_min')
+
     def test_password_ok_lower_1(self):
         ret, err, ds = self._validate({'check_lower': 1}, 'FoE1.A')
         self.assertEquals(err, None)
@@ -511,26 +596,69 @@ class FlashWidgetValidationTest(WidgetValidationTest):
 
     widget_type = CPSFlashWidget
 
-    data = open(TEST_SWF, 'r').read()
-    default_value = File('fake', '', data)
-    default_value.content_type = 'application/x-shockwave-flash'
+    f = open(TEST_SWF, 'r')
+    default_value = FileUpload(FakeFieldStorage(f, 'test.swf'))
+    default_value._p_mtime = ''
 
     def _validate(self, properties, value):
         id = 'ff'
-        data = {id: value}
-        ds = DataStructure(data, datamodel=data)
+        choice = 'change'
+        title = 'title'
+        
+        data = {id: value, id+'_choice': choice, id+'_title': title}
+        dm = FakeDataModel(data)
+        dm._adapters = [FakeAdapter({id: 'foo'})]
+        ds = DataStructure(data, datamodel=dm)
         properties.update({'fields': (id,),})
-        widget = self.widget_type(id, self.default_value)
-        widget.manage_changeProperties(**properties)
+        
+        folder = Folder()
+        folder.mimetypes_registry = FakeMimeTypeRegistry()
+        
+        widget = self.widget_type(id, **properties).__of__(folder)
 
         # Just test the internal validation related to swf
         ret = widget._flash_validate(ds)
         err = ds.getError(id)
         return ret, err, ds
-    
+
     def test_widget_nok_required_1(self):
         pass
-    
+
+class DateTimeWidgetValidationTest(WidgetValidationTest):
+    widget_type = CPSDateTimeWidget
+    default_value = ''
+
+    # base class validation method is not compatible with this widget
+    def _validate(self, properties, value):
+        id = 'ff'
+        data = {
+            id + '_date': value,
+            # dummy values for hour and minute, they're not tested here
+            id + '_hour': '3',
+            id + '_minute': '38',
+            }
+        ds = DataStructure(data, datamodel=data)
+        properties.update({'fields': (id,)})
+        widget = self.widget_type(id, **properties).__of__(fakePortal)
+        ret = widget.validate(ds)
+        err = ds.getError(id)
+        return ret, err, ds
+
+    def test_datetime_ok_1(self):
+        ret, err, ds = self._validate({}, '27/06/2005')
+        self.assertEquals(ret, 1)
+
+    def test_datetime_nok_1(self):
+        ret, err, ds = self._validate({}, '26/27/2005')
+        self.assertEquals(ret, 0)
+        self.assertEquals(err, 'cpsschemas_err_date')
+
+    def test_datetime_nok_2(self):
+        ret, err, ds = self._validate({}, '26/2705k')
+        self.assertEquals(ret, 0)
+        self.assertEquals(err, 'cpsschemas_err_date')
+
+
 # XXX: test more widget types here
 
 def test_suite():
