@@ -19,23 +19,23 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: install.py 21365 2005-03-16 10:09:14Z madarche $
+# $Id: install.py 25337 2005-08-01 14:32:23Z janguenot $
 
 """ CPSSubscriptions Installer
 
 Installer/Updater fot the CPSSubscriptions component.
 """
 
-from zLOG import LOG, INFO, DEBUG
-
 from Products.ExternalMethod.ExternalMethod import ExternalMethod
 
 from Products.CMFCore.utils import getToolByName
 from Products.CMFCore.permissions import View
 from Products.CPSInstaller.CPSInstaller import CPSInstaller
 
-from Products.CPSSubscriptions.CPSSubscriptionsPermissions import \
-     ManageSubscriptions, CanSubscribe, ViewMySubscriptions, CanNotifyContent
+from Products.CPSSubscriptions.permissions import ManageSubscriptions
+from Products.CPSSubscriptions.permissions import CanSubscribe
+from Products.CPSSubscriptions.permissions import ViewMySubscriptions
+from Products.CPSSubscriptions.permissions import CanNotifyContent
 
 SECTIONS_ID = 'sections'
 WORKSPACES_ID = 'workspaces'
