@@ -1,5 +1,5 @@
 # -*- coding: ISO-8859-15 -*-
-# Copyright (c) 2004 Nuxeo SARL <http://nuxeo.com>
+# Copyright (c) 2004-2005 Nuxeo SARL <http://nuxeo.com>
 # Copyright (c) 2004 CGEY <http://cgey.com>
 # Copyright (c) 2004 Ministere de L'interieur (MISILL)
 #               <http://www.interieur.gouv.fr/>
@@ -19,7 +19,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-# $Id: Notifications.py 21453 2005-05-06 18:23:55Z atchertchian $
+# $Id: Notifications.py 32678 2006-02-09 13:48:54Z lregebro $
 
 __author__ = "Julien Anguenot <mailto:ja@nuxeo.com>"
 
@@ -32,37 +32,30 @@ Notifications are subclasses of NotificationRule and store also as
 subobjects, like RecipientsRule.
 """
 
-from smtplib import SMTPException
-
-# Trying to import the TimeOut error class of CPSRSS is installed
-try:
-    from Products.CPSRSS.timeoutsocket import Timeout
-except ImportError:
-    class Timeout:
-        pass
-
 import socket
 import cStringIO
 import string
 import mimify
 import mimetools
 import MimeWriter
+from smtplib import SMTPException
+from urllib import urlencode
 
-from types import StringType, TupleType
-
-from Globals import InitializeClass, MessageDialog
-from Products.MailHost.MailHost import MailHostError
-from Acquisition import aq_base, aq_parent, aq_inner
 from AccessControl import getSecurityManager
 from AccessControl import ClassSecurityInfo
+from Acquisition import aq_base, aq_parent, aq_inner
+from Globals import InitializeClass
+from Globals import MessageDialog
+
+from Products.MailHost.MailHost import MailHostError
 
 from Products.CMFCore.permissions import ManagePortal
 from Products.CMFCore.utils import getToolByName
 from Products.CMFCore.PortalFolder import PortalFolder
 
-from zLOG import LOG, INFO, DEBUG
+from zLOG import LOG, DEBUG, ERROR, TRACE
 
-logKey = 'CPSSubscriptions'
+logKey = 'CPSSubscriptions.Notifications'
 
 class NotificationRule(PortalFolder):
     """Base Notification rule Class.
@@ -83,6 +76,20 @@ class NotificationRule(PortalFolder):
         """
         return aq_parent(aq_inner(self))
 
+    def getSubscriptionContainer(self):
+        """Return the placefull subscription container this notification is in.
+
+        A notification is supposed to be into a 'Subscription Configuration'
+        object, itself in a 'PlaceFull Subscription Container' object.
+        """
+        res = None
+        subscription = self.getParentSubscription()
+        if subscription is not None:
+            container = subscription.getParentContainer()
+            if container is not None:
+                res = container
+        return res
+
 InitializeClass(NotificationRule)
 
 ################################################################
@@ -107,7 +114,12 @@ class MailNotificationRule(NotificationRule):
         writer = MimeWriter.MimeWriter(rendered_message)
 
         # Sender
-        sender = '"%s" <%s>' % (infos['sender_email'], infos['sender_email'])
+        if infos.get('sender_name'):
+            sender = '"%s" <%s>' % (infos['sender_name'],
+                                    infos['sender_email'])
+        else:
+            sender = '"%s" <%s>' % (infos['sender_email'],
+                                    infos['sender_email'])
         writer.addheader('From', sender)
 
         # Subject
@@ -120,6 +132,10 @@ class MailNotificationRule(NotificationRule):
         # To
         writer.addheader(string.capitalize('to'),
                          mimify.mime_encode_header(infos['to']))
+        # Bcc
+        if infos.get('bcc'):
+            writer.addheader(string.capitalize('bcc'),
+                             mimify.mime_encode_header(infos['bcc']))
 
         # Misc
         writer.addheader('X-Mailer', 'Nuxeo CPS 3 : CPSSubscriptions')
@@ -148,10 +164,10 @@ class MailNotificationRule(NotificationRule):
         """Validate the mail_infos structure
         """
 
-        return  (isinstance(mail_infos.get('sender_email'), StringType) and
-                 isinstance(mail_infos.get('to'), StringType) and
-                 isinstance(mail_infos.get('subject'), StringType) and
-                 isinstance(mail_infos.get('body'), TupleType) and
+        return  (isinstance(mail_infos.get('sender_email'), str) and
+                 isinstance(mail_infos.get('to'), str) and
+                 isinstance(mail_infos.get('subject'), str) and
+                 isinstance(mail_infos.get('body'), tuple) and
                  len(mail_infos.get('body')) == 2)
 
     def sendMail(self, mail_infos, object=None, event_id=None, mailhost=None):
@@ -164,7 +180,7 @@ class MailNotificationRule(NotificationRule):
         # It could be build by the user with whatever stuffs within.
         if not self._validateStructure(mail_infos):
             LOG("::  CPSSubscriptions  :: sendMail() :: for",
-                INFO,
+                ERROR,
                 "Error while sending mail",
                 "check the email of the recipients")
             return -1
@@ -172,7 +188,7 @@ class MailNotificationRule(NotificationRule):
         raw_message = self.getRawMessage(mail_infos, object, event_id)
 
         LOG(":: CPSSubscriptions :: sendMail() :: for",
-            INFO,
+            DEBUG,
             raw_message)
 
         try:
@@ -180,21 +196,31 @@ class MailNotificationRule(NotificationRule):
                 mailhost.send(raw_message)
             else:
                 self.MailHost.send(raw_message)
-        except (socket.error, MailHostError, SMTPException, Timeout):
+        except (socket.error, MailHostError, SMTPException):
             LOG("::  CPSSubscriptions  :: sendMail() :: for",
-                INFO,
+                ERROR,
                 "Error while sending mail",
                 "check your SMTP parameters or mailfrom address")
 
-    def _getMailFrom(self, infos):
-        """ Return an email for the mail from field of the mail.
+    def _getMailSenderInfo(self, infos):
+        """Return mail sender information (name and email)
+
+        Try to get email from infos and from the subscriptions container
+        instead of default values.
         """
-        subscription = self.getParentSubscription()
-        if subscription is not None:
-            container = subscription.getParentContainer()
-            if container is not None:
-                return container.getMailFrom()
-        return "no_mail@nomail.com"
+        # default values: admin name, admin email
+        substool = getToolByName(self, 'portal_subscriptions')
+        # XXX AT: maybe use the portal title instead of the portal admin
+        # name to send the email (?)
+        sender_email, sender_name = substool.getMailSenderInfo()
+        # get info from 'infos' dict
+        if infos.has_key('mfrom'):
+            sender_email = infos.get('mfrom')
+        # get from container
+        container = self.getSubscriptionContainer()
+        if container is not None:
+            sender_email = container.getMailFrom()
+        return sender_email, sender_name
 
     def _getSubject(self, infos):
         """ Returns the subject of the email.
@@ -213,7 +239,10 @@ class MailNotificationRule(NotificationRule):
             # modules of Python can't hndle properly non-ASCII in
             # subject header.
             subject = subject.translate(_translation_table)
-        except (KeyError, TypeError):
+        except (KeyError, TypeError), e:
+            LOG('CPSSubscriptions', ERROR,
+                "Error in subject notification template for %r: %s"
+                % (infos.get('event'), str(e)))
             # If the user put wrong variables
             subject = "No Subject"
 
@@ -224,14 +253,15 @@ class MailNotificationRule(NotificationRule):
 
         Is proccessed with the infos given as parameters.
         """
-
         try:
             body = self.portal_subscriptions.getDefaultMessageBody(
                 event_id=infos['event']) % infos
-        except (KeyError, TypeError, ValueError):
+        except (KeyError, TypeError, ValueError), e:
+            LOG('CPSSubscriptions', ERROR,
+                "Error in body notification template with infos %r: %s"
+                % (infos, str(e)))
             # If the user put wrong variables
             body = self.portal_subscriptions.getErrorMessageBody()
-
         return body
 
     def _makeInfoDict(self, event_type, object, infos=None):
@@ -243,22 +273,26 @@ class MailNotificationRule(NotificationRule):
         if infos is None:
             infos = {}
 
-        portal_subscriptions = getToolByName(self, 'portal_subscriptions')
-        memberDirectory = getToolByName(self, 'portal_directories').members
+        stool = getToolByName(self, 'portal_subscriptions')
+        members = getToolByName(self, 'portal_directories').members
         ttool = getToolByName(self, 'portal_types')
 
         context = aq_parent(aq_inner(object))
-        mcat = self.Localizer.default
+        mcat = self.translation_service
 
-        events_from_context = portal_subscriptions.getEventsFromContext(context)
+        events_from_context = stool.getEventsFromContext(context)
 
         # Just more secure in case of the event configuration is badly done.
-        if events_from_context is None:
+        if not events_from_context:
             return {}
-        event_from_context = mcat(
-            events_from_context.get(event_type,
-                                    event_type)).encode('ISO-8859-15',
-                                                        'ignore')
+
+        event_type_trsl = mcat(events_from_context.get(event_type, event_type))
+        if event_type_trsl is not None:
+            event_type_trsl = event_type_trsl.encode('ISO-8859-15', 'ignore')
+        else:
+            event_type_trsl = event_type
+
+        event_from_context = event_type_trsl
 
         portal = self.portal_url.getPortalObject()
         infos['portal_url'] = portal.absolute_url()
@@ -273,17 +307,18 @@ class MailNotificationRule(NotificationRule):
 
         # i18n type title
         type_title = ttool[infos['object_type']].Title()
-        i18n_type_title = mcat(type_title).encode('ISO-8859-15', 'ignore')
-        if  i18n_type_title != type_title:
-            infos['object_type'] = i18n_type_title
+        i18n_type_title = mcat(type_title)
+        if i18n_type_title is not None:
+            i18n_type_title = i18n_type_title.encode('ISO-8859-15', 'ignore')
+            if  i18n_type_title != type_title:
+                infos['object_type'] = i18n_type_title
 
         object_parent = aq_parent(aq_inner(object))
         infos['object_parent_title'] = object_parent.Title()
         infos['object_parent_url'] = object_parent.absolute_url()
 
         object_creator_id = object.Creator()
-        object_creator_user = memberDirectory.getEntry(object_creator_id,
-                                                       default=None)
+        object_creator_user = members.getEntry(object_creator_id, default=None)
         object_creator_name = ''
         if object_creator_user:
             object_creator_name = object_creator_user.get('fullname')
@@ -306,16 +341,17 @@ class MailNotificationRule(NotificationRule):
         # Including information about the user whose action is at the origin of
         # the event.
         user_id = getSecurityManager().getUser().getId()
-        user = memberDirectory.getEntry(user_id, default=None)
+        user = members.getEntry(user_id, default=None)
         user_name = ''
         if user:
             user_name = user.get('fullname')
         infos['user_id'] = user_id
         infos['user_name'] = user_name
 
-        # Making sure that there is always an available "comments" variable so
-        # that this variable is always available for all email message bodies and
-        # thus will prevent producing KeyError errors.
+        # Making sure that there is always an available "comments"
+        # variable so that this variable is always available for all
+        # email message bodies and thus will prevent producing
+        # KeyError errors.
         if infos.get('comments') is None:
             infos['comments'] = ''
 
@@ -330,50 +366,43 @@ class MailNotificationRule(NotificationRule):
         if infos.get('kwargs_comments') is not None:
             infos['comments'] = infos['kwargs_comments']
 
-        LOG('CPSSubscriptions', DEBUG, "available infos in emails: %s" % infos)
+        LOG('CPSSubscriptions', TRACE, "available infos in emails: %s" % infos)
         return infos
 
     security.declareProtected(ManagePortal, 'notifyRecipients')
-    def notifyRecipients(self,
-                         event_type,
-                         object,
-                         infos=None,
-                         emails=[],
-                         members=[],
-                         groups=[],
-                         **kw):
+    def notifyRecipients(self, event_type, object_, infos=None, emails=[],
+                         members=[], groups=[], **kw):
         """ Notify recipients
 
         This method will be called by the Subscription object when a
         notification occurs.
+
+        This method id aware about the fact that for some recipients
+        we won't send a email directly but store the email for further
+        scheduling. (daily, monthly, ...)
         """
 
-        portal = getToolByName(self, 'portal_url').getPortalObject()
-        subscriptions_tool = getToolByName(portal, 'portal_subscriptions')
+        # Construct a mapping for the email notification
+        infos = self._makeInfoDict(event_type, object_, infos)
+        if not infos:
+            # No notification info for that object
+            return
 
-        infos = self._makeInfoDict(event_type, object, infos)
-        mfrom = self._getMailFrom(infos)
-        subject = self._getSubject(infos)
-
-        #
-        # Let's check if we render the content because of the portal_type
-        # or because of the event id.
-        # This is defined tool side.
-        #
-
-        subscriptions_tool = getToolByName(self, 'portal_subscriptions')
-
-        rendered_portal_types = subscriptions_tool.getRenderedPortalTypes()
-        rendered_events = subscriptions_tool.getRenderedEvents()
-
-        if (object is not None and
-            getattr(object, 'portal_type', None) in rendered_portal_types or
-            infos.get('event') in rendered_events):
-            try:
-                body = object.getContent().render(proxy=object)
+        # Let's check if we render the content, instead of a regular
+        # user defined notification message, because of the
+        # portal_type or because of the event id.  This is defined
+        # subscriptions tool side.
+        stool = getToolByName(self, 'portal_subscriptions')
+        rendered_ptypes = stool.getRenderedPortalTypes()
+        rendered_events = stool.getRenderedEvents()
+        if (object_ is not None and
+            getattr(object, 'portal_type', '') in rendered_ptypes or
+            infos.get('event', '') in rendered_events):
+            # Is the object_ a CPSDocument ?
+            if hasattr(aq_parent(aq_inner(object_)), 'render'):
+                body = object_.getContent().render(proxy=object_)
                 mime_type = 'text/html'
-            except AttributeError:
-                # Not a CPSDocument
+            else:
                 # XXX : we might handle whatever sort of content for rendering
                 # in here. Using the main_template flag maybe.
                 body = self._getBody(infos)
@@ -382,42 +411,46 @@ class MailNotificationRule(NotificationRule):
             body = self._getBody(infos)
             mime_type = 'text/plain'
 
-        LOG(":: CPSSubscriptions :: MailNotificationRule :: on",
-            INFO,
-            infos)
+        # Save the email notification body for furher scheduling.
+        # The scheduling table is stored on the subscriptions tool for now.
+        archive_id = stool.addNotificationMessageBodyObject(body, mime_type)
 
-        # Save the email notification body
-        archive_id = subscriptions_tool.addNotificationMessageBodyObject(body,
-                                                                         mime_type)
-
-        # Dealing with emails
-        # XXX should send all the email at the same time with bcc
+        # Filter for the recipients we need to notify `real time` For
+        # the other ones, we will add an entry within the scheduling
+        # table
+        real_time = []
         for email in emails:
-            mail_infos = {}
-            mail_infos['sender_name'] = portal.Title()
-            mail_infos['sender_email'] = mfrom
-            mail_infos['subject'] = subject
-            mail_infos['to'] = email
-            mail_infos['body'] = (body, mime_type)
-
-            # Check user_mode and take actions
-            container = subscriptions_tool.getSubscriptionContainerFromContext(self)
+            container = stool.getSubscriptionContainerFromContext(self)
             user_mode = container.getUserMode(email)
-
             if user_mode != 'mode_real_time':
-                # Store the message_id within the scheduling table for a given user.
-                subscriptions_tool.scheduleNotificationMessageFor(user_mode,
-                                                                  email,
-                                                                  archive_id)
+                stool.scheduleNotificationMessageFor(user_mode, email,
+                                                     archive_id)
             else:
-                # Send the notification message
-                self.sendMail(mail_infos, object, event_id=infos['event'])
+                real_time.append(email)
 
-        # Dealing with members
+        # Send all the emails in batches for those in `real time` mode
+        max_recipients = stool.max_recipients_per_notification
+        semail, sname = self._getMailSenderInfo(infos)
+        while real_time:
+            batch = real_time[:max_recipients]
+            real_time = real_time[max_recipients:]
+            bcc = ','.join(batch)
+            mail_infos = {
+                'sender_name': sname,
+                'sender_email': semail,
+                'subject': self._getSubject(infos),
+                'to': semail,
+                'bcc': bcc,
+                'body': (body, mime_type),
+                }
+            self.sendMail(mail_infos, object_, event_id=infos.get('event', ''))
+            LOG(logKey + ' notifyRecipients()', TRACE, repr(mail_infos))
+
+        # TODO: Dealing with members
         for member in members:
             pass
 
-        # Dealing with groups
+        # TODO: Dealing with groups
         for group in groups:
             pass
 
@@ -434,31 +467,36 @@ class MailNotificationRule(NotificationRule):
         tool = getToolByName(self, 'portal_subscriptions')
         container = tool.getSubscriptionContainerFromContext(context)
         portal = getToolByName(self,'portal_url').getPortalObject()
+        url_args = {
+            'fake': 'subscriptions',
+            'event_id': event_id,
+            'email': email,
+            }
+        url_args = urlencode(url_args)
         object_url = context.absolute_url() \
-                     + "/folder_confirm_subscribe_form?fake=subscriptions" \
-                     + "&event_id=" \
-                     + event_id \
-                     + "&email=" \
-                     + email
-
+                     + "/folder_confirm_subscribe_form?%s" %(url_args,)
 
         # Pre process for body/subject
-        infos = {'portal_title': portal.Title(),
-                 'object_url'  : object_url,
-                 'event_id'    : event_id,
-                 'email'       : email,
-                 'mfrom'       : container.getMailFrom()}
+        infos = {
+            'portal_title': portal.Title(),
+            'object_url'  : object_url,
+            'event_id'    : event_id,
+            'email'       : email,
+            'mfrom'       : container.getMailFrom(),
+            }
 
         subject = tool.getSubscribeConfirmEmailTitle() % infos
         body = tool.getSubscribeConfirmEmailBody() % infos
+        semail, sname = self._getMailSenderInfo(infos)
 
         # For building the E-Mail
-        mail_infos = {}
-        mail_infos['sender_name'] = infos.get('portal_title')
-        mail_infos['sender_email'] = infos.get('mfrom', 'no_mail@no_mail.com')
-        mail_infos['subject'] = subject
-        mail_infos['to'] = email
-        mail_infos['body'] = (body, 'text/plain')
+        mail_infos = {
+            'sender_name': sname,
+            'sender_email': semail,
+            'subject': subject,
+            'to': email,
+            'body': (body, 'text/plain'),
+            }
 
         # Send mail then.
         self.sendMail(mail_infos)
@@ -478,29 +516,32 @@ class MailNotificationRule(NotificationRule):
         object_url = context.absolute_url()
 
         # Pre process for body/subject
-        infos = {'portal_title': portal.Title(),
-                 'object_url'  : object_url,
-                 'object_title': context.title_or_id(),
-                 'info_url'    : info_url,
-                 'event_id'    : event_id,
-                 'email'       : email,
-                 'mfrom'       : container.getMailFrom()}
+        infos = {
+            'portal_title': portal.Title(),
+            'object_url'  : object_url,
+            'object_title': context.title_or_id(),
+            'info_url'    : info_url,
+            'event_id'    : event_id,
+            'email'       : email,
+            'mfrom'       : container.getMailFrom(),
+            }
 
         subject = tool.getSubscribeWelcomeEmailTitle() % infos
         body = tool.getSubscribeWelcomeEmailBody() % infos
+        semail, sname = self._getMailSenderInfo(infos)
 
         # Post process
         infos['body'] =  body
         infos['subject'] = subject.replace('\n', '')
 
         # For building the E-Mail
-        mail_infos = {}
-        mail_infos['sender_name'] = infos.get('portal_title')
-        mail_infos['sender_email'] = infos.get('mfrom', 'no_mail@no_mail.com')
-        mail_infos['subject'] = infos.get('subject', 'No Subject')
-        mail_infos['to'] = email
-        mail_infos['body'] = (infos.get('body', ''), 'text/plain')
-
+        mail_infos = {
+            'sender_name': sname,
+            'sender_email': semail,
+            'subject': infos.get('subject', 'No Subject'),
+            'to': email,
+            'body': (infos.get('body', ''), 'text/plain'),
+            }
         # Send mail then.
         self.sendMail(mail_infos)
 
@@ -522,28 +563,32 @@ class MailNotificationRule(NotificationRule):
         object_url = context.absolute_url()
 
         # infos contains information needed to generated messages
-        infos = {'portal_title': portal.Title(),
-                 'object_url'  : object_url,
-                 'object_title': context.title_or_id(),
-                 'info_url'    : info_url,
-                 'event_id'    : event_id,
-                 'email'       : email,
-                 'mfrom'       : container.getMailFrom()}
+        infos = {
+            'portal_title': portal.Title(),
+            'object_url'  : object_url,
+            'object_title': context.title_or_id(),
+            'info_url'    : info_url,
+            'event_id'    : event_id,
+            'email'       : email,
+            'mfrom'       : container.getMailFrom(),
+            }
 
         subject = tool.getUnSubscribeEmailTitle() % infos
         body = tool.getUnSubscribeEmailBody() % infos
+        semail, sname = self._getMailSenderInfo(infos)
 
         # Post process
         infos['body'] =  body
         infos['subject'] = subject.replace('\n', '')
 
         # For building the E-Mail
-        mail_infos = {}
-        mail_infos['sender_name'] = infos.get('portal_title')
-        mail_infos['sender_email'] = infos.get('mfrom', 'no_mail@no_mail.com')
-        mail_infos['subject'] = infos.get('subject', 'No Subject')
-        mail_infos['to'] = email
-        mail_infos['body'] = (infos.get('body', ''), 'text/plain')
+        mail_infos = {
+            'sender_name': sname,
+            'sender_email': semail,
+            'subject': infos.get('subject', 'No Subject'),
+            'to': email,
+            'body': (infos.get('body', ''), 'text/plain'),
+            }
 
         # Send mail then.
         self.sendMail(mail_infos)
@@ -561,38 +606,43 @@ class MailNotificationRule(NotificationRule):
 
         info_url = context.absolute_url() + '/folder_subscribe_form'
         object_url = context.absolute_url()
-        url = context.absolute_url() \
-              + '/folder_confirm_unsubscribe_form?fake=subscriptions' \
-              '&event_id='\
-              + event_id \
-              + '&email='\
-              + email
+        url_args = {
+            'fake': 'subscriptions',
+            'event_id': event_id,
+            'email': email,
+            }
+        url_args = urlencode(url_args)
+        object_url = context.absolute_url() \
+                     + "/folder_confirm_unsubscribe_form?%s" %(url_args,)
 
         # infos contains information needed to generated messages
-        infos = {'portal_title': portal.Title(),
-                 'object_url'  : object_url,
-                 'url'         : url,
-                 'object_title': context.title_or_id(),
-                 'info_url'    : info_url,
-                 'event_id'    : event_id,
-                 'email'       : email,
-                 'mfrom'       : container.getMailFrom()}
+        infos = {
+            'portal_title': portal.Title(),
+            'object_url'  : object_url,
+            'url'         : info_url,
+            'object_title': context.title_or_id(),
+            'info_url'    : info_url,
+            'event_id'    : event_id,
+            'email'       : email,
+            'mfrom'       : container.getMailFrom(),
+            }
 
         subject = tool.getUnSubscribeConfirmEmailTitle() % infos
         body = tool.getUnSubscribeConfirmEmailBody() % infos
+        semail, sname = self._getMailSenderInfo(infos)
 
         # Post process
         infos['body'] =  body
         infos['subject'] = subject.replace('\n', '')
 
         # For building the E-Mail
-        mail_infos = {}
-        mail_infos['sender_name'] = infos.get('portal_title')
-        mail_infos['sender_email'] = infos.get('mfrom', 'no_mail@no_mail.com')
-        mail_infos['subject'] = infos.get('subject', 'No Subject')
-        mail_infos['to'] = email
-        mail_infos['body'] = (infos.get('body', ''), 'text/plain')
-
+        mail_infos = {
+            'sender_name': sname,
+            'sender_email': semail,
+            'subject': infos.get('subject', 'No Subject'),
+            'to': email,
+            'body': (infos.get('body', ''), 'text/plain'),
+            }
         # Send mail then.
         self.sendMail(mail_infos)
 
