@@ -19,7 +19,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-# $Id: RecipientsRules.py 21442 2005-05-06 14:55:32Z atchertchian $
+# $Id: RecipientsRules.py 30928 2005-12-22 18:34:54Z ogrisel $
 
 __author__ = "Julien Anguenot <mailto:ja@nuxeo.com>"
 
@@ -29,7 +29,7 @@ Classes defining how to compute recipients. They are stored within
 the subscription container.
 """
 
-from types import TupleType
+from re import compile
 from DateTime.DateTime import DateTime
 
 from Globals import InitializeClass, MessageDialog, DTMLFile
@@ -44,10 +44,10 @@ from Products.CMFCore.permissions import View
 from Products.CMFCore.Expression import Expression
 from Products.CMFCore.Expression import getEngine
 
-from CPSSubscriptionsPermissions import CanSubscribe, ManageSubscriptions
-
-from zLOG import LOG, DEBUG, INFO
+from Products.CPSSubscriptions.permissions import CanSubscribe
+from Products.CPSSubscriptions.permissions import ManageSubscriptions
 
+from zLOG import LOG, ERROR
 
 class RecipientsRule(PortalFolder):
     """Recipients Rule Class.
@@ -236,7 +236,7 @@ class ExplicitRecipientsRule(RecipientsRule):
 
         Returns a list of ids
         """
-        return self.members
+        return list(self.members)
 
     security.declarePublic("getMemberIds")
     def getMemberIds(self, context=None):
@@ -255,7 +255,7 @@ class ExplicitRecipientsRule(RecipientsRule):
                         continue
             return res
 
-    security.declareProtected(ManageSubscriptions, "getMemberStrucById")
+    security.declareProtected(ManageSubscriptions, "getMemberStructById")
     def getMemberStructById(self, member_id):
         """Return the index of member_id in the list
         """
@@ -436,8 +436,10 @@ class ExplicitRecipientsRule(RecipientsRule):
         """Add the list of emails to the subscriber list
         """
         self._p_changed = 1
+        email_pat = compile(r"^([-\w_.'+])+@(([-\w])+\.)+([\w]{2,4})$")
         for email in list:
-            self.updateSubscriberEmails(email)
+            if email_pat.match(email):
+                self.updateSubscriberEmails(email)
 
     ######################################################
     ######################################################
@@ -470,7 +472,7 @@ class ExplicitRecipientsRule(RecipientsRule):
         notification_rule = getattr(self, notification_rule_id, None)
         if notification_rule is None:
             LOG(" ::CPSSubscriptions:: subscribeTo()",
-                INFO,
+                ERROR,
                 "Error : No mail notification found")
             return 0
 
@@ -496,7 +498,7 @@ class ExplicitRecipientsRule(RecipientsRule):
             membership_tool = getToolByName(self, 'portal_membership')
             member = membership_tool.getAuthenticatedMember()
             member_id = member.getMemberId()
-            member_email = self.getMemberEmail(member_id) #skins
+            member_email = membership_tool.getEmailFromUsername(member_id)
 
             # Building member struct with compuslory information
             utool = getToolByName(self, 'portal_url')
@@ -542,7 +544,7 @@ class ExplicitRecipientsRule(RecipientsRule):
         notification_rule = getattr(self, notification_rule_id, None)
         if notification_rule is None:
             LOG(" ::CPSSubscriptions:: subscribeTo()",
-                INFO,
+                ERROR,
                 "Error : No mail notification found")
             return 0
 
@@ -585,7 +587,7 @@ class ExplicitRecipientsRule(RecipientsRule):
         notification_rule = getattr(self, notification_rule_id, None)
         if notification_rule is None:
             LOG(" ::CPSSubscriptions:: subscribeTo()",
-                INFO,
+                ERROR,
                 "Error : No mail notification found")
             return 0
 
@@ -605,7 +607,7 @@ class ExplicitRecipientsRule(RecipientsRule):
             membership_tool = getToolByName(self, 'portal_membership')
             member = membership_tool.getAuthenticatedMember()
             member_id = member.getMemberId()
-            member_email = self.getMemberEmail(member_id) #skins
+            member_email = membership_tool.getEmailFromUsername(member_id)
 
             if member_id in self.getMemberIds():
                 # Building member struct with compuslory information
@@ -644,7 +646,7 @@ class ExplicitRecipientsRule(RecipientsRule):
         notification_rule = getattr(self, notification_rule_id, None)
         if notification_rule is None:
             LOG(" ::CPSSubscriptions:: subscribeTo()",
-                INFO,
+                ERROR,
                 "Error : No mail notification found")
             return 0
 
@@ -664,7 +666,7 @@ class ExplicitRecipientsRule(RecipientsRule):
     ###############################################################
 
     security.declareProtected(View, "getRecipients")
-    def getRecipients(self, event_type, object, infos):
+    def getRecipients(self, event_type, object, infos={}):
         """Get the recipients.
 
         Returns a mapping with 'members' and 'emails' as keys.
@@ -675,7 +677,7 @@ class ExplicitRecipientsRule(RecipientsRule):
         aclu = getattr(self, 'acl_users', None)
 
         #
-        # It might not be an event nortification but just a request
+        # It might not be an event notification but just a request
         # in a given context such as the manager editing subscriptions
         # parameters. Thus we won't change the context since the context is
         # already given as a parameter (object)
@@ -689,20 +691,24 @@ class ExplicitRecipientsRule(RecipientsRule):
 
         # Members subscribed
         for member_id in self.getMemberIds(context=object):
-            email = self.getMemberEmail(member_id)
-            member_email_mapping[email] = member_id
+            email = mtool.getEmailFromUsername(member_id)
+            if email is not None:
+                member_email_mapping[email] = member_id
 
         # Groups subscribed
+        dead_groups = []
         for group_id in self.getGroups():
             try:
                 group = aclu.getGroupById(group_id)
                 group_users = group.getUsers()
                 for member_id in group_users:
-                    email = self.getMemberEmail(member_id)
-                    member_email_mapping[email] = member_id
+                    email = mtool.getEmailFromUsername(member_id)
+                    if email is not None:
+                        member_email_mapping[email] = member_id
             except KeyError:
-                # XXX
-                pass
+                dead_groups.append(group_id)
+        for dead_group in dead_groups:
+            self.groups.remove(dead_group)
 
         # Explicit emails
         for email in self.getEmails():
@@ -845,9 +851,8 @@ class RoleRecipientsRule(RecipientsRule):
                             group = aclu.getGroupById(group_id)
                             member_ids = group.getUsers()
                         for member_id in member_ids:
-                            member = mtool.getMemberById(member_id)
-                            if member is not None:
-                                email = self.getMemberEmail(member_id)
+                            email = mtool.getEmailFromUsername(member_id)
+                            if email is not None:
                                 member_email_mapping[email] = member_id
         else:
             #
@@ -858,8 +863,9 @@ class RoleRecipientsRule(RecipientsRule):
                 member_id = member[0]
                 for role in self.getRoles():
                     if role in member[1]:
-                        email = self.getMemberEmail(member_id)
-                        member_email_mapping[email] = member_id
+                        email = mtool.getEmailFromUsername(member_id)
+                        if email is not None:
+                            member_email_mapping[email] = member_id
 
             local_group_roles = container.get_local_group_roles()
             for group in local_group_roles:
@@ -870,16 +876,16 @@ class RoleRecipientsRule(RecipientsRule):
                         group = aclu.getGroupById(group_id)
                         group_users = group.getUsers()
                         for member_id in group_users:
-                            email = self.getMemberEmail(member_id)
-                            member_email_mapping[email] = member_id
-
+                            email = mtool.getEmailFromUsername(member_id)
+                            if email is not None:
+                                member_email_mapping[email] = member_id
 
         #
         # Removing the members who asked for unsubsciption
         #
 
         for member_id in self.getUnSubscribedMembers():
-            member_email = self.getMemberEmail(member_id)
+            member_email = mtool.getEmailFromUsername(member_id)
             if member_email in member_email_mapping.keys():
                 del member_email_mapping[member_email]
 
