@@ -19,7 +19,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-# $Id: SubscriptionsTool.py 21387 2005-04-25 22:56:45Z janguenot $
+# $Id: SubscriptionsTool.py 33749 2006-03-02 17:32:26Z div $
 
 __author__ = "Julien Anguenot <mailto:ja@nuxeo.com>"
 
@@ -28,26 +28,38 @@ __author__ = "Julien Anguenot <mailto:ja@nuxeo.com>"
 Defines the Subscriptions Tool class
 """
 
+import warnings
+from zLOG import LOG, DEBUG
+
 from types import DictType, StringType
 
 from Globals import InitializeClass, DTMLFile
 from Acquisition import aq_parent, aq_inner
 from AccessControl import ClassSecurityInfo
 
-from Products.BTreeFolder2.CMFBTreeFolder import CMFBTreeFolder
+from zope.interface import implements
 
-from Products.CMFCore.permissions import ManagePortal, View, \
-     ModifyPortalContent
-from Products.CMFCore.utils import UniqueObject, getToolByName, \
-     _checkPermission
+from Products.ZCatalog.ZCatalog import ZCatalog
+from Products.CMFCore.CMFBTreeFolder import CMFBTreeFolder
+
+from Products.CMFCore.utils import UniqueObject
+from Products.CMFCore.utils import getToolByName
+from Products.CMFCore.utils import _checkPermission
 from Products.CMFCore.ActionProviderBase import ActionProviderBase
 
-from CPSSubscriptionsPermissions import ViewMySubscriptions, CanSubscribe
+from Products.CMFCore.permissions import ManagePortal
+from Products.CMFCore.permissions import View
+from Products.CMFCore.permissions import ModifyPortalContent
+
+from permissions import ViewMySubscriptions
+from permissions import CanSubscribe
+from permissions import ManageSubscriptions
+
 from Notifications import MailNotificationRule
 from NotificationMessageBody import addNotificationMessageBody
-from Products.ZCatalog.ZCatalog import ZCatalog
+from EventSubscriptionsManager import get_event_subscriptions_manager
 
-from zLOG import LOG, DEBUG, INFO
+from Products.CPSSubscriptions.interfaces import ISubscriptionsTool
 
 ##############################################################
 
@@ -69,15 +81,23 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
     the object.
     """
 
+    __implements__ = ActionProviderBase.__implements__
+
+    implements(ISubscriptionsTool)
+
     id = 'portal_subscriptions'
     meta_type = 'Subscriptions Tool'
+    _actions = ()
 
     security = ClassSecurityInfo()
 
     _properties = (
         {'id': 'notify_hidden_object',
          'type': 'boolean', 'mode':'w',
-         'label' : 'Notify hidden files'},
+         'label' : 'Notify hidden files'}, # FIXME Unused for now
+        {'id': 'max_recipients_per_notification',
+         'type': 'int', 'mode':'w',
+         'label': 'Max recipients per notification message'},
         {'id': 'render_content_for_portal_types',
          'type': 'lines', 'mode':'w',
          'label' : 'Render the content type responsible of the notification \
@@ -95,14 +115,20 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         {'id': 'mapping_local_roles_context',
          'type': 'string', 'mode':'r',
          'label': 'Local roles within the different context'},
+        {'id': 'ignore_events', 'type': 'boolean', 'mode': 'w',
+         'label': "Ignore events"},
         )
 
+    # Maximim recipients (emails) for one notification.
+    # Check the maik server capability
+    max_recipients_per_notification = 20
+
     mapping_modes = {'weekly' : 'mode_weekly',
                      'monthly': 'mode_monthly',
                      'daily'  : 'mode_daily'}
 
     mapping_local_roles_context = {}
-    _actions = ()
+    ignore_events = False
 
     ###################################################
     # ZMI
@@ -144,6 +170,8 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
     def manage_resetEventMessages(self, REQUEST=None):
         """Reset the email messages to their default values.
         """
+        warnings.warn("manage_resetEventMessages is obsolete and will be "
+                      "removed in CPS 3.5.0", DeprecationWarning, 2)
         self.resetEvents()
         if REQUEST is not None:
             REQUEST.RESPONSE.redirect(
@@ -159,6 +187,8 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
                                        unsubscribe_email_body,
                                        unsubscribe_confirm_email_title,
                                        unsubscribe_confirm_email_body,
+                                       event_default_email_title=None,
+                                       event_default_email_body=None,
                                        REQUEST=None):
         """Edit the default event email messages.
         """
@@ -172,6 +202,12 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         # Notifications
         self.event_error_email_body = event_error_email_body
 
+        # Defaults
+        if event_default_email_title is not None:
+            self.event_default_email_title = event_default_email_title
+        if event_default_email_body is not None:
+            self.event_default_email_body = event_default_email_body
+
         # Subscriptions
         self.subscribe_confirm_email_body = subscribe_confirm_email_body
         self.subscribe_confirm_email_title = subscribe_confirm_email_title
@@ -192,14 +228,6 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
                                 event_id,
                                 event_email_title,
                                 event_email_body,
-                                subscribe_confirm_email_body='',
-                                subscribe_confirm_email_title='',
-                                subscribe_welcome_email_body='',
-                                subscribe_welcome_email_title='',
-                                unsubscribe_email_title='',
-                                unsubscribe_email_body='',
-                                unsubscribe_confirm_email_title='',
-                                unsubscribe_confirm_email_body='',
                                 REQUEST=None):
         """Edit a custom event message
         """
@@ -207,14 +235,7 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
 
         struct = [event_email_title,
                   event_email_body,
-                  subscribe_confirm_email_title,
-                  subscribe_confirm_email_body,
-                  subscribe_welcome_email_title,
-                  subscribe_welcome_email_body,
-                  unsubscribe_email_title,
-                  unsubscribe_email_body,
-                  unsubscribe_confirm_email_title,
-                  unsubscribe_confirm_email_body,]
+                  ]
 
         self.mapping_event_email_content[event_id] = struct
 
@@ -240,21 +261,12 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
             self.mapping_event_email_content[event_id] = [
                 self.getDefaultMessageTitle(),
                 self.getDefaultMessageBody(),
-                self.getSubscribeConfirmEmailTitle(),
-                self.getSubscribeConfirmEmailBody(),
-                self.getSubscribeWelcomeEmailTitle(),
-                self.getSubscribeWelcomeEmailBody(),
-                self.getUnSubscribeConfirmEmailTitle(),
-                self.getUnSubscribeConfirmEmailBody(),
-                self.getUnSubscribeEmailTitle(),
-                self.getUnSubscribeEmailBody(),
                 ]
 
         if REQUEST is not None:
             REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_events')
 
     #######################################################################
-    #######################################################################
 
     def manage_addLocalRoleArea(self,
                                 area_portal_type='',
@@ -341,8 +353,8 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         self.unsubscribe_email_body = ''
 
         # Rendering content at notification time
-        self.render_content_for_portal_types = []
-        self.render_content_for_events = []
+        self.render_content_for_portal_types = ()
+        self.render_content_for_events = ()
 
         # Here, it's stored the notification scheduling table
         # Structure :
@@ -362,60 +374,57 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         # Local roles / context
         self.mapping_local_roles_context = {}
 
-    ######################################################
     #####################################################
 
-    security.declareProtected(ManagePortal, 'addRenderedPortalTypes')
+    security.declareProtected(ManagePortal, 'addRenderedPortalType')
     def addRenderedPortalType(self, portal_type=''):
         """Add a portal type for wich the render of the content
         type will be included into the notification email body.
         """
-
-        self._p_changed = 1
-
         if (isinstance(portal_type, StringType) and
             portal_type not in self.render_content_for_portal_types):
-            self.render_content_for_portal_types.append(portal_type)
+            self.render_content_for_portal_types += (portal_type,)
             return 1
         return 0
 
-    security.declareProtected(ManagePortal, 'addRenderedEvents')
+    security.declareProtected(ManagePortal, 'addRenderedEvent')
     def addRenderedEvent(self, event_id=''):
         """Add an event for wich the render of the content
         type will be included into the notification email body.
         """
-
-        self._p_changed = 1
-
         if (isinstance(event_id, StringType) and
             event_id not in self.render_content_for_events):
-            self.render_content_for_events.append(event_id)
+            self.render_content_for_events += (event_id,)
             return 1
         return 0
 
     security.declarePublic('getRenderedPortalTypes')
     def getRenderedPortalTypes(self):
-        """Return the portal_types that we are gonna render
+        """Return the portal_types that we are going to render
         and add the rendering within the email notification body
         """
         return self.render_content_for_portal_types
 
     security.declarePublic('getRenderedEvents')
     def getRenderedEvents(self):
-        """Return the events for wich we are gonna render the content
+        """Return the events for wich we are going to render the content
         type which is responsible of the notifications
         and then add this rendering within the email notification body
         """
         return self.render_content_for_events
 
     ###########################################################
-    ###########################################################
 
     security.declareProtected(ManagePortal, 'setupEvents')
     def setupEvents(self):
-        """ Setup events on which to react
+        """Setup events on which to react.
         """
+
+        warnings.warn("setupEvents is obsolete and will be "
+                      "removed in CPS 3.5.0", DeprecationWarning, 2)
+
         portal = getToolByName(self, 'portal_url').getPortalObject()
+        # FIXME AT: getEvents should not be a skin script...
         mapping_context_events = portal.getEvents()
         for context in mapping_context_events.keys():
             for event_id in mapping_context_events[context].keys():
@@ -426,14 +435,16 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
 
     security.declarePublic('resetEvents')
     def resetEvents(self):
-        """ Reset events
+        """Reset events.
         """
+        warnings.warn("resetEvents is obsolete and will be "
+                      "removed in CPS 3.5.0", DeprecationWarning, 2)
         self.__init__()
         self.setupEvents()
 
     security.declarePublic('getSubscriptionContainerId')
     def getSubscriptionContainerId(self):
-        """ Returns the default id for subscription containers
+        """Returns the default id for subscription containers.
 
         .cps_subscriptions by default
         """
@@ -441,7 +452,7 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
 
     security.declarePublic('getExplicitRecipientsRuleId')
     def getExplicitRecipientsRuleId(self):
-        """ Returns an in use id.
+        """Returns an in use id.
 
         Id in use for the ExplicitRecipientsRule object
         """
@@ -464,7 +475,6 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         return SUBSCRIPTION_PREFIX
 
     ###########################################################
-    ###########################################################
 
     security.declarePublic('getDefaultMessageTitle')
     def getDefaultMessageTitle(self, event_id=None):
@@ -476,6 +486,9 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         if event_id is not None:
             if self.mapping_event_email_content.has_key(event_id):
                 return self.mapping_event_email_content[event_id][0]
+            else:
+                # if the event message is not found, use the default message
+                return self.event_default_email_title
         else:
             if not self.event_default_email_title:
                 # Init of the variable here.
@@ -493,6 +506,9 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         if event_id is not None:
             if self.mapping_event_email_content.has_key(event_id):
                 return self.mapping_event_email_content[event_id][1]
+            else:
+                # if the event message is not found, use the default message
+                return self.event_default_email_body
         else:
             if not self.event_default_email_body:
                 # Init of the variable here.
@@ -510,15 +526,11 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         If event_id is specified then let's check if we get a custom
         one.
         """
-        if event_id is not None:
-            if self.mapping_event_email_content.has_key(event_id):
-                return self.mapping_event_email_content[event_id][2]
-        else:
-            if not self.event_error_email_body:
-                # Init of the variable here.
-                self.event_error_email_body = self.getMailTemplate()[
-                    'mail_error_body']
-            return self.event_error_email_body
+        if not self.event_error_email_body:
+            # Init of the variable here.
+            self.event_error_email_body = self.getMailTemplate()[
+                'mail_error_body']
+        return self.event_error_email_body
 
     security.declarePublic('getSubscribeConfirmEmailTitle')
     def getSubscribeConfirmEmailTitle(self, event_id=None):
@@ -633,7 +645,6 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         return self.unsubscribe_confirm_email_body
 
     #######################################################
-    #######################################################
 
     security.declarePublic('getSubscribablePortalTypes')
     def getSubscribablePortalTypes(self):
@@ -647,7 +658,7 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         """ Returns events given a context.
         """
         if context is not None:
-            context_portal_type = context.portal_type
+            context_portal_type = getattr(context, 'portal_type', '')
             if self.mapping_context_events.has_key(context_portal_type):
                 return self.mapping_context_events[context_portal_type]
         return {}
@@ -706,7 +717,6 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
                         key_portal_type][key_event]
 
     #########################################################
-    #########################################################
 
     security.declarePublic('addSusbcriptionContainerFromContext')
     def addSubscriptionContainerInContext(self, context):
@@ -716,35 +726,56 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
             return None
 
         subscription_id = self.getSubscriptionContainerId()
-        if subscription_id not in context.objectIds():
-            context.manage_addProduct[
-                'CPSSubscriptions'].addSubscriptionContainer()
+        if context.isPrincipiaFolderish:
+            if subscription_id not in context.objectIds():
+                context.manage_addProduct[
+                    'CPSSubscriptions'].addSubscriptionContainer()
+        else:
+            # `context` is not a folderish document so we can't create
+            # a subscription container within.
+            return self.getSubscriptionContainerFromContext(
+                aq_parent(aq_inner(context)), force_local_creation=True)
+        
         return getattr(context, subscription_id)
 
     security.declarePublic('getSusbcriptionContainerFromContext')
-    def getSubscriptionContainerFromContext(self, context, force_local_creation=0):
-        """Returns a subscriptions container id.
+    def getSubscriptionContainerFromContext(self, context,
+                                            force_local_creation=0):
+        """Returns a subscriptions container in the context
 
-        Lookup to find one through acquisition
+        Lookup to find one through acquisition with the accurate
+        rights for the user to manage it. Note the user is supposed to
+        have the rights to manage the container in the context if not
+        then this method will return None
         """
-        container_id = self.getSubscriptionContainerId()
-        container = getattr(context, container_id, None)
 
-        #
+        # Lookup container by acquisition
+        container = getattr(context, self.getSubscriptionContainerId(), None)
+
         # No subscription container by acquisition
         # create one in the context if the current member
         # has the ModifyPortalContent permission
-        #
-
-        mtool = getToolByName(self, 'portal_membership')
-        auth_user = mtool.getAuthenticatedMember()
-
-
-        if (container is None or force_local_creation) \
-            and _checkPermission(ModifyPortalContent, context):
+        if ((container is None or force_local_creation)
+            and _checkPermission(ModifyPortalContent, context)):
             container = self.addSubscriptionContainerInContext(context)
+
+        # If the container is looked up by acquisition but the current
+        # user doesn't have the right permissions to perform
+        # modifications on it then create on in the context
+        if not _checkPermission(ManageSubscriptions, container):
+            container = self.addSubscriptionContainerInContext(context)
+
         return container
 
+    security.declarePrivate('notify_processed_event')
+    def notify_processed_event(self, event_type, object, infos):
+        """EventManager's callable
+        """
+        subscriptions = self.getSubscriptionsFor(event_type, object, infos)
+        for subscription in subscriptions:
+            if subscription.isInterestedInEvent(event_type, object, infos):
+                subscription.sendEvent(event_type, object, infos)
+
     security.declarePrivate("notify_event")
     def notify_event(self, event_type, object, infos):
         """Standard event hook.
@@ -755,11 +786,13 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         For workflow events, infos must contain the additional
         keyword arguments passed to the transition.
         """
+        if self.ignore_events:
+            return
 
-        subscriptions = self.getSubscriptionsFor(event_type, object, infos)
-        for subscription in subscriptions:
-            if subscription.isInterestedInEvent(event_type, object, infos):
-                subscription.sendEvent(event_type, object, infos)
+        # Pre-filtering : we don't want to notify the user on
+        # repository objects
+        if not 'portal_repository' in object.getPhysicalPath():
+            get_event_subscriptions_manager().push(event_type, object, infos)
 
     security.declarePublic("getSubscriptionsFor")
     def getSubscriptionsFor(self, event_type, object, infos=None):
@@ -820,13 +853,12 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
                     else:
                         LOG("::CPSSubscriptions :: "
                             "ComputeRecipientsRules ERROR",
-                            INFO,
+                            DEBUG,
                             "You should provide a dictionnary",
                             pt_recipient_rule.absolute_url())
         return recipients
 
     #############################################################
-    #############################################################
 
     def _makeEltDict(self, ob, subscription):
         """Build a dict with an object
@@ -874,10 +906,10 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         #
 
         # Member information
+        membership_tool = getToolByName(self, 'portal_membership')
         if not member_id:
-            membership_tool = getToolByName(self, 'portal_membership')
             member_id = membership_tool.getAuthenticatedMember().getMemberId()
-        email = self.getMemberEmail(member_id)
+        email = membership_tool.getEmailFromUsername(member_id)
 
         # Place
         if context is not None:
@@ -894,7 +926,7 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         # parent folder
         containers = ZCatalog.searchResults(catalog, None, **query)
         LOG(":: CPSSubscriptions :: catalog search for containers" % query,
-            INFO, str([x.getPath() for x in containers]))
+            DEBUG, str([x.getPath() for x in containers]))
 
         #
         # Now let's get the subcription containers and check if the computed
@@ -928,7 +960,10 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
                         if subscriber != -1:
                             urls = subscriber['subscription_relative_url']
                             for url in urls:
-                                ob = self.restrictedTraverse(url)
+                                ob = self.restrictedTraverse(url, None)
+                                if ob is None:
+                                    # ob has since been removed
+                                    continue
                                 elt = self._makeEltDict(ob, subscription)
                                 subscriptions_list.append(elt)
 
@@ -961,7 +996,7 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         if not isAno:
             member = membership_tool.getAuthenticatedMember()
             member_id = member.getMemberId()
-            member_email = self.getMemberEmail(member_id)
+            member_email = membership_tool.getEmailFromUsername(member_id)
 
             if role_based:
                 # Check here if the member is computed based on his local roles
@@ -1003,7 +1038,6 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         return
 
     ################################################################
-    ################################################################
 
     security.declareProtected(ManagePortal, 'addNotificationMessageBodyObject')
     def addNotificationMessageBodyObject(self, message_body='',
@@ -1046,6 +1080,23 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
             return 0
         return -1
 
+    security.declarePublic('getMailSenderInfo')
+    def getMailSenderInfo(self, use_portal_title=0):
+        """Return the sender information (name and email address)
+
+        If use_portal_title is set to 1, then the portal title is used instead
+        of the portal administrator name.
+        """
+        portal = getToolByName(self, 'portal_url').getPortalObject()
+        sender_email = portal.getProperty('email_from_address',
+                                          'nobody@nobody.com')
+        if use_portal_title == 1:
+            sender_name = portal.title_or_id()
+        else:
+            sender_name = portal.getProperty('email_from_name',
+                                              'Portal administrator')
+        return sender_email, sender_name
+
     def _getMailInfoFor(self, subscription_mode, email_to, messages):
         """Build the infos dict
 
@@ -1054,15 +1105,12 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
 
         portal = getToolByName(self, 'portal_url').getPortalObject()
         portal_title = getattr(portal, 'title', 'Portal')
-        mcat = self.Localizer.default
+        mcat = self.translation_service
 
         infos = {}
-        infos['sender_email'] = getattr(portal,
-                                        'email_from_address',
-                                        'nobody@nobody.com')
-        infos['sender_name']  = getattr(portal,
-                                        'email_from_name',
-                                        'Portal administrator')
+        sender_email, sender_name = self.getMailSenderInfo()
+        infos['sender_email'] = sender_email
+        infos['sender_name']  = sender_name
         infos['to'] = email_to
 
         # Message body
@@ -1135,11 +1183,11 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
                 table[email] = []
             self.notification_scheduling_table[
                 self.mapping_modes[subscription_mode]] = table
+            return "Notifications sent"
         else:
             return -1
 
     ###############################################################
-    ###############################################################
 
     security.declareProtected(ManagePortal, 'setLocalRolesArea')
     def setLocalRolesArea(self, area='', value={}):
@@ -1219,7 +1267,6 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
                         role_id] = role_label
 
     ##############################################################
-    ##############################################################
 
     def _getLocalRoleAreaFromContext(self, context):
         """Get Local Role area from context
@@ -1252,8 +1299,6 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         if context is not None:
             context_area = self._getLocalRoleAreaFromContext(context)
             context_portal_type = getattr(context, 'portal_type', None)
-            LOG("context_area ----------", DEBUG, context_area)
-            LOG("context_portal_type ----------", DEBUG, context_portal_type)
             if (context_area is not None and
                 context_portal_type is not None and
                 context_portal_type in self.getContainerPortalTypes()):
@@ -1262,7 +1307,6 @@ class SubscriptionsTool(UniqueObject, CMFBTreeFolder, ActionProviderBase):
         return {}
 
     #########################################################################
-    #########################################################################
 
     def sendmail(self, infos={}):
         """Send a mail given a mapping
