@@ -1,5 +1,5 @@
 ##parameters=member_id
-#$Id: getMemberEmail.py 21033 2004-04-30 13:36:35Z madarche $
+#$Id: getMemberEmail.py 26351 2005-09-01 10:03:40Z lregebro $
 """ Returns the mail of a given user
 
 This script will be usefull in custom member directories such as LDAP
@@ -8,9 +8,7 @@ CPS ones.
 """
 
 if member_id:
-    members = context.portal_directories.members
-    member = members.getEntry(member_id, default=None)
-    if member:
-        return member.get('email')
+    membership = context.portal_membership
+    return membership.getEmailFromUsername(member_id)
 
 return None
