@@ -36,12 +36,15 @@ from Products.CPSSchemas.DataStructure import DataStructure
 from Products.CPSSchemas.DataModel import DataModel
 from Products.CPSCore.CPSBase import CPSBaseFolder
 from Products.CPSCore.ProxyBase import ProxyFolder
-from Products.CPSSchemas.PropertiesPostProcessor import PropertiesPostProcessor
-from Products.CPSSchemas.WidgetTypesTool import WidgetTypeRegistry
+from Products.CPSUtil.PropertiesPostProcessor import PropertiesPostProcessor
 import ExtensionClass
 from Products.CMFCore.permissions import ViewManagementScreens
+from Products.CPSSchemas.Widget import widgetRegistry
 from Products.CPSSchemas.BasicWidgets import CPSSelectWidget
 from types import StringType
+
+from interfaces import ITypeMaker
+from zope.interface import implements
 """
 TypeMakerTool depends on these tools :
 
@@ -65,6 +68,7 @@ class TypeMakerTool(UniqueObject, Folder, PropertiesPostProcessor):
     """
     id = 'portal_typemaker'
     meta_type = 'CPS Type Maker Tool'
+    implements(ITypeMaker)
 
     # TO be externalized
     default_preview = 'rien.'
@@ -112,7 +116,8 @@ class TypeMakerTool(UniqueObject, Folder, PropertiesPostProcessor):
         'is_required']
 
     type_filter_list = ['Dummy Widget', 'Search Widget', 'Link Widget',
-        'Text Image Widget', 'Text Widget', 'Range List Widget', 'File Widget']
+                        'Text Image Widget', 'Text Widget', 'Range List Widget',
+                        'File Widget', 'CPS Portlet Custom Widget']
 
     multiple_layouts = False
     flexible_aware = False
@@ -124,7 +129,7 @@ class TypeMakerTool(UniqueObject, Folder, PropertiesPostProcessor):
     metadata_layout = 'metadata'
     metadata_schemas = ['metadata', 'common']
     immediate_view = 'cpsdocument_view'
-    hidden_layout_modes = []
+    hidden_layout_modes = ()
     max_rows = 4
 
     process_after_t = ''
@@ -344,7 +349,7 @@ class TypeMakerTool(UniqueObject, Folder, PropertiesPostProcessor):
                         return RESPONSE.redirect(utool() + redir)
                     else:
                         return
-                new_widget_id = generateId(new_widget_title)
+                new_widget_id = generateId(new_widget_title, lower=False)
                 if new_widget_id is not None:
                     res = self._addwidget(layout, type_id, new_widget_id,
                         new_widget_title, new_widget_type)
@@ -433,7 +438,7 @@ class TypeMakerTool(UniqueObject, Folder, PropertiesPostProcessor):
 
 
             if action == 'add':
-                new_widget_id = generateId(new_widget_title)
+                new_widget_id = generateId(new_widget_title, lower=False)
 
                 if new_widget_id is not None:
 
@@ -649,10 +654,10 @@ class TypeMakerTool(UniqueObject, Folder, PropertiesPostProcessor):
 
 
         if RESPONSE:
-            Localizer = getToolByName(self, 'Localizer')
+            translation_service = getToolByName(self, 'translation_service')
 
-            if Localizer:
-                mcat = Localizer.default
+            if translation_service:
+                mcat = translation_service
                 message = 'psm_changed'
             else:
                 message = ''
@@ -1034,7 +1039,7 @@ class TypeMakerTool(UniqueObject, Folder, PropertiesPostProcessor):
             else:
                 return
 
-        id = generateId(title)
+        id = generateId(title, lower=False)
 
         # getting tool links
         ttool = getToolByName(self, 'portal_types')
@@ -1168,60 +1173,38 @@ class TypeMakerTool(UniqueObject, Folder, PropertiesPostProcessor):
 
     security.declarePrivate('_listWidgets')
     def _listWidgets(self, attribute='_marker', translated = False):
-        """ returns a list of ids and meta_types
+        """ returns a list of widget meta_types
             filtered if needed
             by a boolean attribute
             meta_type is translated if needeed
         """
+        mcat = getToolByName(self, 'translation_service', None)
+
         res = []
-        portal_widget_types = getToolByName(self, 'portal_widget_types')
-        Localizer = getToolByName(self, 'Localizer')
-        if Localizer:
-            mcat = Localizer.default
-
-        for id, widget in portal_widget_types.objectItems():
-            if id not in self.type_filter_list:
-                # theses are widget types
-                if (attribute == '_marker') or not hasattr(widget, attribute)\
-                or widget.is_addable:
-                    meta_type = widget.meta_type
-
-                    if meta_type <> 'Broken Because Product is Gone':
-                        # we don't want duplicate meta_types
-                        already_in = False
-
-                        for sort, element in res:
-                            if element['meta_type'] == meta_type:
-                                already_in = True
-                                break
-
-                        if not already_in:
-                            element = {}
-                            element['id'] = widget.getId()
-                            element['meta_type'] = meta_type
-
-                            sort_key = widget.getId()
-
-                            if translated and mcat:
-                                trad_key = mcat(sort_key)
-                                if trad_key == sort_key:
-                                    trad_key = mcat('CPS '+sort_key)
-                                    if trad_key == 'CPS '+sort_key:
-                                        trad_key = sort_key
-                            else:
-                                trad_key = sort_key
-
-                            sorter =  (trad_key, element,)
-                            res.append(sorter)
+        for meta_type in widgetRegistry.listWidgetMetaTypes():
+            class_ = widgetRegistry.getClass(meta_type)
+            if meta_type in self.type_filter_list:
+                continue
+            if ((attribute != '_marker') and hasattr(class_, attribute)
+                and not class_.is_addable):
+                continue
+            sort_key = meta_type
+            if translated and mcat:
+                trad_key = mcat(sort_key)
+                if trad_key == sort_key:
+                    trad_key = mcat('CPS '+sort_key)
+                    if trad_key == 'CPS '+sort_key:
+                        trad_key = sort_key
+            else:
+                trad_key = sort_key
+            sorter =  (trad_key, meta_type)
+            res.append(sorter)
 
         # need to find out why it raises
         # on unicode errors here
         # when mcat make traductions with accents on ascii
         res.sort()
-        result = []
-        for item in res:
-            result.append(item[1])
-        return result
+        return [meta_type for (key, meta_type) in res]
 
     security.declareProtected(ViewManagementScreens, 'listWidgetTypes')
     def listWidgetTypes(self, translated = False):
@@ -1248,7 +1231,7 @@ class TypeMakerTool(UniqueObject, Folder, PropertiesPostProcessor):
 
         return self.widget_renderer
 
-    security.declareProtected(ViewManagementScreens, 'getWidgetType')
+    security.declareProtected(ViewManagementScreens, 'layoutPreview')
     def layoutPreview(self,type_id):
         """ this renders a preview
             made up with
@@ -1327,10 +1310,10 @@ class TypeMakerTool(UniqueObject, Folder, PropertiesPostProcessor):
             'size_max': '0'
             }
         if current.get('show_in_view'):
-            kw['hidden_layout_modes'] = self.hidden_layout_modes
+            kw['hidden_layout_modes'] = tuple(self.hidden_layout_modes)
         else:
-            kw['hidden_layout_modes'] = ['view']+\
-                self.hidden_layout_modes
+            kw['hidden_layout_modes'] = ('view',) + \
+                tuple(self.hidden_layout_modes)
 
         widget.manage_changeProperties(**kw)
         # Update indexing:
@@ -1558,14 +1541,12 @@ if not hasattr(CPSSelectWidget, '_getVocabulary'):
 
 ########################################################################################
 
-def addCPSTypeMakerTool(container, id=None, REQUEST=None, **kw):
-    """Add a CPS MemberData Tool."""
-    if id is None:
-        id = container.computeId()
+def addCPSTypeMakerTool(container, REQUEST=None, **kw):
+    """Add a CPS TypeMaker Tool."""
     container = container.this() # For FactoryDispatcher.
-
     t = TypeMakerTool()
-    container._setObject(t.getId(), t)
+    id = t.getId()
+    container._setObject(id, t)
     if REQUEST is not None:
-        t = container._getOb(t.getId())
+        t = container._getOb(id)
         REQUEST.RESPONSE.redirect(t.absolute_url()+'/manage_overview')
