@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: CPSUserFolder.py 17449 2005-04-19 16:32:54Z fguillaume $
+# $Id: CPSUserFolder.py 33548 2006-02-27 19:49:01Z fguillaume $
 """
 CPSUserFolder
 
@@ -24,6 +24,7 @@ A user folder based on CPSDirectory and CPSSchemas.
 
 from zLOG import LOG, DEBUG, WARNING, ERROR, TRACE
 
+from copy import deepcopy
 from types import ListType
 import base64
 
@@ -40,12 +41,15 @@ from AccessControl.PermissionRole import _what_not_even_god_should_do
 from Products.CMFCore.utils import getToolByName
 from Products.CMFCore.utils import SimpleItemWithProperties
 from Products.CMFCore.permissions import ManagePortal
-from Products.CPSSchemas.PropertiesPostProcessor import PropertiesPostProcessor
+from Products.CPSUtil.PropertiesPostProcessor import PropertiesPostProcessor
 
 from Products.CPSDirectory.BaseDirectory import AuthenticationFailed
 
 from Products.CPSUserFolder import TimeoutCache
 
+from zope.interface import implements
+from Products.CPSUserFolder.interfaces import ICPSUserFolder
+
 
 _marker = []
 CACHE_KEY = 'CPSUserFolder'
@@ -69,7 +73,10 @@ class CPSUserFolder(PropertiesPostProcessor, SimpleItemWithProperties,
       id -> user_info = {'password', 'roles', 'groups', 'entry'}
         the password may be None if no password has yet been checked
     """
-    meta_type = 'CPSUserFolder'
+
+    implements(ICPSUserFolder)
+
+    meta_type = 'CPS User Folder'
     id = 'acl_users'
     title = 'CPS User Folder'
 
@@ -227,13 +234,11 @@ class CPSUserFolder(PropertiesPostProcessor, SimpleItemWithProperties,
         if dtool is None:
             # User folder has been instanciated outside a CPS site.
             return None
-        try:
-            dir = getattr(dtool, self.users_dir)
-        except AttributeError:
-            LOG('CPSUserFolder', WARNING,
-                "Missing directory '%s'" % self.users_dir)
-            dir = None
-        return dir
+        users_dir = self.users_dir
+        if getattr(aq_base(dtool), users_dir, None) is None:
+            LOG('CPSUserFolder', WARNING, "Missing directory %r" % users_dir)
+            return None
+        return getattr(dtool, users_dir)
 
     security.declarePrivate('_buildUser')
     def _buildUser(self, id, user_info):
@@ -456,7 +461,7 @@ class CPSUserFolder(PropertiesPostProcessor, SimpleItemWithProperties,
             self.users_roles_field: tuple(roles),
             self.users_groups_field: tuple(groups),
             })
-        dir.createEntry(entry)
+        dir._createEntry(entry)
 
     security.declarePrivate('_doChangeUser')
     def _doChangeUser(self, name, password, roles, domains, groups=None, **kw):
@@ -472,7 +477,7 @@ class CPSUserFolder(PropertiesPostProcessor, SimpleItemWithProperties,
             })
         if groups is not None:
             entry[self.users_groups_field] = tuple(groups)
-        dir.editEntry(entry)
+        dir._editEntry(entry)
         # Invalidate cache for name.
         self._removeUserFromIdCache(name)
 
@@ -483,7 +488,7 @@ class CPSUserFolder(PropertiesPostProcessor, SimpleItemWithProperties,
         if dir is None:
             raise ValueError("The directory %s doesn't exist" % self.users_dir)
         for name in names:
-            dir.deleteEntry(name)
+            dir._deleteEntry(name)
             # Invalidate cache for name.
             self._removeUserFromIdCache(name)
 
@@ -539,13 +544,18 @@ class CPSUserFolder(PropertiesPostProcessor, SimpleItemWithProperties,
         groups_dir = self._getGroupsDirectory()
         if groups_dir is not None:
             if not groupname.startswith('role:'):
-                if groups_dir.hasEntry(groupname):
-                    group_entry = groups_dir.getEntry(groupname, default)
+                if groups_dir._hasEntry(groupname):
+                    group_entry = groups_dir._getEntry(groupname, default)
                     group_members = group_entry.get(self.groups_members_field,
                                                     ())
                     return Group(groupname, group_members)
+            else:
+                # Backward compatibility with UserFolderWithGroups for
+                # CPSSubscriptions.RecipientsRules (see ticket:890)
+                return Group(groupname, ())
             if default is not _marker:
                 return default
+            raise KeyError, groupname
         else:
             raise ValueError, "The directory %s doesn't exist" % self.groups_dir
 
@@ -553,27 +563,20 @@ class CPSUserFolder(PropertiesPostProcessor, SimpleItemWithProperties,
     # Private UserFolder object interface
     #
 
-##     def identify(self, auth):
-##         """Identify a user.
+    def identify(self, auth):
+        """Add certificate based authentication (cf SecureAuth)
+        """
+        if auth and auth.lower().startswith('clcert '):
+            name = base64.decodestring(auth.split(' ')[-1])
+            password = None
+            return name, password
+        else:
+            return BasicUserFolder.identify(self, auth)
 
-##         (Called by validate.)
-
-##         From 'auth' which comes from the request, identify the user.
-
-##         Returns its username and password, or (None, None).
-##         """
-##         if auth and auth.lower().startswith('basic '):
-##             try:
-##                 name, password = (base64.decodestring(auth.split(' ')[-1])
-##                                   .split(':', 1))
-##             except:
-##                 raise 'BadRequest', 'Invalid authentication token'
-##             return name, password
-##         else:
-##             return None, None
 
     def authenticate(self, name, password, request):
-        """Authenticate a user from a name and password.
+        """Authenticate a user from a name and password or a from
+        certificate (Apache and SecureAuth required)
 
         (Called by validate).
 
@@ -589,6 +592,10 @@ class CPSUserFolder(PropertiesPostProcessor, SimpleItemWithProperties,
             else:
                 return None
         else:
+            if request._auth and request._auth.lower().startswith('clcert '):
+                # A certificate as been validated by an apache frontend: no
+                # password required
+                return self.getUserWithAuthentication(name, None, use_login=0)
             return self.getUserWithAuthentication(name, password, use_login=1)
 
     #def authorize(self, user, accessed, container, name, value, roles):
@@ -677,7 +684,7 @@ class CPSUserFolder(PropertiesPostProcessor, SimpleItemWithProperties,
                 object = aq_inner(object.im_self)
                 continue
             break
-        return merged
+        return deepcopy(merged)
 
     security.declarePrivate('mergedLocalRolesWithPath')
     def mergedLocalRolesWithPath(self, object, withgroups=0):
@@ -730,7 +737,7 @@ class CPSUserFolder(PropertiesPostProcessor, SimpleItemWithProperties,
                 object = aq_inner(object.im_self)
                 continue
             break
-        return merged
+        return deepcopy(merged)
 
     security.declarePrivate('getAllowedRolesAndUsersOfObject')
     def getAllowedRolesAndUsersOfObject(self, ob):
@@ -933,6 +940,32 @@ class CPSUser(BasicUser):
         self._entry = user._entry
         LOG('setProperties', DEBUG, '    entry = %s' % user._entry)
 
+    # CPS extension
+    security.declarePrivate('_setProperties')
+    def _setProperties(self, **kw):
+        """Set the value of properties for the user (unrestricted)."""
+        id = self._id
+        aclu = self._aclu
+        dir = aclu._getUsersDirectory()
+
+        # Remove the user from the cache
+        aclu._removeUserIdFromCache(id)
+
+        # Set the properties
+        kw[dir.id_field] = id
+        dir._editEntry(kw, check_acls=False)
+
+        # Now update this object to make it correspond to the new user.
+        user = aclu.getUserById(id)
+        if user is None:
+            raise KeyError(id)
+
+        LOG('_setProperties', DEBUG, 'old entry = %s' % self._entry)
+        self._roles = user._roles
+        self._groups = user._groups
+        self._entry = user._entry
+        LOG('_setProperties', DEBUG, '    entry = %s' % user._entry)
+
     #
     # Internal API
     #
@@ -1074,7 +1107,7 @@ class Group:
     def __repr__(self):
         # I hope no code assumes that __repr__ is the groupname
         return "<Group %s>" % self.id
-    
+
     def getUsers(self):
         return self.users
 
@@ -1090,5 +1123,5 @@ class Group:
     def setMemberRoles(self, userid, roles):
         raise NotImplementedError
 
-############################################################################    
-    
+############################################################################
+
