@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: UserFolderWithGroups.py 17447 2005-04-08 13:08:29Z sfermigier $
+# $Id: UserFolderWithGroups.py 30977 2005-12-26 19:10:36Z fguillaume $
 """
 User with Groups, and User Folder with Groups
 
@@ -27,6 +27,8 @@ to preserve backward compatibility.
 
 from zLOG import LOG, DEBUG, ERROR
 
+from copy import deepcopy
+
 from Globals import InitializeClass
 from Globals import DTMLFile
 from Globals import Persistent
@@ -47,6 +49,8 @@ from AccessControl.PermissionRole import _what_not_even_god_should_do
 from AccessControl.PermissionRole import rolesForPermissionOn
 from Products.CMFCore.utils import getToolByName
 
+from zope.interface import implements
+from Products.CPSUserFolder.interfaces import IUserFolderWithGroups
 
 _marker = []
 
@@ -398,6 +402,8 @@ class UserFolderWithGroups(UserFolder):
     at the same time, and to centralize management.
     """
 
+    implements(IUserFolderWithGroups)
+
     meta_type = 'User Folder With Groups'
     title = 'User Folder With Groups'
 
@@ -677,7 +683,7 @@ class UserFolderWithGroups(UserFolder):
                 object = aq_inner(object.im_self)
                 continue
             break
-        return merged
+        return deepcopy(merged)
 
     security.declarePrivate('mergedLocalRolesWithPath')
     def mergedLocalRolesWithPath(self, object, withgroups=0):
