@@ -16,11 +16,21 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: __init__.py 17501 2005-03-31 11:58:49Z madarche $
+# $Id: __init__.py 30933 2005-12-22 20:23:37Z fguillaume $
 
 # Making sure that the ModuleSecurityInfo info statements of the following
 # modules are taken into account.
+from Products.CMFCore.DirectoryView import registerDirectory
+
 import id
 import integration
 import text
 import html
+import session
+
+registerDirectory('skins', globals())
+
+# for eggification
+def initialize(self):
+    return "done"
+
