@@ -18,12 +18,11 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: test_stack_elements.py 18306 2005-03-01 14:56:54Z janguenot $
+# $Id: test_stack_elements.py 31004 2005-12-27 19:58:13Z fguillaume $
 
 import unittest
 from Testing.ZopeTestCase import ZopeTestCase
 from Products.PageTemplates.TALES import CompilerError
-from Interface.Verify import verifyClass
 
 from Products.CPSWorkflow.stackelement import StackElement
 
@@ -44,6 +43,7 @@ from Products.CPSWorkflow.interfaces import IStackElement
 class TestStackElements(ZopeTestCase):
 
     def test_interface(self):
+        from zope.interface.verify import verifyClass
         verifyClass(IStackElement, StackElement)
 
     def test_stack_element_viewguard(self):
