@@ -18,12 +18,10 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: test_stackdefinitions.py 18339 2005-04-12 18:19:36Z atchertchian $
+# $Id: test_stackdefinitions.py 31004 2005-12-27 19:58:13Z fguillaume $
 
-import Zope
 import unittest
 from OFS.Folder import Folder
-from Interface.Verify import verifyClass
 
 from AccessControl.SecurityManagement import newSecurityManager
 
@@ -103,6 +101,7 @@ class TestCPSWorkflowStackDefinition(SecurityRequestTest):
         newSecurityManager(None, self.acl_users.getUser(member_id))
 
     def test_interface(self):
+        from zope.interface.verify import verifyClass
         verifyClass(IWorkflowStackDefinition, StackDefinition)
         verifyClass(IWorkflowStackDefinition, SimpleStackDefinition)
         verifyClass(IWorkflowStackDefinition, HierarchicalStackDefinition)
