@@ -18,11 +18,10 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: test_stacks.py 18339 2005-04-12 18:19:36Z atchertchian $
+# $Id: test_stacks.py 31004 2005-12-27 19:58:13Z fguillaume $
 
 import unittest
 from Testing.ZopeTestCase import ZopeTestCase
-from Interface.Verify import verifyClass
 
 from Products.CPSWorkflow.stack import Stack
 from Products.CPSWorkflow.basicstacks import SimpleStack
@@ -38,7 +37,7 @@ from Products.CPSWorkflow.interfaces import IHierarchicalWorkflowStack
 class TestCPSWorkflowStacks(ZopeTestCase):
 
     def test_interface(self):
-
+        from zope.interface.verify import verifyClass
         verifyClass(IWorkflowStack, Stack)
         verifyClass(IWorkflowStack, SimpleStack)
         verifyClass(IWorkflowStack, HierarchicalStack)
