@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: test_workflow_with_stacks.py 18350 2005-04-18 16:57:22Z sfermigier $
+# $Id: test_workflow_with_stacks.py 31004 2005-12-27 19:58:13Z fguillaume $
 """Tests for the CPS Workflow Tool and CPS Worflow with stacks
 
 This test case implements a full delegation / validation workflow and covers
@@ -43,7 +43,6 @@ ZopeTestCase.installProduct('CPSCore')
 ZopeTestCase.installProduct('CPSDefault')
 ZopeTestCase.installProduct('CPSUserFolder')
 
-import Zope
 import unittest
 
 from Acquisition import aq_parent, aq_inner
@@ -65,11 +64,7 @@ from Products.CPSWorkflow.configuration import addConfiguration
 
 from Products.CPSWorkflow.workflowtool import Config_id
 
-from Products.CPSWorkflow.transitions import *
-from Products.CPSWorkflow.states import *
-
-from Products.CPSWorkflow.transitions import \
-     transition_behavior_export_dict as tbdict
+from Products.CPSWorkflow.constants import *
 
 from Products.CPSWorkflow.basicstackdefinitions import SimpleStackDefinition
 from Products.CPSWorkflow.basicstackdefinitions import \
@@ -540,8 +535,7 @@ class WorkflowToolTests(ZopeTestCase.PortalTestCase):
         t.setProperties('title',
                         'create',
                         trigger_type=TRIGGER_USER_ACTION,
-                        transition_behavior=(tbdict[TRANSITION_INITIAL_CREATE],
-                                             ))
+                        transition_behavior=(TRANSITION_INITIAL_CREATE,))
 
         # Allow sub-content creation
         wf2.transitions.addTransition('create_content')
@@ -549,8 +543,7 @@ class WorkflowToolTests(ZopeTestCase.PortalTestCase):
         t.setProperties('title',
                         'create',
                         trigger_type=TRIGGER_USER_ACTION,
-                        transition_behavior=(tbdict[TRANSITION_ALLOWSUB_CREATE],
-                                             ))
+                        transition_behavior=(TRANSITION_ALLOWSUB_CREATE,))
 
     ###################################################################
     # TESTS STARTS HERE
