@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-15 -*-
-# (C) Copyright 2002-2004 Nuxeo SARL <http://nuxeo.com>
+# (C) Copyright 2002-2006 Nuxeo SAS <http://nuxeo.com>
 # Author: Florent Guillaume <fg@nuxeo.com>
 # Contributor : Julien Anguenot <ja@nuxeo.com>
 #               - Refactoring CPS Workflow
@@ -21,71 +21,43 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: workflow.py 18353 2005-04-29 15:50:07Z janguenot $
+# $Id: workflow.py 33814 2006-03-03 13:09:22Z sfermigier $
 
-"""Workflow extending DCWorklfow with enhanced transitions.
-It adds, as well, a stack workflow support to be able to define dynamic
-delegation / validation chains through dedicated transitions.
+"""CPS Workflow
+
+Workflow extending DCWorkflow with enhanced transitions. It adds, as well, a
+stack workflow support to be able to define dynamic delegation / validation
+chains through dedicated transitions.
 """
 
-from zLOG import LOG, ERROR, DEBUG, INFO, TRACE
-from cgi import escape
-from types import StringType
+from zLOG import LOG, TRACE
 
-from Acquisition import aq_parent, aq_inner
-from Globals import InitializeClass, DTMLFile
-from AccessControl import ClassSecurityInfo, getSecurityManager
+from AccessControl import ClassSecurityInfo
+from AccessControl import getSecurityManager
+from Acquisition import aq_parent
+from Acquisition import aq_inner
+from Globals import InitializeClass
+from Globals import DTMLFile
 
-from Products.CMFCore.utils import getToolByName
 from Products.CMFCore.permissions import ManagePortal
-from Products.CMFCore.WorkflowCore import ObjectMoved, ObjectDeleted
+from Products.CMFCore.utils import getToolByName
+from Products.CMFCore.WorkflowCore import ObjectMoved
+from Products.CMFCore.WorkflowCore import ObjectDeleted
 from Products.CMFCore.WorkflowCore import WorkflowException
-from Products.CMFCore.WorkflowTool import addWorkflowFactory
 
 from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
-from Products.DCWorkflow.Transitions \
-    import TransitionDefinition as DCWFTransitionDefinition
-from Products.DCWorkflow.Transitions import Transitions as DCWFTransitions
-from Products.DCWorkflow.Transitions import TRIGGER_AUTOMATIC
 from Products.DCWorkflow.Transitions import TRIGGER_USER_ACTION
-from Products.DCWorkflow.Transitions import TRIGGER_WORKFLOW_METHOD
-
-from transitions import TRANSITION_INITIAL_CREATE
 
 from expression import CPSStateChangeInfo as StateChangeInfo
 from expression import createExprContext
 
 from stackregistries import WorkflowStackRegistry as StackReg
 
-#
-# CPSCore is optional now.
-# Check DEPENDENCIES.txt
-#
+from zope.interface import implements
+from Products.CPSWorkflow.interfaces import ICPSWorkflowDefinition
 
-try:
-    from Products.CPSCore.ProxyBase import ProxyBase
-    from Products.CPSCore.EventServiceTool import getEventService
-except ImportError, e:
-    if str(e) not in ('No module named CPSCore.EventServiceTool'
-                      'No module named CPSCore.ProxyBase'):
-        raise
-
-    LOG("Optional Dependencies missing", INFO,
-        "CPSCore.EventServiceTool and CPSCore.ProxyBase")
-
-    #
-    # Here defines optional dependencies on CPSCore elt.
-    #
-
-    class FakeEventService:
-        def notify(self, *args, **kw):
-            pass
-
-    def getEventService(context):
-        return FakeEventService
-
-    class ProxyBase:
-        pass
+from Products.CPSCore.ProxyBase import ProxyBase
+from Products.CPSCore.EventServiceTool import getEventService
 
 #
 # Backwards compatibility with CPS3 <= 3.2.x
@@ -103,6 +75,8 @@ class WorkflowDefinition(DCWorkflowDefinition):
     - Knowledge of proxies.
     """
 
+    implements(ICPSWorkflowDefinition)
+
     meta_type = 'CPS Workflow'
     title = 'CPS Workflow Definition'
     state_var = 'review_state'
@@ -293,7 +267,7 @@ class WorkflowDefinition(DCWorkflowDefinition):
 
         return (changed and stackdefs_changed)
 
-    def _checkStackGuards(self, t, ob):
+    def _checkStackGuards(self, t, ob, **kw):
         """Check the stack workflow transition guards on the transition for ob
 
         If one of them allowed the user to manage the stack it means the user
@@ -317,7 +291,7 @@ class WorkflowDefinition(DCWorkflowDefinition):
             var_ids = t.getStackWorkflowVariablesForBehavior(behavior)
             for var_id in var_ids:
 
-                if wftool.canManageStack(ob, var_id):
+                if wftool.canManageStack(ob, var_id, **kw):
 
 
                     #
@@ -349,7 +323,7 @@ class WorkflowDefinition(DCWorkflowDefinition):
         guard = t.guard
         return (guard is None or
                 guard.check(getSecurityManager(), self, ob, **kw) and
-                self._checkStackGuards(t, ob))
+                self._checkStackGuards(t, ob, **kw))
 
     def _changeStateOf(self, ob, tdef=None, kwargs=None):
         """
@@ -421,7 +395,9 @@ class WorkflowDefinition(DCWorkflowDefinition):
         new_sdef = self.states.get(new_state, None)
         if new_sdef is None:
             raise WorkflowException, (
-                'Destination state undefined: ' + new_state)
+                "Destination state '%s' undefined for "
+                "transition '%s' in workflow '%s'" %(
+                new_state, tdef.getId(), self.getId()))
 
 
         ### CPS: Behavior sanity checks.
@@ -1017,6 +993,25 @@ class WorkflowDefinition(DCWorkflowDefinition):
         tool = aq_parent(aq_inner(self))
         tool.setStatusOf(self.id, ob, status)
 
+        # Update role to permission assignments.
+        kw = {}
+        kw['current_wf_var_id'] = kwargs.get('current_wf_var_id', '')
+        kw['tdef'] = tdef
+        self.updateRoleMappingsFor(ob, **kw)
+
+        # Execute the "after" script.
+        if tdef is not None and tdef.after_script_name:
+            script = self.scripts[tdef.after_script_name]
+            # Taking care of the case of an "after" script for a deleted object.
+            # The condition delete_ob == ob is here to make sure we don't reset
+            # ob in case of a TRANSITION_BEHAVIOR_MERGE.
+            if delete_ob is not None and delete_ob == ob:
+                ob = None
+            # Pass lots of info to the script in a single parameter.
+            sci = StateChangeInfo(
+                ob, self, status, tdef, old_sdef, new_sdef, stacks, kwargs)
+            script(sci)  # May throw an exception.
+
         ### CPS: Delete. Done after setting status, to keep history.
         #
         if delete_ob is not None:
@@ -1030,20 +1025,6 @@ class WorkflowDefinition(DCWorkflowDefinition):
             else:
                 raise ObjectDeleted
 
-        # Update role to permission assignments.
-        kw = {}
-        kw['current_wf_var_id'] = kwargs.get('current_wf_var_id', '')
-        kw['tdef'] = tdef
-        self.updateRoleMappingsFor(ob, **kw)
-
-        # Execute the "after" script.
-        if tdef is not None and tdef.after_script_name:
-            script = self.scripts[tdef.after_script_name]
-            # Pass lots of info to the script in a single parameter.
-            sci = StateChangeInfo(
-                ob, self, status, tdef, old_sdef, new_sdef, stacks, kwargs)
-            script(sci)  # May throw an exception.
-
         ### CPS: Event notification. This has to be done after all the
         # potential transition scripts.
         #
@@ -1102,8 +1083,9 @@ class WorkflowDefinition(DCWorkflowDefinition):
         """
         tdef = self.transitions.get(initial_transition, None)
         if tdef is None:
-            raise WorkflowException("No initial transition '%s'" %
-                                    initial_transition)
+            raise WorkflowException("No initial transition '%s' "
+                                    "in workflow '%s'" %
+                                    (initial_transition, self.getId()))
         # Check it's really an initial transition.
         if initial_behavior not in tdef.transition_behavior:
             raise WorkflowException("workflow=%s transition=%s"
@@ -1210,9 +1192,9 @@ class WorkflowDefinition(DCWorkflowDefinition):
         return self.permissions
 
     def _objectMaybeFromRpath(self, ob):
-        if isinstance(ob, StringType):
+        if isinstance(ob, str):
             rpath = ob
-            if not rpath or rpath.find('..') >= 0 or rpath.startswith('/'):
+            if not rpath or rpath.find('../') >= 0 or rpath.startswith('/'):
                 raise WorkflowException("Unauthorized rpath %s" % rpath)
             portal = getToolByName(self, 'portal_url').getPortalObject()
             ob = portal.unrestrictedTraverse(rpath) # XXX unrestricted ?
@@ -1254,110 +1236,6 @@ class WorkflowDefinition(DCWorkflowDefinition):
         res.sort()
         return map((lambda (id, val): val), res)
 
-    #
-    # XML Serialization
-    #
-
-    def toXML(self, indent=''):
-        """Serialize the workflow to XML.
-
-        Returns an XML string, without header.
-        """
-        nindent = indent + '  '
-        contents = '\n' + self._transitionsToXML(indent=nindent) + '\n' + indent
-        return _renderXMLTag('workflow', id=self.getId(),
-                             title=self.title, contents=contents)
-
-    def _varExprsToXML(self, var_exprs, indent=''):
-        nindent = indent + '  '
-        if var_exprs is None:
-            return indent + '<variables></variables>'
-        contents = 'XXXTODO'
-        return indent + _renderXMLTag('variables', contents=contents)
-
-    def _permissionsToXML(self, permissions, indent=''):
-        nindent = indent + '  '
-        res = ['']
-        for permission in permissions:
-            res.append(nindent + _renderXMLTag('permission',
-                                               contents=permission))
-        res.append(indent)
-        contents = '\n'.join(res)
-        return indent + _renderXMLTag('permissions', contents=contents)
-
-    def _rolesToXML(self, roles, indent=''):
-        nindent = indent+'  '
-        res = ['']
-        for role in roles:
-            res.append(nindent + _renderXMLTag('role', contents=role))
-        res.append(indent)
-        contents = '\n'.join(res)
-        return indent + _renderXMLTag('roles', contents=contents)
-
-    def _guardToXML(self, guard, indent=''):
-        if guard is None:
-            return indent + '<guard></guard>'
-        nindent = indent + '  '
-        res = ['']
-        res.append(self._permissionsToXML(guard.permissions, indent=nindent))
-        res.append(self._rolesToXML(guard.roles, indent=nindent))
-        res.append(indent)
-        contents = '\n'.join(res)
-        return indent + _renderXMLTag('guard', contents=contents)
-
-    def _behaviorsToXML(self, tdef, indent=''):
-        nindent = indent+'  '
-        res = ['']
-        for behavior in tdef.transition_behavior:
-            type = transition_behavior_export_dict.get(behavior, behavior)
-            kwargs = {'type': type}
-            if behavior == TRANSITION_BEHAVIOR_PUBLISHING:
-                kwargs['transitions'] = ' '.join(
-                    tdef.clone_allowed_transitions)
-            elif behavior == TRANSITION_BEHAVIOR_CHECKOUT:
-                kwargs['transitions'] = ' '.join(
-                    tdef.checkout_allowed_initial_transitions)
-            elif behavior == TRANSITION_BEHAVIOR_CHECKIN:
-                kwargs['transitions'] = ' '.join(
-                    tdef.checkin_allowed_transitions)
-            res.append(nindent + _renderXMLTag('behavior', **kwargs))
-        res.append(indent)
-        behaviors = '\n'.join(res)
-        return indent + _renderXMLTag('behaviors', contents=behaviors)
-
-    def _transitionToXML(self, tid, indent=''):
-        nindent = indent+'  '
-        LOG('_transitionToXML', DEBUG, 'tid=%s' % `tid`)
-        tdef = self.transitions.get(tid)
-        trigger = trigger_export_dict.get(tdef.trigger_type,
-                                          tdef.trigger_type)
-        behaviors = self._behaviorsToXML(tdef, indent=nindent)
-        guard = self._guardToXML(tdef.guard, indent=nindent)
-        var_exprs = self._varExprsToXML(tdef.var_exprs, indent=nindent)
-        contents = '\n'+behaviors+'\n'+guard+'\n'+var_exprs+'\n'+indent
-        return indent + _renderXMLTag(
-            'transition',
-            id=tid,
-            title=tdef.title,
-            description=tdef.description,
-            destination=tdef.new_state_id,
-            trigger=trigger,
-            actionName=tdef.actbox_name,
-            actionUrl=tdef.actbox_url,
-            actionCategory=tdef.actbox_category,
-            beforeScript=tdef.script_name,
-            afterScript=tdef.after_script_name,
-            contents=contents)
-
-    def _transitionsToXML(self, indent=''):
-        nindent = indent+'  '
-        res = ['']
-        for tid in self.transitions.keys():
-            res.append(self._transitionToXML(tid, indent=nindent))
-        res.append(indent)
-        contents = '\n'.join(res)
-        return indent + _renderXMLTag('transitions', contents=contents)
-
     #
     # ZMI
     #
@@ -1372,31 +1250,6 @@ class WorkflowDefinition(DCWorkflowDefinition):
 
 InitializeClass(WorkflowDefinition)
 
+from Products.CMFCore.WorkflowTool import addWorkflowFactory
 addWorkflowFactory(WorkflowDefinition, id='cps_workflow',
                    title='Web-configurable workflow for CPS')
-
-
-def _renderXMLTag(tagname, **kw):
-    """Render an XML tag."""
-    if kw.get('css_class'):
-        kw['class'] = kw['css_class']
-        del kw['css_class']
-    if kw.has_key('contents'):
-        contents = kw['contents']
-        del kw['contents']
-    else:
-        contents = None
-    attrs = []
-    for key, value in kw.items():
-        if value is None:
-            value = key
-        if value != '':
-            attrs.append('%s="%s"' % (key, escape(str(value))))
-    if attrs:
-        res = '<%s %s>' % (tagname, ' '.join(attrs))
-    else:
-        res = '<%s>' % tagname
-    if contents is not None:
-        res += contents
-    res += '</%s>' % tagname
-    return res
