@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-15 -*-
-# (C) Copyright 2002, 2003, 2004 Nuxeo SARL <http://nuxeo.com>
+# (C) Copyright 2002-2005 Nuxeo SARL <http://nuxeo.com>
 # Author: Florent Guillaume <fg@nuxeo.com>
 # Contributor: Julien Anguenot <ja@nuxeo.com>
 #              - Stack workflow API
@@ -18,18 +18,21 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: workflowtool.py 21975 2005-05-31 13:46:24Z fguillaume $
+# $Id: workflowtool.py 31552 2006-01-12 17:40:21Z janguenot $
 """Workflow Tool with CPS proxy knowledge, CPS placeful workflow support and
 stack workflows support.
 """
 
 from zLOG import LOG, ERROR, DEBUG, TRACE, INFO
 
+import sys
 from types import StringType
 from Acquisition import aq_base, aq_parent, aq_inner
 from Globals import InitializeClass, DTMLFile
 from AccessControl import ClassSecurityInfo, Unauthorized
 from OFS.Folder import Folder
+from OFS.ObjectManager import IFAwareObjectManager
+from webdav.WriteLockInterface import WriteLockInterface
 
 from Products.CMFCore.utils import _checkPermission, getToolByName
 from Products.CMFCore.permissions import View, ModifyPortalContent
@@ -55,6 +58,10 @@ from transitions import TRANSITION_INITIAL_CHECKOUT
 from transitions import TRANSITION_ALLOW_CHECKIN
 from transitions import TRANSITION_BEHAVIOR_PUBLISHING
 
+from zope.interface import implements, implementedBy
+from Products.CPSWorkflow.interfaces import ICPSWorkflowTool
+from Products.CMFCore.interfaces import IWorkflowDefinition
+
 #
 # CPSCore is optional now.
 # Check DEPENDENCIES.txt
@@ -64,11 +71,8 @@ try:
     from Products.CPSCore.ProxyBase import ProxyBase, ProxyFolderishDocument
     from Products.CPSCore.ProxyBase import ProxyBTreeFolderishDocument
     from Products.CPSCore.EventServiceTool import getEventService
-except ImportError, e:
-    if str(e) not in ('No module named CPSCore.EventServiceTool'
-                      'No module named CPSCore.ProxyBase'):
-        raise
-
+except ImportError:
+    if sys.exc_info()[2].tb_next is not None: raise
     LOG("Optional Dependencies missing", INFO,
         "CPSCore.EventServiceTool and CPSCore.ProxyBase")
 
@@ -92,9 +96,11 @@ except ImportError, e:
     class ProxyBTreeFolderishDocument:
         pass
 
-# id of the cps workflow configuration objects
-Config_id = '.cps_workflow_configuration'
 
+# id of the cps workflow configuration objects
+LOCAL_WORKFLOW_CONFIG_ID = '.cps_workflow_configuration'
+# BBB: old name
+Config_id = LOCAL_WORKFLOW_CONFIG_ID
 
 class WorkflowTool(BaseWorkflowTool):
     """A Workflow Tool extending the CMFCore one with CPS features.
@@ -104,12 +110,15 @@ class WorkflowTool(BaseWorkflowTool):
     - Delegates storage of workflow history for proxies to repository tool
     """
 
+    implements(ICPSWorkflowTool)
+    _product_interfaces = (IWorkflowDefinition,)
+
     id = 'portal_workflow'
     meta_type = 'CPS Workflow Tool'
-    title = 'CPS Workflow Tool'
+    title = ''
 
-    manage_options = ( Folder.manage_options[0], 
-                       { 'label' : 'Workflows', 
+    manage_options = ( Folder.manage_options[0],
+                       { 'label' : 'Workflows',
                          'action' : 'manage_selectWorkflows'
                        },
                        { 'label' : 'Overview', 'action' : 'manage_overview' }
@@ -251,8 +260,8 @@ class WorkflowTool(BaseWorkflowTool):
     def getDefaultLanguage(self):
         """Get the default language for a new object."""
         portal = aq_parent(aq_inner(self))
-        if hasattr(portal, 'Localizer'):
-            return portal.Localizer.get_default_language()
+        if hasattr(portal, 'translation_service'):
+            return portal.translation_service.getDefaultLanguage()
         else:
             return 'en'
 
@@ -274,7 +283,8 @@ class WorkflowTool(BaseWorkflowTool):
         LOG('invokeFactoryFor', DEBUG,
             "Called with container=%s type_name=%s id=%s "
             "language=%s initial_transition=%s" %
-            (container.getId(), type_name, id, language, initial_transition))
+            ('/'.join(container.getPhysicalPath()), type_name, id,
+             language, initial_transition))
         if language is None:
             language = self.getDefaultLanguage()
         if initial_transition is None:
@@ -399,13 +409,20 @@ class WorkflowTool(BaseWorkflowTool):
             break
 
         if initial_behavior == TRANSITION_INITIAL_PUBLISHING:
-            ob = container.copyContent(old_ob, id) # XXX should not notify cmfadd
-            ob.manage_afterCMFAdd(ob, container) # XXX later! the object is not finished yet !
+            # XXX should not notify cmfadd
+            ob = container.copyContent(old_ob, id)
+            # Removing any possible WebDAV locks on the checked-out object
+            # because the copyContent method also copy locks (which might or
+            # might not be considered a bug).
+            if WriteLockInterface.isImplementedBy(ob):
+                ob.wl_clearLocks()
+            # XXX later! the object is not finished yet!
+            ob.manage_afterCMFAdd(ob, container)
             self._insertWorkflowRecursive(ob, initial_transition,
                                           initial_behavior, kwargs)
         elif initial_behavior == TRANSITION_INITIAL_CREATE:
             if not proxy_type:
-                # XXX constructContent doesn't exist everywhere !
+                # XXX constructContent doesn't exist everywhere!
                 # XXX especially when creating at the root of the portal.
                 id = container.constructContent(type_name, id, **kwargs)
                 # constructContent indexed the object (CMF contract)
@@ -537,24 +554,61 @@ class WorkflowTool(BaseWorkflowTool):
 
         (Called by CPSWorkflow during merge transition.)
         """
+        pxtool = getToolByName(self, 'portal_proxies')
         dest_ob = self._checkObjectMergeable(ob, dest_container,
                                              state_var, new_state)[0]
         if dest_ob is not None:
-            pxtool = getToolByName(self, 'portal_proxies')
             pxtool.checkinRevisions(ob, dest_ob)
 
         # For folderish documents, copy subobjects into new container.
-        if isinstance(dest_ob, ProxyFolderishDocument) or \
-               isinstance(dest_ob, ProxyBTreeFolderishDocument):
-            # Erase old
-            ids = [id for id in dest_ob.objectIds() if not id.startswith('.')]
-            dest_ob.manage_delObjects(ids)
+        if (isinstance(dest_ob, ProxyFolderishDocument) or
+            isinstance(dest_ob, ProxyBTreeFolderishDocument)):
+            for id_ in ob.objectIds():
+                if id_.startswith('.'):
+                    continue
+                # Merge objects
+                if id_ in dest_ob.objectIds():
+                    subob = ob._getOb(id_)
+                    dest_subob = self._checkObjectMergeable(
+                        subob, dest_ob, state_var, new_state)[0]
+                    if subob is not None:
+                        # Recursive mergeObject()
+                        self.mergeObject(subob, dest_ob, state_var,
+                                         new_state)
+                # Insert into the container with workflow
+                else:
+                    subob = ob._getOb(id_)
+                    dest_ob.copyContent(subob, id_)
+                    subob = dest_ob._getOb(id_)
 
-            # Copy new
-            ids = [id for id in ob.objectIds() if not id.startswith('.')]
-            for id in ids:
-                subob = ob._getOb(id)
-                dest_ob.copyContent(subob, id)
+                    type_name = subob.getTypeInfo().getId()
+                    inserted = False
+                    # XXX enough ??
+                    for behavior in (TRANSITION_INITIAL_CREATE,
+                                     TRANSITION_INITIAL_PUBLISHING):
+                        if inserted:
+                            break
+                        crtrans = self.getInitialTransitions(subob, type_name,
+                                                             behavior)
+                        for initial_transition in crtrans:
+                            initial_behavior = behavior
+                            try:
+                                self._insertWorkflowRecursive(
+                                    subob, initial_transition,
+                                    initial_behavior, {})
+                                inserted = True
+                                break
+                            except (WorkflowException, Unauthorized):
+                                # Let's try other transitions
+                                pass
+                    if not inserted:
+                        raise WorkflowException(
+                            "You are not allowed to copy / paste here !")
+
+            # Now erase the ones that are not part of the new revision anymore
+            ids = [id_ for id_ in dest_ob.objectIds()
+                   if not id_.startswith('.') and id_ not in ob.objectIds()]
+            dest_ob.manage_delObjects(ids)
 
         return dest_ob
 
@@ -663,15 +717,13 @@ class WorkflowTool(BaseWorkflowTool):
         if wf_id is None:
             if not wfs:
                 raise WorkflowException('No workflows found.')
-            found = 0
             for wf in wfs:
                 LOG('_doActionFor', TRACE, ' testing wf %s' % wf.getId())
                 if wf.isActionSupported(ob, action, **kw):
                     LOG('_doActionFor', TRACE, ' found!')
-                    found = 1
                     break
                 LOG('_doActionFor', TRACE, ' not found')
-            if not found:
+            else:
                 raise WorkflowException(
                     'No workflow provides the "%s" action.' % action)
         else:
@@ -760,6 +812,8 @@ class WorkflowTool(BaseWorkflowTool):
         if provided.
         """
 ##         import traceback
+##         from zExceptions.ExceptionFormatter import format_exception
+##         traceback.format_exception = format_exception
 ##         from StringIO import StringIO
 ##         s = StringIO()
 ##         traceback.print_stack(file=s)
@@ -780,10 +834,10 @@ class WorkflowTool(BaseWorkflowTool):
                 'getChainFor: no container for ob %s' % (ob,))
             return ()
         # Find placeful workflow configuration object.
-        wfconf = getattr(container, Config_id, None)
+        wfconf = getattr(container, LOCAL_WORKFLOW_CONFIG_ID, None)
         if wfconf is not None:
             # Was it here or did we acquire?
-            start_here = hasattr(aq_base(container), Config_id)
+            start_here = hasattr(aq_base(container), LOCAL_WORKFLOW_CONFIG_ID)
             chain = wfconf.getPlacefulChainFor(pt, start_here=start_here)
             if chain is not None:
                 return chain
@@ -877,7 +931,7 @@ class WorkflowTool(BaseWorkflowTool):
         return self.getStacks(ob).get(stack_id)
 
     security.declarePublic('canManageStack')
-    def canManageStack(self, ob, stack_id):
+    def canManageStack(self, ob, stack_id, **kw):
         """Can the authenticated use or the user given it's member_id
         manage the stack given its id.
 
@@ -890,7 +944,7 @@ class WorkflowTool(BaseWorkflowTool):
             mtool = getToolByName(self, 'portal_membership')
             aclu = self.acl_users
             ds = self.getStackFor(ob, stack_id)
-            canManage = stackdef._canManageStack(ds, aclu, mtool, ob)
+            canManage = stackdef._canManageStack(ds, aclu, mtool, ob, **kw)
             if canManage:
                 return 1
             else:
@@ -904,7 +958,7 @@ class WorkflowTool(BaseWorkflowTool):
                 for stackdef_id in stackdefs.keys():
                     ostack_def = stackdefs[stackdef_id]
                     canManage = ostack_def._canManageStack(None, aclu, mtool,
-                                                           ob)
+                                                           ob, **kw)
                     manager_stack_ids = ostack_def.getManagerStackIds()
                     if (canManage and
                         stack_id in manager_stack_ids):
@@ -974,14 +1028,14 @@ class WorkflowTool(BaseWorkflowTool):
 
     manage_overview = DTMLFile('zmi/explainCPSWorkflowTool', globals())
 
-    def all_meta_types(self):
-        return ({'name': 'CPS Workflow',
-                 'action': 'manage_addWorkflowForm',
-                 'permission': ManagePortal},
-                {'name': 'Workflow',
-                 'action': 'manage_addWorkflowForm',
-                 'permission': ManagePortal},
-                )
+    #def all_meta_types(self):
+    #    return ({'name': 'CPS Workflow',
+    #             'action': 'manage_addWorkflowForm',
+    #             'permission': ManagePortal},
+    #            {'name': 'Workflow',
+    #             'action': 'manage_addWorkflowForm',
+    #             'permission': ManagePortal},
+    #            )
 
 
 InitializeClass(WorkflowTool)
